<?php
/**
 * 2013 - 2021 PayPlug SAS
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0).
 * It is available through the world-wide-web at this URL:
 * https://opensource.org/licenses/osl-3.0.php
 * If you are unable to obtain it through the world-wide-web, please send an email
 * to contact@payplug.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PayPlug module to newer
 * versions in the future.
 *
 * @author    PayPlug SAS
 * @copyright 2013 - 2021 PayPlug SAS
 * @license   https://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 *  International Registered Trademark & Property of PayPlug SAS
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

require_once(_PS_MODULE_DIR_ . 'payplug/classes/PayplugBackward.php');
require_once(_PS_MODULE_DIR_ . 'payplug/classes/MyLogPHP.class.php');
require_once(_PS_MODULE_DIR_ . 'payplug/lib/init.php');

class Payplug extends PaymentModule
{
    /** @var string */
    private $html = '';

    /** @var array */
    public $validationErrors = array();

    /** @var PayplugAdmin */
    public $payplugAdmin;

    /** @var int */
    const PAYMENT_STATUS_PAID = 0;

    /** @var int */
    const PAYMENT_STATUS_REFUND = 4;

    /** @var int */
    const INST_MIN_AMOUNT = 4;

    /** @var array */
    public $routes = array(
        'login' => '/v1/keys',
        'account' => '/v1/account',
        'patch' => '/v1/payments/',
        'oney_sim' => '/v1/oney_payment_simulations'
    );

    /** @var string */
    public $api_url = '';

    /** @var array */
    public $check_configuration = array();

    /** @var string */
    public $email;

    /** @var string */
    public $api_live;

    /** @var string */
    public $api_test;

    /** @var PayPlugLogger */
    public $logger;

    /** @var bool */
    public $ssl_enable;
    public $log_general;
    public $log_install;
    public $payplug_url = '';
    public $confirmUninstall = '';
    public $current_api_key = '';
    public $api_version = '2019-08-06';

    public $payment_status = array();

    public $available_oney_payments = array(
        'x3_with_fees',
        'x4_with_fees',
    );

    /** @var array */
    private $entities = array(
        'PayPlugPayment/PayPlugPayment',
        'PayPlugPayment/PayPlugPaymentStandard',
        'PayPlugPayment/PayPlugPaymentOneClick',
        'PayPlugPayment/PayPlugPaymentInstallment',
        'PayPlugPayment/PayPlugPaymentOney',
        'PayPlugCache',
        'PayPlugCard',
        'PayPlugNotifications',
        'PayplugLock',
        'PayPlugValidation',
        'PayPlugAjax',
        'PPPayment',
        'PPPaymentInstallment',
        'PayPlugLogger',
    );

    private $conf = array(
        'PAYPLUG_ADMIN_INSTALLMENT_TAB' => array('value' => '', 'type' => 'int'),
        'PAYPLUG_ALLOW_SAVE_CARD' => array('value' => '', 'type' => ''),
        'PAYPLUG_COMPANY_ID' => array('value' => '', 'type' => 'int'),
        'PAYPLUG_COMPANY_STATUS' => array('value' => '', 'type' => 'int'),
        'PAYPLUG_CURRENCIES' => array('value' => '', 'type' => 'string'),
        'PAYPLUG_DEBUG_MODE' => array('value' => '', 'type' => 'bool'),
        'PAYPLUG_EMAIL' => array('value' => '', 'type' => 'string'),
        'PAYPLUG_EMBEDDED_MODE' => array('value' => '', 'type' => 'bool'),
        'PAYPLUG_INST' => array('value' => '', 'type' => 'bool'),
        'PAYPLUG_INST_MIN_AMOUNT' => array('value' => '', 'type' => 'float'),
        'PAYPLUG_INST_MODE' => array('value' => '', 'type' => 'int'),
        'PAYPLUG_KEEP_CARDS' => array('value' => '', 'type' => ''),
        'PAYPLUG_LIVE_API_KEY' => array('value' => '', 'type' => 'string'),
        'PAYPLUG_MAX_AMOUNTS' => array('value' => '', 'type' => 'string'),
        'PAYPLUG_MIN_AMOUNTS' => array('value' => '', 'type' => 'string'),
        'PAYPLUG_ONE_CLICK' => array('value' => '', 'type' => 'bool'),
        'PAYPLUG_ORDER_STATE_AUTH' => array('value' => '', 'type' => 'int'),
        'PAYPLUG_ORDER_STATE_AUTH_TEST' => array('value' => '', 'type' => 'int'),
        'PAYPLUG_ORDER_STATE_EXP' => array('value' => '', 'type' => 'int'),
        'PAYPLUG_ORDER_STATE_EXP_TEST' => array('value' => '', 'type' => 'int'),
        'PAYPLUG_ORDER_STATE_ERROR' => array('value' => '', 'type' => 'int'),
        'PAYPLUG_ORDER_STATE_ERROR_TEST' => array('value' => '', 'type' => 'int'),
        'PAYPLUG_ORDER_STATE_INST_PG' => array('value' => '', 'type' => 'int'),
        'PAYPLUG_ORDER_STATE_INST_PG_TEST' => array('value' => '', 'type' => 'int'),
        'PAYPLUG_ORDER_STATE_PAID' => array('value' => '', 'type' => 'int'),
        'PAYPLUG_ORDER_STATE_PAID_TEST' => array('value' => '', 'type' => 'int'),
        'PAYPLUG_ORDER_STATE_PENDING' => array('value' => '', 'type' => 'int'),
        'PAYPLUG_ORDER_STATE_PENDING_TEST' => array('value' => '', 'type' => 'int'),
        'PAYPLUG_ORDER_STATE_REFUND' => array('value' => '', 'type' => 'int'),
        'PAYPLUG_ORDER_STATE_REFUND_TEST' => array('value' => '', 'type' => 'int'),
        'PAYPLUG_SANDBOX_MODE' => array('value' => '', 'type' => 'bool'),
        'PAYPLUG_SHOW' => array('value' => '', 'type' => 'bool'),
        'PAYPLUG_TEST_API_KEY' => array('value' => '', 'type' => 'string'),
        'PS_SSL_ENABLED' => array('value' => '', 'type' => 'bool'),
        'PS_LANG_DEFAULT' => array('value' => '', 'type' => 'int'),
        'PS_TAX' => array('value' => '', 'type' => 'bool'),
    );

    private $oney_conf = array(
        'PAYPLUG_ONEY' => array('value' => '', 'type' => 'bool'),
        'PAYPLUG_ONEY_ALLOWED_COUNTRIES' => array('value' => '', 'type' => 'string'),
        'PAYPLUG_ONEY_MAX_AMOUNTS' => array('value' => '', 'type' => 'string'),
        'PAYPLUG_ONEY_MIN_AMOUNTS' => array('value' => '', 'type' => 'string'),
        'PAYPLUG_ONEY_OPTIMIZED' => array('value' => '', 'type' => 'bool'),
        'PAYPLUG_ONEY_TOS' => array('value' => '', 'type' => 'bool'),
        'PAYPLUG_ONEY_TOS_URL' => array('value' => '', 'type' => 'string'),
        'PAYPLUG_ORDER_STATE_ONEY_PG' => array('value' => '', 'type' => 'int'),
        'PAYPLUG_ORDER_STATE_ONEY_PG_TEST' => array('value' => '', 'type' => 'int'),
    );

    private $order_states = array(
        'paid' => array(
            'cfg' => '_PS_OS_PAYMENT_',
            'template' => 'payment',
            'logable' => true,
            'send_email' => true,
            'paid' => true,
            'module_name' => 'payplug',
            'hidden' => false,
            'delivery' => false,
            'invoice' => true,
            'color' => '#04b404',
            'name' => array(
                'en' => 'Payment successful',
                'fr' => 'Paiement effectué',
                'es' => 'Pago efectuado',
                'it' => 'Pagamento effettuato',
            ),
        ),
        'refund' => array(
            'cfg' => '_PS_OS_REFUND_',
            'template' => 'refund',
            'logable' => false,
            'send_email' => true,
            'paid' => false,
            'module_name' => 'payplug',
            'hidden' => false,
            'delivery' => false,
            'invoice' => true,
            'color' => '#ea3737',
            'name' => array(
                'en' => 'Refunded',
                'fr' => 'Remboursé',
                'es' => 'Reembolsado',
                'it' => 'Rimborsato',
            ),
        ),
        'pending' => array(
            'cfg' => '_PS_OS_PENDING_',
            'template' => null,
            'logable' => false,
            'send_email' => false,
            'paid' => false,
            'module_name' => 'payplug',
            'hidden' => false,
            'delivery' => false,
            'invoice' => false,
            'color' => '#a1f8a1',
            'name' => array(
                'en' => 'Payment in progress',
                'fr' => 'Paiement en cours',
                'es' => 'Pago en curso',
                'it' => 'Pagamento in corso',
            ),
        ),
        'error' => array(
            'cfg' => '_PS_OS_ERROR_',
            'template' => 'payment_error',
            'logable' => false,
            'send_email' => true,
            'paid' => false,
            'module_name' => 'payplug',
            'hidden' => false,
            'delivery' => false,
            'invoice' => false,
            'color' => '#8f0621',
            'name' => array(
                'en' => 'Payment failed',
                'fr' => 'Paiement échoué',
                'es' => 'Payment failed',
                'it' => 'Payment failed',
            ),
        ),
        'auth' => array(
            'cfg' => null,
            'template' => null,
            'logable' => false,
            'send_email' => false,
            'paid' => true,
            'module_name' => 'payplug',
            'hidden' => false,
            'delivery' => false,
            'invoice' => false,
            'color' => '#04b404',
            'name' => array(
                'en' => 'Payment authorised',
                'fr' => 'Paiement autorisé',
                'es' => 'Pago',
                'it' => 'Pagamento',
            ),
        ),
        'exp' => array(
            'cfg' => null,
            'template' => null,
            'logable' => false,
            'send_email' => false,
            'paid' => false,
            'module_name' => 'payplug',
            'hidden' => false,
            'delivery' => false,
            'invoice' => false,
            'color' => '#8f0621',
            'name' => array(
                'en' => 'Authorization expired',
                'fr' => 'Autorisation expirée',
                'es' => 'Autorización vencida',
                'it' => 'Autorizzazione scaduta',
            ),
        ),
    );


    public $oney_order_state = array(
        'oney_pg' => array(
            'cfg' => null,
            'template' => null,
            'logable' => false,
            'send_email' => false,
            'paid' => false,
            'module_name' => 'payplug',
            'hidden' => false,
            'delivery' => false,
            'invoice' => false,
            'color' => '#a1f8a1',
            'name' => array(
                'en' => 'Oney - Pending',
                'fr' => 'Oney - En attente',
                'es' => 'Oney - Pending',
                'it' => 'Oney - Pending',
            ),
        ),
    );

    /** @var PayPlugCache */
    private $payplug_cache;

    public function loadConfigurations()
    {
        if (empty($this->conf)) {
            return false;
        }

        $keys = array_keys($this->conf);
        $values = PayplugBackward::getMultipleConfiguration($keys);

        foreach ($values as $key => $value) {
            $type = $this->conf[$key]['type'];
            switch ($type) {
                case 'bool':
                    $this->conf[$key]['value'] = (bool)$value ? 1 : 0;
                    break;
                case 'int':
                    $this->conf[$key]['value'] = (int)$value;
                    break;
                case 'float':
                    $this->conf[$key]['value'] = (float)$value;
                    break;
                case 'string':
                default:
                    $this->conf[$key]['value'] = (string)$value;
                    break;
            }
        }

        if (!empty($this->oney_conf) && $this->checkVersion()) {
            $keys = array_keys($this->oney_conf);
            $values = PayplugBackward::getMultipleConfiguration($keys);

            foreach ($values as $key => $value) {
                $type = $this->oney_conf[$key]['type'];
                switch ($type) {
                    case 'bool':
                        $this->conf[$key]['value'] = (bool)$value ? 1 : 0;
                        break;
                    case 'int':
                        $this->conf[$key]['value'] = (int)$value;
                        break;
                    case 'float':
                        $this->conf[$key]['value'] = (float)$value;
                        break;
                    case 'string':
                    default:
                        $this->conf[$key]['value'] = (string)$value;
                        break;
                }
            }
        }
    }

    public function getConfiguration($key)
    {
        if (isset($this->conf[$key])) {
            return $this->conf[$key]['value'];
        } else {
            return PayplugBackward::getConfiguration($key);
        }
    }

    public function setConfiguration($key, $value)
    {
        $this->conf[$key]['value'] = $value;
        return PayplugBackward::updateConfiguration($key, $value);
    }

    /**
     * Constructor
     *
     * @return Payplug
     */
    public function __construct()
    {
        $this->setPrimaryModuleProperties();
        $this->loadEntities();
        parent::__construct();
        $this->setEnvironment();
        $this->setConfigurationProperties();
        $this->setSecretKey();
        $this->setUserAgent();

        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            // Backward compatibility
            // required for $this->context who does'nt exists on 1.4
            require_once(_PS_MODULE_DIR_ . $this->name . '/backward_compatibility/backward.php');
            $cookie_admin = new Cookie(
                'psAdmin',
                Tools::substr($_SERVER['PHP_SELF'], Tools::strlen(__PS_BASE_URI__), -10)
            );
            if (Tools::getValue('tab') == 'AdminPayment'
                && Tools::getValue('token') != Tools::getAdminTokenLite('AdminPayment')
            ) {
                // Force admin status
                $this->context->cookie->profile = $cookie_admin->profile;
                $url = 'index.php?tab=AdminPayment';
                $url .= '&token=' . Tools::getAdminTokenLite('AdminPayment');
                Tools::redirectAdmin($url);
            }

            // Call everytime to prevent the change of the module by a recent one
            $this->database_version = $this->getConfiguration('PAYPLUG_VERSION_1_4');
            $this->_updateProcess();
        }

        $this->setLoggers();

        $this->initializeCache();
    }

    public function initializeApi($sandbox = null)
    {
        if ($sandbox === null) {
            $payplug_key = $this->current_api_key;
        } else {
            $payplug_key = $this->getConfiguration('PAYPLUG_' . ($sandbox ? 'TEST' : 'LIVE') . '_API_KEY');
        }

        try {
            \Payplug\Payplug::init(array('secretKey' => $payplug_key, 'apiVersion' => $this->api_version));

            return $payplug_key;
        } catch (Exception $e) {
            // todo: return error log
            return false;
        }
    }

    /**
     * Load PayPlug entities from props
     *
     * @return bool
     */
    public function loadEntities()
    {
        if (empty($this->entities)) {
            return false;
        }

        foreach ($this->entities as $entity) {
            $entity_path = _PS_MODULE_DIR_ . 'payplug/classes/' . $entity . '.php';
            if (file_exists($entity_path)) {
                include_once($entity_path);
            }
        }

        return true;
    }

    private function setPrimaryModuleProperties()
    {
        // Must be set before translations
        $this->name = 'payplug';

        $this->author = 'PayPlug';
        $this->bootstrap = true;
        $this->currencies = true;
        $this->currencies_mode = 'checkbox';
        $this->description = $this->l(
            'The online payment solution combining simplicity and first-rate support to boost your sales.'
        );
        $this->displayName = 'PayPlug';
        $this->module_key = '1ee28a8fb5e555e274bd8c2e1c45e31a';
        $this->need_instance = true;
        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => '1.7');
        $this->tab = 'payments_gateways';
        $this->version = '2.30.2';
    }

    private function setLoggers()
    {
        $this->log_general = new MyLogPHP(_PS_MODULE_DIR_ . $this->name . '/log/general-log.csv');
        $this->log_install = new MyLogPHP(_PS_MODULE_DIR_ . $this->name . '/log/install-log.csv');
        $this->log_payment = new MyLogPHP(_PS_MODULE_DIR_ . $this->name . '/log/prepare_payment.csv');
        $this->logger = new PayPlugLogger('payplug');
        if ($this->active) {
            $this->logger->flush();
        }
    }

    private function initializeCache()
    {
        $this->payplug_cache = new PayPlugCache();
    }

    /**
     * Determine witch environment is used
     *
     * @return void
     */
    public function setEnvironment()
    {
        if (isset($_SERVER['PAYPLUG_API_URL'])) {
            $this->api_url = $_SERVER['PAYPLUG_API_URL'];
        } else {
            $this->api_url = 'https://api.payplug.com';
        }

        if (isset($_SERVER['PAYPLUG_SITE_URL'])) {
            $this->payplug_url = $_SERVER['PAYPLUG_SITE_URL'];
        } else {
            $this->payplug_url = 'https://www.payplug.com';
        }
    }

    private function setConfigurationProperties()
    {
        $this->loadConfigurations();

        $this->api_live = $this->getConfiguration('PAYPLUG_LIVE_API_KEY');
        $this->api_test = $this->getConfiguration('PAYPLUG_TEST_API_KEY');

        // Set the uninstall notice according to the "keep_cards" configuration
        $this->confirmUninstall =
            $this->l('Are you sure you wish to uninstall this module and delete your settings?');
        if ($this->getConfiguration('PAYPLUG_KEEP_CARDS')) {
            $this->confirmUninstall .= ' ' . $this->l('All the registered cards of your customer will be kept.');
        } else {
            $this->confirmUninstall .= ' ' . $this->l('All the registered cards of your customer will be deleted.');
        }

        $this->current_api_key = $this->getAPIKey();
        $this->email = $this->getConfiguration('PAYPLUG_EMAIL');
        $this->ssl_enable = $this->getConfiguration('PS_SSL_ENABLED');

        if ((!isset($this->email) || (!isset($this->api_live) && empty($this->api_test)))) {
            $this->warning = $this->l(
                'In order to accept payments you need to configure your module by connecting your PayPlug account.'
            );
        }

        $this->payment_status = array(
            1 => $this->l('not paid'),
            2 => $this->l('paid'),
            3 => $this->l('failed'),
            4 => $this->l('partially refunded'),
            5 => $this->l('refunded'),
            6 => $this->l('on going'),
            7 => $this->l('cancelled'),
            8 => $this->l('authorized'),
            9 => $this->l('authorization expired'),
            10 => $this->l('oney pending'),
            11 => $this->l('abandoned'),
        );
    }

    private function setSecretKey()
    {
        if ($this->current_api_key != null) {
            $this->initializeApi();
        }
    }

    private function setUserAgent()
    {
        if ($this->current_api_key != null) {
            \Payplug\Core\HttpClient::addDefaultUserAgentProduct(
                'PayPlug-Prestashop',
                $this->version,
                'Prestashop/' . _PS_VERSION_
            );
        }
    }

    /**
     * @return bool
     * @see Module::install()
     *
     */
    public function install($do_parent_install = true)
    {
        $log = new MyLogPHP(_PS_MODULE_DIR_ . $this->name . '/log/install-log.csv');
        $log->info('Starting installation.');
        $report = $this->checkRequirements();
        if (!$report['php']['up2date']) {
            $this->_errors[] = Tools::displayError($this->l('Your server must run PHP 5.3 or greater'));
            $log->error('Installation failed: PHP Requirement.');
        }
        if (!$report['curl']['up2date']) {
            $this->_errors[] = Tools::displayError($this->l('PHP cURL extension must be enabled on your server'));
            $log->error('Installation failed: cURL Requirement.');
        }
        if (!$report['openssl']['up2date']) {
            $this->_errors[] = Tools::displayError($this->l('OpenSSL 1.0.1 or later'));
            $log->error('Installation failed: OpenSSL Requirement.');
        }

        if (version_compare(_PS_VERSION_, '1.5', '>=')) {
            if (Shop::isFeatureActive()) {
                Shop::setContext(Shop::CONTEXT_ALL);
            }
        }

        if ($do_parent_install) {
            if (!parent::install()) {
                $log->error('Installation failed: parent install.');
                return false;
            }
        }

        if (!$this->installHook() ||
            !$this->createConfig() ||
            !$this->createOrderStates() ||
            !$this->installSQL()
        ) {
            $log->error('Installation failed: hooks, configs, order states or sql.');
            return false;
        }

        if (version_compare(_PS_VERSION_, '1.6', '>=')) {
            if (!$this->registerHook('displayBackOfficeHeader') ||
                !$this->registerHook('actionOrderStatusUpdate')
            ) {
                $log->error('Installation failed: hooks for 1.6.');
                return false;
            }
        } elseif (version_compare(_PS_VERSION_, '1.5', '>=')) {
            if (!$this->registerHook('actionOrderStatusUpdate')) {
                $log->error('Installation failed: hook order status update.');
                return false;
            }
        } else {
            if (!$this->registerHook('updateOrderStatus')) {
                $log->error('Installation failed: hook order status update.');
                return false;
            }
        }

        if (version_compare(_PS_VERSION_, '1.5', '>=')) {
            if (!$this->registerHook('registerGDPRConsent') ||
                !$this->registerHook('actionDeleteGDPRCustomer') ||
                !$this->registerHook('actionExportGDPRData')
            ) {
                $log->error('Installation failed: hooks GDPR.');
                return false;
            }
        }

        if (!$this->registerHook('payment')) {
            $log->error('Installation failed: hook payment.');
            return false;
        }

        if (!$this->installTab()) {
            $log->error('Installation failed: tab.');
            return false;
        }

        $log->info('Default installation complete.');

        // install features who depends on version
        if ($this->checkVersion('1.6')) {
            $this->installOney();
        }

        $log->info('Installation complete.');

        return true;
    }

    /**
     * Install the required hooks
     * @return bool
     */
    private function installHook()
    {
        $hooks = array(
            'adminOrder',
            'customerAccount',
            'header',
            'paymentReturn',
            'actionAdminPerformanceControllerAfter',
        );

        $flag = true;
        foreach ($hooks as $hook) {
            $flag = $this->registerHook($hook) && $flag;
        }

        return $flag;
    }

    public function installTab()
    {
        $tab = new Tab();
        $tab->active = 1;
        $tab->class_name = 'AdminPayPlugInstallment';
        $tab->name = array();
        foreach (Language::getLanguages(true) as $lang) {
            $tab->name[$lang['id_lang']] = $this->l('Installment Plans');
        }

        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            $tab->id_parent = 3;
        } elseif (version_compare(_PS_VERSION_, '1.5', '>=') && version_compare(_PS_VERSION_, '1.6', '<')) {
            $tab->id_parent = 10;
        } else {
            $tab->id_parent = 0;
        }
        $tab->module = $this->name;
        $tab->position = Tab::getNbTabs(0);

        /*
        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            @copy(_PS_MODULE_DIR_.$this->name.'views/img/admin/tab.png', _PS_IMG_DIR_.'t/'.$tab->class_name.'.png');
        }
        */

        if (!$tab->add()) {
            return false;
        }

        $this->setConfiguration('PAYPLUG_ADMIN_INSTALLMENT_TAB', $tab->id);

        if (version_compare(_PS_VERSION_, '1.6', '>=')) {
            $legacyController = _PS_MODULE_DIR_ . $this->name . '/AdminPayPlugInstallment.php';
            $disabledLegacyController = _PS_MODULE_DIR_ . $this->name . '/__AdminPayPlugInstallment.php';
            if (file_exists($legacyController)) {
                rename($legacyController, $disabledLegacyController);
            }
        }

        return true;
    }

    private function uninstallTab()
    {
        $flag = true;
        $id_tab = $this->getConfiguration('PAYPLUG_ADMIN_INSTALLMENT_TAB');

        if ($id_tab) {
            $tab = new Tab($id_tab);
            $flag = $tab->delete();
        }

        // we have to remove this configuration only after uninstall of the admin tab for obvious reasons
        if (!Configuration::deleteByName('PAYPLUG_ADMIN_INSTALLMENT_TAB')) {
            $flag = false;
        }

        return $flag;
    }

    /**
     * @return bool
     * @see Module::uninstall()
     *
     */
    public function uninstall()
    {
        $keep_cards = $this->getConfiguration('PAYPLUG_KEEP_CARDS');

        $log = new MyLogPHP(_PS_MODULE_DIR_ . 'payplug/log/install-log.csv');
        $log->info('Starting uninstallation.');

        if (!$keep_cards) {
            PayPlugCard::deleteAll();
        }

        if (!parent::uninstall() ||
            !$this->deleteConfig() ||
            !$this->uninstallSQL($keep_cards)
        ) {
            $log->error('Uninstallation failed: configs or sql.');
            return false;
        }

        if (!$this->uninstallTab()) {
            $log->error('Uninstallation failed: tab.');
            return false;
        }
        $log->info('Uninstallation complete.');

        return true;
    }

    /**
     * Create basic configuration
     *
     * @return bool
     */
    public function createConfig()
    {
        $log = new MyLogPHP(_PS_MODULE_DIR_ . 'payplug/log/install-log.csv');

        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            $this->setConfiguration('PAYPLUG_VERSION_1_4', (string)$this->version);
        }

        if (!$this->setConfiguration('PAYPLUG_CURRENCIES', 'EUR') ||
            !$this->setConfiguration('PAYPLUG_MIN_AMOUNTS', 'EUR:1') ||
            !$this->setConfiguration('PAYPLUG_MAX_AMOUNTS', 'EUR:1000000') ||
            !$this->setConfiguration('PAYPLUG_TEST_API_KEY', null) ||
            !$this->setConfiguration('PAYPLUG_LIVE_API_KEY', null) ||
            !$this->setConfiguration('PAYPLUG_COMPANY_ID', null) ||
            !$this->setConfiguration('PAYPLUG_COMPANY_STATUS', '') ||
            !$this->setConfiguration('PAYPLUG_ALLOW_SAVE_CARD', 0) ||
            !$this->setConfiguration('PAYPLUG_SANDBOX_MODE', 1) ||
            !$this->setConfiguration('PAYPLUG_SHOW', 0) ||
            !$this->setConfiguration('PAYPLUG_EMAIL', null) ||
            !$this->setConfiguration('PAYPLUG_EMBEDDED_MODE', 0) ||
            !$this->setConfiguration('PAYPLUG_ONE_CLICK', null) ||
            !$this->setConfiguration('PAYPLUG_INST', null) ||
            !$this->setConfiguration('PAYPLUG_INST_MODE', 3) ||
            !$this->setConfiguration('PAYPLUG_INST_MIN_AMOUNT', 150) ||
            !$this->setConfiguration('PAYPLUG_DEBUG_MODE', 0) ||
            !$this->setConfiguration('PAYPLUG_DEFERRED', 0) ||
            !$this->setConfiguration('PAYPLUG_DEFERRED_AUTO', 0) ||
            !$this->setConfiguration('PAYPLUG_DEFERRED_STATE', 0) ||
            !$this->setConfiguration('PAYPLUG_KEEP_CARDS', 0)
        ) {
            $log->error('Installation failed: configurations failed.');
            return false;
        }

        return true;
    }

    /**
     * Delete basic configuration
     *
     * @return bool
     */
    public function deleteConfig()
    {
        $log = new MyLogPHP(_PS_MODULE_DIR_ . 'payplug/log/install-log.csv');

        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            Configuration::deleteByName('PAYPLUG_VERSION_1_4');
        }

        if (!Configuration::deleteByName('PAYPLUG_CURRENCIES') ||
            !Configuration::deleteByName('PAYPLUG_MIN_AMOUNTS') ||
            !Configuration::deleteByName('PAYPLUG_MAX_AMOUNTS') ||
            !Configuration::deleteByName('PAYPLUG_TEST_API_KEY') ||
            !Configuration::deleteByName('PAYPLUG_LIVE_API_KEY') ||
            !Configuration::deleteByName('PAYPLUG_COMPANY_ID') ||
            !Configuration::deleteByName('PAYPLUG_COMPANY_STATUS') ||
            !Configuration::deleteByName('PAYPLUG_ALLOW_SAVE_CARD') ||
            !Configuration::deleteByName('PAYPLUG_SANDBOX_MODE') ||
            !Configuration::deleteByName('PAYPLUG_SHOW') ||
            !Configuration::deleteByName('PAYPLUG_EMAIL') ||
            !Configuration::deleteByName('PAYPLUG_EMBEDDED_MODE') ||
            !Configuration::deleteByName('PAYPLUG_ORDER_STATE_PAID') ||
            !Configuration::deleteByName('PAYPLUG_ORDER_STATE_REFUND') ||
            !Configuration::deleteByName('PAYPLUG_ORDER_STATE_PENDING') ||
            !Configuration::deleteByName('PAYPLUG_ORDER_STATE_ERROR') ||
            !Configuration::deleteByName('PAYPLUG_ORDER_STATE_PAID_TEST') ||
            !Configuration::deleteByName('PAYPLUG_ORDER_STATE_REFUND_TEST') ||
            !Configuration::deleteByName('PAYPLUG_ORDER_STATE_PENDING_TEST') ||
            !Configuration::deleteByName('PAYPLUG_ORDER_STATE_ERROR_TEST') ||
            !Configuration::deleteByName('PAYPLUG_ORDER_STATE_AUTH') ||
            !Configuration::deleteByName('PAYPLUG_ORDER_STATE_AUTH_TEST') ||
            !Configuration::deleteByName('PAYPLUG_ORDER_STATE_EXP') ||
            !Configuration::deleteByName('PAYPLUG_ORDER_STATE_EXP_TEST') ||
            !Configuration::deleteByName('PAYPLUG_ORDER_STATE_ONEY_PG') ||
            !Configuration::deleteByName('PAYPLUG_ORDER_STATE_ONEY_PG_TEST') ||
            !Configuration::deleteByName('PAYPLUG_CONFIGURATION_OK') ||
            !Configuration::deleteByName('PAYPLUG_ONE_CLICK') ||
            !Configuration::deleteByName('PAYPLUG_ONEY') ||
            !Configuration::deleteByName('PAYPLUG_ONEY_ALLOWED_COUNTRIES') ||
            !Configuration::deleteByName('PAYPLUG_ONEY_MAX_AMOUNTS') ||
            !Configuration::deleteByName('PAYPLUG_ONEY_MIN_AMOUNTS') ||
            !Configuration::deleteByName('PAYPLUG_ONEY_OPTIMIZED') ||
            !Configuration::deleteByName('PAYPLUG_ONEY_TOS') ||
            !Configuration::deleteByName('PAYPLUG_ONEY_TOS_URL') ||
            !Configuration::deleteByName('PAYPLUG_INST') ||
            !Configuration::deleteByName('PAYPLUG_INST_MODE') ||
            !Configuration::deleteByName('PAYPLUG_INST_MIN_AMOUNT') ||
            !Configuration::deleteByName('PAYPLUG_DEBUG_MODE') ||
            !Configuration::deleteByName('PAYPLUG_DEFERRED') ||
            !Configuration::deleteByName('PAYPLUG_DEFERRED_AUTO') ||
            !Configuration::deleteByName('PAYPLUG_DEFERRED_STATE') ||
            !Configuration::deleteByName('PAYPLUG_KEEP_CARDS')
        ) {
            $log->error('Uninstallation failed: configurations failed.');
            return false;
        }

        return true;
    }

    /**
     * Create usual status
     *
     * @return bool
     */
    public function createOrderStates()
    {
        $log = new MyLogPHP(_PS_MODULE_DIR_ . 'payplug/log/install-log.csv');
        $log->info('Order state creation starting.');

        foreach ($this->order_states as $key => $state) {
            $this->createOrderState($key, $state, true);
            $this->createOrderState($key, $state, false);
        }

        $log->info('Order state creation ended.');
        return true;
    }

    /**
     * Create payplug order state and upgrade configuration table
     * @param $name
     * @param $state
     * @param bool $sandbox
     * @param bool $force
     * @return mixed
     */
    public function createOrderState($name, $state, $sandbox = true, $force = false)
    {
        $log = new MyLogPHP(_PS_MODULE_DIR_ . 'payplug/log/install-log.csv');
        $key_config = 'PAYPLUG_ORDER_STATE_' . Tools::strtoupper($name) . ($sandbox ? '_TEST' : '');

        $log->info('Order state: ' . $name . ($sandbox ? ' - test' : ''));
        $os = $this->getConfiguration($key_config);

        if (!$os) {
            if ($val = $this->findOrderState($state['name'], $sandbox)) {
                $os = $val;
            } elseif (!$sandbox && defined($state['cfg'])) {
                $os = constant($state['cfg']);
            } elseif (!$sandbox && $state['template'] != null) {
                $sql = 'SELECT DISTINCT `id_order_state`
                        FROM `' . _DB_PREFIX_ . 'order_state_lang` 
                        WHERE `template` = \'' . pSQL($state['template']) . '\'';
                $os = Db::getInstance()->getValue($sql);
            }
        }
        if (!$os || $force) {
            $log->info('Creating new order state.');
            $order_state = new OrderState();
            $order_state->logable = $state['logable'];
            $order_state->send_email = $state['send_email'];
            $order_state->paid = $state['paid'];
            $order_state->module_name = $state['module_name'];
            $order_state->hidden = $state['hidden'];
            $order_state->delivery = $state['delivery'];
            $order_state->invoice = $state['invoice'];
            $order_state->color = $state['color'];

            $tag = $sandbox ? ' [TEST]' : ' [PayPlug]';
            foreach (Language::getLanguages(false) as $lang) {
                $order_state->template[$lang['id_lang']] = $state['template'];
                if (in_array($lang['iso_code'], array('en', 'au', 'ca', 'ie', 'gb', 'uk', 'us'))) {
                    $order_state->name[$lang['id_lang']] = $state['name']['en'] . $tag;
                } elseif (in_array($lang['iso_code'], array('fr', 'be', 'lu', 'ch'))) {
                    $order_state->name[$lang['id_lang']] = $state['name']['fr'] . $tag;
                } elseif (in_array($lang['iso_code'], array('es', 'ar', 'cl', 'co', 'mx', 'py', 'uy', 've'))) {
                    $order_state->name[$lang['id_lang']] = $state['name']['es'] . $tag;
                } elseif (in_array($lang['iso_code'], array('it', 'sm', 'va'))) {
                    $order_state->name[$lang['id_lang']] = $state['name']['it'] . $tag;
                } else {
                    $order_state->name[$lang['id_lang']] = $state['name']['en'] . $tag;
                }
            }
            if ($order_state->add()) {
                $source = _PS_MODULE_DIR_ . $this->name . '/views/img/os/' . $name . '.gif';
                $destination = _PS_ROOT_DIR_ . '/img/os/' . $order_state->id . '.gif';
                @copy($source, $destination);
                $log->info('State created');
            }
            $os = $order_state->id;
            $log->info('ID: ' . $os);
        }
        return $this->setConfiguration($key_config, $os);
    }

    /**
     * Find id_order_state by name
     *
     * @param array $name
     * @param bool $test_mode
     * @return int OR bool
     */
    public function findOrderState($name, $test_mode = false)
    {
        if (!is_array($name) || empty($name)) {
            return false;
        } else {
            $sql = 'SELECT DISTINCT `id_order_state` 
                    FROM `' . _DB_PREFIX_ . 'order_state_lang`
                    WHERE `name` LIKE \'' . pSQL($name['en'] . ($test_mode ? ' [TEST]' : ' [PayPlug]')) . '\' 
                    OR `name` LIKE \'' . pSQL($name['fr'] . ($test_mode ? ' [TEST]' : ' [PayPlug]')) . '\' 
                    OR `name` LIKE \'' . pSQL($name['es'] . ($test_mode ? ' [TEST]' : ' [PayPlug]')) . '\' 
                    OR `name` LIKE \'' . pSQL($name['it'] . ($test_mode ? ' [TEST]' : ' [PayPlug]')) . '\'';
            return Db::getInstance()->getValue($sql);
        }
    }

    /**
     * Install SQL tables used by module
     *
     * @return bool
     */
    public function installSQL()
    {
        $log = new MyLogPHP(_PS_MODULE_DIR_ . 'payplug/log/install-log.csv');
        $log->info('Installation SQL Starting.');

        if (!defined('_MYSQL_ENGINE_')) {
            define('_MYSQL_ENGINE_', 'InnoDB');
        }

        $req_payplug_lock = '
            CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'payplug_lock` (
            `id_payplug_lock` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
            `id_cart` INT(11) UNSIGNED NOT NULL,
            `id_order` VARCHAR(100),
            `date_add` DATETIME NOT NULL DEFAULT \'1000-01-01 00:00:00\',
            `date_upd` DATETIME NOT NULL DEFAULT \'1000-01-01 00:00:00\',
            CONSTRAINT lock_cart_unique UNIQUE (id_cart)
            ) ENGINE=' . _MYSQL_ENGINE_;
        $res_payplug_lock = Db::getInstance()->execute($req_payplug_lock);

        if (!$res_payplug_lock) {
            $log->error('Installation SQL failed: PAYPLUG_LOCK.');
            return false;
        }

        $req_payplug_card = '
            CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'payplug_card` (
            `id_payplug_card` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
            `id_customer` int(11) UNSIGNED NOT NULL,
            `id_company` int(11) UNSIGNED NOT NULL,
            `is_sandbox` int(1) UNSIGNED NOT NULL,
            `id_card` varchar(255) NOT NULL,
            `last4` varchar(4) NOT NULL,
            `exp_month` varchar(4) NOT NULL,
            `exp_year` varchar(4) NOT NULL,
            `brand` varchar(255) DEFAULT NULL,
            `country` varchar(3) NOT NULL,
            `metadata` varchar(255) DEFAULT NULL
            ) ENGINE=' . _MYSQL_ENGINE_;
        $res_payplug_card = Db::getInstance()->execute($req_payplug_card);

        if (!$res_payplug_card) {
            $log->error('Installation SQL failed: PAYPLUG_CARD.');
            return false;
        }

        $req_payplug_payment_cart = '
            CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'payplug_payment_cart` (
            `id_payplug_payment_cart` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
            `id_payment` VARCHAR(255) NOT NULL,
            `id_cart` INT(11) UNSIGNED NOT NULL,
            `is_pending` TINYINT(1) NOT NULL DEFAULT 0, 
            `date_upd` DATETIME NULL
            ) ENGINE=' . _MYSQL_ENGINE_;
        $res_payplug_payment_cart = Db::getInstance()->execute($req_payplug_payment_cart);

        if (!$res_payplug_payment_cart) {
            $log->error('Installation SQL failed: PAYPLUG_PAYMENT_CART.');
            return false;
        }

        $req_payplug_installment_cart = '
            CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'payplug_installment_cart` (
            `id_payplug_installment_cart` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
            `id_installment` VARCHAR(255) NOT NULL,
            `id_cart` INT(11) UNSIGNED NOT NULL,
            `is_pending` TINYINT(1) NOT NULL DEFAULT 0, 
            `date_upd` DATETIME NULL
            ) ENGINE=' . _MYSQL_ENGINE_;
        $res_payplug_installment_cart = Db::getInstance()->execute($req_payplug_installment_cart);

        if (!$res_payplug_installment_cart) {
            $log->error('Installation SQL failed: PAYPLUG_INSTALLMENT_CART.');
            return false;
        }

        $req_payplug_installment = '
            CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'payplug_installment` (
            `id_payplug_installment` int(11) UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
            `id_installment` VARCHAR(255) NOT NULL,
            `id_payment` VARCHAR(255) NULL,
            `id_order` INT(11) UNSIGNED NOT NULL,
            `id_customer` INT(11) UNSIGNED NOT NULL,
            `order_total` INT(11) UNSIGNED NOT NULL,
            `step` VARCHAR(11) NOT NULL,
            `amount` INT(11) UNSIGNED NOT NULL,
            `status` INT(11) UNSIGNED NOT NULL,
            `scheduled_date` DATETIME NOT NULL
            ) ENGINE=' . _MYSQL_ENGINE_;
        $res_payplug_installment = Db::getInstance()->execute($req_payplug_installment);

        if (!$res_payplug_installment) {
            $log->error('Installation SQL failed: PAYPLUG_INSTALLMENTS.');
            return false;
        }


        // install table `payplug_logger`
        $req_payplug_logger = '
            CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'payplug_logger` (
            `id_payplug_logger` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
            `process` VARCHAR(255) NOT NULL,
            `content` TEXT NOT NULL,
            `date_add` DATETIME NULL,
            `date_upd` DATETIME NULL
            ) ENGINE=' . _MYSQL_ENGINE_;

        $res_payplug_logger = Db::getInstance()->execute($req_payplug_logger);

        if (!$res_payplug_logger) {
            $log->error('Installation SQL failed: PAYPLUG_LOGGERS.');
            return false;
        }

        $req_payplug_order_payment = '
        CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'payplug_order_payment` (
        `id_payplug_order_payment` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
        `id_order` INT(11) UNSIGNED NOT NULL,
        `id_payment` VARCHAR(255) NOT NULL
        ) ENGINE=' . _MYSQL_ENGINE_;
        $res_payplug_order_payment = Db::getInstance()->execute($req_payplug_order_payment);

        if (!$res_payplug_order_payment) {
            $log->error('Installation SQL failed: PAYPLUG_ORDER_PAYMENT.');
            return false;
        }

        // install table `payplug_cache`
        $req_payplug_cache = '
            CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'payplug_cache` (
            `id_payplug_cache` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
            `cache_key` VARCHAR(255) NOT NULL,
            `cache_value` TEXT NOT NULL,
            `date_add` DATETIME NULL,
            `date_upd` DATETIME NULL
            ) ENGINE=' . _MYSQL_ENGINE_;

        $res_payplug_cache = Db::getInstance()->execute($req_payplug_cache);

        if (!$res_payplug_cache) {
            $log->error('Installation SQL failed: PAYPLUG_CACHE.');
            return false;
        }
        $log->info('Installation SQL ended.');
        return true;
    }

    /**
     * Remove SQL tables used by module
     *
     * @return bool
     */
    public function uninstallSQL($keep_cards = false)
    {
        $log = new MyLogPHP(_PS_MODULE_DIR_ . 'payplug/log/install-log.csv');
        $log->info('Uninstallation SQL starting.');
        $req_payplug_lock = '
            DROP TABLE IF EXISTS `' . _DB_PREFIX_ . 'payplug_lock`';
        $res_payplug_lock = Db::getInstance()->execute($req_payplug_lock);

        if (!$res_payplug_lock) {
            $log->error('Uninstallation SQL failed: PAYPLUG_LOCK.');
            return false;
        }

        if (!$keep_cards) {
            $req_payplug_card = '
            DROP TABLE IF EXISTS `' . _DB_PREFIX_ . 'payplug_card`';
            $res_payplug_card = Db::getInstance()->execute($req_payplug_card);

            if (!$res_payplug_card) {
                $log->error('Uninstallation SQL failed: PAYPLUG_CARD.');
                return false;
            }
        } else {
            $log->info('Cards kept in database.');
        }

        $req_payplug_payment_cart = '
            DROP TABLE IF EXISTS `' . _DB_PREFIX_ . 'payplug_payment_cart`';
        $res_payplug_payment_cart = Db::getInstance()->execute($req_payplug_payment_cart);

        if (!$res_payplug_payment_cart) {
            $log->error('Uninstallation SQL failed: PAYPLUG_PAYMENT_CART.');
            return false;
        }

        $req_payplug_installment_cart = '
            DROP TABLE IF EXISTS `' . _DB_PREFIX_ . 'payplug_installment_cart`';
        $res_payplug_installment_cart = Db::getInstance()->execute($req_payplug_installment_cart);

        if (!$res_payplug_installment_cart) {
            $log->error('Uninstallation SQL failed: PAYPLUG_INSTALLMENT_CART.');
            return false;
        }

        $req_payplug_installment = '
            DROP TABLE IF EXISTS `' . _DB_PREFIX_ . 'payplug_installment`';
        $res_payplug_installment = Db::getInstance()->execute($req_payplug_installment);

        if (!$res_payplug_installment) {
            $log->error('Uninstallation SQL failed: PAYPLUG_INSTALLMENTS.');
            return false;
        }

        $req_payplug_cache = '
            DROP TABLE IF EXISTS `' . _DB_PREFIX_ . 'payplug_cache`';
        $res_payplug_cache = Db::getInstance()->Execute($req_payplug_cache);

        if (!$res_payplug_cache) {
            $log->error('Uninstallation SQL failed: PAYPLUG_CACHE.');
            return false;
        }

        $req_payplug_carrier = '
            DROP TABLE IF EXISTS `' . _DB_PREFIX_ . 'payplug_carrier`';
        $res_payplug_carrier = Db::getInstance()->Execute($req_payplug_carrier);

        if (!$res_payplug_carrier) {
            $log->error('Uninstallation SQL failed: PAYPLUG_CARRIER.');
            return false;
        }

        $req_payplug_order_payment = '
        DROP TABLE IF EXISTS `' . _DB_PREFIX_ . 'payplug_order_payment`';
        $res_payplug_order_payment = Db::getInstance()->execute($req_payplug_order_payment);

        if (!$res_payplug_order_payment) {
            $log->error('Uninstallation SQL failed: PAYPLUG_ORDER_PAYMENT.');
            return false;
        }

        $req_payplug_logger = '
        DROP TABLE IF EXISTS `' . _DB_PREFIX_ . 'payplug_logger`';
        $res_payplug_logger = Db::getInstance()->execute($req_payplug_logger);

        if (!$res_payplug_logger) {
            $log->error('Uninstallation SQL failed: PAYPLUG_LOGGER.');
            return false;
        }

        $log->info('Uninstallation SQL ended.');
        return true;
    }


    /**
     * Run the upgrade for a given module name and version.
     *
     * @return array
     */
    public function runUpgrade()
    {
        // List of upgraded version existing
        $files_version = array(
            '2.16.0',
            '2.18.0',
            '2.22.0',
            '2.24.1',
            '2.24.0',
            '2.26.0',
            '2.26.1',
            '2.26.2',
            '2.28.0',
            '2.30.0'
        );

        $upgrade_path = dirname(__FILE__) . '/upgrade/';
        if (_PS_VERSION_ < '1.5') {
            foreach ($files_version as $version) {
                $file = $upgrade_path . 'upgrade-' . $version . '.php';
                if ($this->database_version < $version && file_exists($file)) {
                    include_once($file);
                    call_user_func('upgrade_module_' . str_replace('.', '_', $version), $this);
                }
            }
        }
    }

    public function runUpgradeModule()
    {
        $upgrade = parent::runUpgradeModule();

        $this->checkOrderStates();

        return $upgrade;
    }

    private function _updateProcess()
    {
        if (Module::isInstalled('payplug')
            && $this->database_version < $this->version) {
            $this->runUpgrade();
        }
    }

    /**
     * assign variable for smarty
     *
     * @param string $variable
     * @param mixed $content
     * @return void
     */
    public function assignForVersion($variable, $content = null)
    {
        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            if (version_compare(_PS_VERSION_, '1.4', '<')) {
                //global $smarty;
                //$smarty->assign($variable, $content);
            } else {
                $this->context->smarty->assign($variable, $content);
            }
        } else {
            $this->smarty->assign($variable, $content);
        }
    }

    /**
     * Redirection
     *
     * @param string $link
     * @return void
     */
    public static function redirectForVersion($link)
    {
        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            Tools::redirectLink($link);
        } else {
            Tools::redirect($link);
        }
    }

    /**
     * Check if amount is correct
     *
     * @param int $amount
     * @param Order $order
     * @return bool
     */
    public static function checkAmountPaidIsCorrect($amount, $order)
    {
        $order_amount = $order->total_paid;

        if ($amount != 0) {
            return abs($order_amount - $amount) / $amount < 0.00001;
        } elseif ($order_amount != 0) {
            return abs($amount - $order_amount) / $order_amount < 0.00001;
        } else {
            return true;
        }
    }

    /**
     * @return void
     * @see Module::postProcess()
     *
     */
    private function postProcess()
    {
        $output = '';
        $curl_exists = extension_loaded('curl');
        $openssl_exists = extension_loaded('openssl');
        //$this->validationErrors['installment_config_error'] = false;

        if (Tools::isSubmit('submitAccount')) {
            $email = Tools::getValue('PAYPLUG_EMAIL');
            //we have to use $_POST because Tools:getValue() escape some characters and invalid passwords
            $password = $_POST['PAYPLUG_PASSWORD'];
            if ((!Validate::isEmail($email) || !Validate::isPasswd($password))
                && (Tools::getValue('PAYPLUG_EMAIL') != false)
            ) {
                $this->validationErrors['username_password'] = $this->l('The email and/or password was not correct.');
            } elseif ($curl_exists && $openssl_exists) {
                if ($this->login($email, $password)) {
                    $this->setConfiguration('PAYPLUG_EMAIL', $email);
                    $output .= $this->displayConfirmation($this->l('Settings updated'));
                    $this->setConfiguration('PAYPLUG_SHOW', 1);
                } else {
                    $this->validationErrors['username_password']
                        = $this->l('The email and/or password was not correct.');
                }
            }
        }

        if (Tools::isSubmit('submitDisconnect')) {
            $this->createConfig();
            $this->setConfiguration('PAYPLUG_SHOW', 0);
        }

        if (Tools::isSubmit('submitSettings')) {
            if (Tools::getValue('PAYPLUG_INST_MODE') && Tools::getValue('PAYPLUG_INST_MIN_AMOUNT') < 4) {
                $output .= $this->displayError($this->l('Settings not updated'));
                //$this->validationErrors['installment_config_error'] = true;
            } else {
                $this->setConfiguration('PAYPLUG_SANDBOX_MODE', Tools::getValue('payplug_sandbox'));
                $this->setConfiguration('PAYPLUG_EMBEDDED_MODE', Tools::getValue('payplug_embedded'));
                $this->setConfiguration('PAYPLUG_ONE_CLICK', Tools::getValue('payplug_one_click'));
                $this->setConfiguration('PAYPLUG_INST', Tools::getValue('payplug_inst'));
                $this->setConfiguration('PAYPLUG_INST_MODE', Tools::getValue('PAYPLUG_INST_MODE'));
                $this->setConfiguration('PAYPLUG_INST_MIN_AMOUNT', Tools::getValue('PAYPLUG_INST_MIN_AMOUNT'));
                $this->setConfiguration('PAYPLUG_ONEY', Tools::getValue('payplug_oney'));
                $this->setConfiguration('PAYPLUG_ONEY_TOS', Tools::getValue('payplug_oney_tos'));
                $this->setConfiguration('PAYPLUG_ONEY_TOS_URL', Tools::getValue('payplug_oney_tos_url'));
                $this->setConfiguration('PAYPLUG_ONEY_OPTIMIZED', Tools::getValue('payplug_oney_optimized'));
                $this->setConfiguration('PAYPLUG_SHOW', Tools::getValue('PAYPLUG_SHOW'));
                $this->setConfiguration('PAYPLUG_DEFERRED', Tools::getValue('payplug_deferred'));
                $this->setConfiguration('PAYPLUG_DEFERRED_AUTO', Tools::getValue('PAYPLUG_DEFERRED_AUTO'));
                if (Tools::getValue('PAYPLUG_DEFERRED_AUTO')) {
                    $this->setConfiguration('PAYPLUG_DEFERRED_STATE', Tools::getValue('PAYPLUG_DEFERRED_STATE'));
                }
                $output .= $this->displayConfirmation($this->l('Settings updated'));
            }
        }

        if (Tools::isSubmit('submitUninstallSettings')) {
            $this->setConfiguration('PAYPLUG_KEEP_CARDS', Tools::getValue('PAYPLUG_KEEP_CARDS'));
            $output .= $this->displayConfirmation($this->l('Settings updated'));
        }

        return $output;
    }

    /**
     * Send cURL request to PayPlug to patch a given payment
     *
     * @param String $pay_id
     * @param Array $data
     * @return Array
     */
    public function patchPayment($pay_id, $data)
    {
        $payment = new PayplugPayment();
        return $payment->update($pay_id, $data);
    }

    /**
     * login to Payplug API
     *
     * @param string $email
     * @param string $password
     * @return bool
     */
    public function login($email, $password)
    {
        $data = array
        (
            'email' => $email,
            'password' => $password
        );
        $data_string = json_encode($data);

        $url = $this->api_url . $this->routes['login'];
        $curl_version = curl_version();
        $process = curl_init($url);
        curl_setopt(
            $process,
            CURLOPT_HTTPHEADER,
            array(
                'Content-Type:application/json',
                'Content-Length: ' . Tools::strlen($data_string)
            )
        );
        curl_setopt($process, CURLOPT_POSTFIELDS, $data_string);
        curl_setopt($process, CURLOPT_POST, true);
        curl_setopt($process, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($process, CURLINFO_HEADER_OUT, true);
        curl_setopt($process, CURLOPT_SSL_VERIFYPEER, true);
        # >= 7.26 to 7.28.1 add a notice message for value 1 will be remove
        curl_setopt(
            $process,
            CURLOPT_SSL_VERIFYHOST,
            (version_compare($curl_version['version'], '7.21', '<') ? true : 2)
        );
        curl_setopt($process, CURLOPT_CAINFO, realpath(dirname(__FILE__) . '/cacert.pem')); //work only wiht cURL 7.10+
        $answer = curl_exec($process);
        $error_curl = curl_errno($process);

        curl_close($process);

        if ($error_curl) {
            return false;
        }

        $json_answer = Tools::jsonDecode($answer);

        if ($this->setApiKeysbyJsonResponse($json_answer)) {
            return true;
        }

        return false;
    }

    /**
     * Get account permission from Payplug API
     *
     * @param string $api_key
     * @return array OR bool
     */
    public function getAccount($api_key)
    {
        $url = $this->api_url . $this->routes['account'];
        $curl_version = curl_version();
        $process = curl_init($url);
        curl_setopt($process, CURLOPT_HTTPHEADER, array('Authorization: Bearer ' . $api_key));
        curl_setopt($process, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($process, CURLINFO_HEADER_OUT, true);
        curl_setopt($process, CURLOPT_SSL_VERIFYPEER, true);
        # >= 7.26 to 7.28.1 add a notice message for value 1 will be remove
        curl_setopt(
            $process,
            CURLOPT_SSL_VERIFYHOST,
            (version_compare($curl_version['version'], '7.21', '<') ? true : 2)
        );
        curl_setopt($process, CURLOPT_CAINFO, realpath(dirname(__FILE__) . '/cacert.pem')); //work only with cURL 7.10+
        $answer = curl_exec($process);
        $error_curl = curl_errno($process);

        curl_close($process);

        if ($error_curl) {
            return false;
        }

        $json_answer = Tools::jsonDecode($answer);

        if ($permissions = $this->treatAccountResponse($json_answer)) {
            return $permissions;
        }

        return false;
    }

    /**
     * Read API response and return permissions
     *
     * @param string $json_answer
     * @return array OR bool
     */
    public function treatAccountResponse($json_answer)
    {
        if ((isset($json_answer->object) && $json_answer->object == 'error')
            || empty($json_answer)
        ) {
            return false;
        }

        $id = $json_answer->id;

        $configuration = array(
            'currencies' => $this->getConfiguration('PAYPLUG_CURRENCIES'),
            'min_amounts' => $this->getConfiguration('PAYPLUG_MIN_AMOUNTS'),
            'max_amounts' => $this->getConfiguration('PAYPLUG_MAX_AMOUNTS'),
            'oney_allowed_countries' => $this->getConfiguration('PAYPLUG_ONEY_ALLOWED_COUNTRIES'),
            'oney_max_amounts' => $this->getConfiguration('PAYPLUG_ONEY_MAX_AMOUNTS'),
            'oney_min_amounts' => $this->getConfiguration('PAYPLUG_ONEY_MIN_AMOUNTS'),
        );
        if (isset($json_answer->configuration)) {
            if (isset($json_answer->configuration->currencies)
                && !empty($json_answer->configuration->currencies)
                && sizeof($json_answer->configuration->currencies)
            ) {
                $configuration['currencies'] = array();
                foreach ($json_answer->configuration->currencies as $value) {
                    $configuration['currencies'][] = $value;
                }
            }
            if (isset($json_answer->configuration->min_amounts)
                && !empty($json_answer->configuration->min_amounts)
                && sizeof($json_answer->configuration->min_amounts)
            ) {
                $configuration['min_amounts'] = '';
                foreach ($json_answer->configuration->min_amounts as $key => $value) {
                    $configuration['min_amounts'] .= $key . ':' . $value . ';';
                }
                $configuration['min_amounts'] = Tools::substr($configuration['min_amounts'], 0, -1);
            }
            if (isset($json_answer->configuration->max_amounts)
                && !empty($json_answer->configuration->max_amounts)
                && sizeof($json_answer->configuration->max_amounts)
            ) {
                $configuration['max_amounts'] = '';
                foreach ($json_answer->configuration->max_amounts as $key => $value) {
                    $configuration['max_amounts'] .= $key . ':' . $value . ';';
                }
                $configuration['max_amounts'] = Tools::substr($configuration['max_amounts'], 0, -1);
            }

            if (isset($json_answer->configuration->oney)) {
                if (isset($json_answer->configuration->oney->allowed_countries)
                    && !empty($json_answer->configuration->oney->allowed_countries)
                    && sizeof($json_answer->configuration->oney->allowed_countries)) {
                    $allowed = '';
                    foreach ($json_answer->configuration->oney->allowed_countries as $country) {
                        $allowed .= $country . ',';
                    }
                    $configuration['oney_allowed_countries'] = Tools::substr($allowed, 0, -1);
                }
                if (isset($json_answer->configuration->oney->min_amounts)
                    && !empty($json_answer->configuration->oney->min_amounts)
                    && sizeof($json_answer->configuration->oney->min_amounts)) {
                    $configuration['oney_min_amounts'] = '';
                    foreach ($json_answer->configuration->oney->min_amounts as $key => $value) {
                        $configuration['oney_min_amounts'] .= $key . ':' . $value . ';';
                    }
                    $configuration['oney_min_amounts'] = Tools::substr($configuration['oney_min_amounts'], 0, -1);
                }
                if (isset($json_answer->configuration->oney->max_amounts)
                    && !empty($json_answer->configuration->oney->max_amounts)
                    && sizeof($json_answer->configuration->oney->max_amounts)) {
                    $configuration['oney_max_amounts'] = '';
                    foreach ($json_answer->configuration->oney->max_amounts as $key => $value) {
                        $configuration['oney_max_amounts'] .= $key . ':' . $value . ';';
                    }
                    $configuration['oney_max_amounts'] = Tools::substr($configuration['oney_max_amounts'], 0, -1);
                }
            }
        }

        $permissions = array(
            'use_live_mode' => $json_answer->permissions->use_live_mode,
            'can_save_cards' => $json_answer->permissions->can_save_cards,
            'can_create_installment_plan' => $json_answer->permissions->can_create_installment_plan,
            'can_create_deferred_payment' => $json_answer->permissions->can_create_deferred_payment,
            'can_use_oney' => $json_answer->permissions->can_use_oney,
        );

        // If sandbox mode active, no allowed countries sent
        // Then set default as `FR,MQ,YT,RE,GF,GP,IT`
        if (isset($json_answer->is_live) && !$json_answer->is_live) {
            $configuration['oney_allowed_countries'] = 'FR,MQ,YT,RE,GF,GP,IT';
        }

        $currencies = implode(';', $configuration['currencies']);
        $this->setConfiguration('PAYPLUG_CURRENCIES', $currencies);
        $this->setConfiguration('PAYPLUG_MIN_AMOUNTS', $configuration['min_amounts']);
        $this->setConfiguration('PAYPLUG_MAX_AMOUNTS', $configuration['max_amounts']);
        $this->setConfiguration('PAYPLUG_ONEY_ALLOWED_COUNTRIES', $configuration['oney_allowed_countries']);
        $this->setConfiguration('PAYPLUG_ONEY_MAX_AMOUNTS', $configuration['oney_max_amounts']);
        $this->setConfiguration('PAYPLUG_ONEY_MIN_AMOUNTS', $configuration['oney_min_amounts']);
        $this->setConfiguration('PAYPLUG_COMPANY_ID', $id);

        return $permissions;
    }

    /**
     * Register API Keys
     *
     * @param string $json_answer
     * @return bool
     */
    public function setApiKeysbyJsonResponse($json_answer)
    {
        if (isset($json_answer->object) && $json_answer->object == 'error') {
            return false;
        }

        $api_keys = array();
        $api_keys['test_key'] = '';
        $api_keys['live_key'] = '';

        if (isset($json_answer->secret_keys)) {
            if (isset($json_answer->secret_keys->test)) {
                $api_keys['test_key'] = $json_answer->secret_keys->test;
            }
            if (isset($json_answer->secret_keys->live)) {
                $api_keys['live_key'] = $json_answer->secret_keys->live;
            }
        }

        $this->setConfiguration('PAYPLUG_TEST_API_KEY', $api_keys['test_key']);
        $this->setConfiguration('PAYPLUG_LIVE_API_KEY', $api_keys['live_key']);

        return true;
    }

    /**
     * @return string
     * @see Module::getContent()
     *
     */
    public function getContent()
    {
        if (Tools::getValue('_ajax')) {
            $this->adminAjaxController();
        }

        $this->html = '';
        $this->html .= $this->postProcess();

        $this->assignContentVar();

        $this->html .= $this->fetchTemplateRC('views/templates/admin/admin.tpl');

        return $this->html;
    }

    public function getLogin()
    {
        $this->html = '';
        $this->html .= $this->postProcess();

        $this->assignContentVar();

        $this->html = $this->fetchTemplateRC('views/templates/admin/login.tpl');

        return $this->html;
    }

    private function assignContentVar()
    {
        if (Tools::getValue('uninstall_config')) {
            return $this->getUninstallContent();
        }

        require_once(_PS_MODULE_DIR_ . 'payplug/classes/PayplugAdmin.php');
        $this->payplugAdmin = new PayplugAdmin();
        $this->checkConfiguration();

        $configurations = array(
            'show' => $this->getConfiguration('PAYPLUG_SHOW'),
            'email' => $this->getConfiguration('PAYPLUG_EMAIL'),
            'sandbox_mode' => $this->getConfiguration('PAYPLUG_SANDBOX_MODE'),
            'embedded_mode' => $this->getConfiguration('PAYPLUG_EMBEDDED_MODE'),
            'one_click' => $this->getConfiguration('PAYPLUG_ONE_CLICK'),
            'inst' => $this->getConfiguration('PAYPLUG_INST'),
            'inst_mode' => $this->getConfiguration('PAYPLUG_INST_MODE'),
            'inst_min_amount' => $this->getConfiguration('PAYPLUG_INST_MIN_AMOUNT'),
            'test_api_key' => $this->getConfiguration('PAYPLUG_TEST_API_KEY'),
            'live_api_key' => $this->getConfiguration('PAYPLUG_LIVE_API_KEY'),
            'debug_mode' => $this->getConfiguration('PAYPLUG_DEBUG_MODE'),
            'deferred' => $this->getConfiguration('PAYPLUG_DEFERRED'),
            'deferred_auto' => $this->getConfiguration('PAYPLUG_DEFERRED_AUTO'),
            'deferred_state' => $this->getConfiguration('PAYPLUG_DEFERRED_STATE'),
        );

        if ($this->checkVersion()) {
            $configurations = array_merge(
                $configurations,
                array(
                    'oney' => $this->getConfiguration('PAYPLUG_ONEY'),
                    'oney_tos' => $this->getConfiguration('PAYPLUG_ONEY_TOS'),
                    'oney_tos_url' => $this->getConfiguration('PAYPLUG_ONEY_TOS_URL'),
                    'oney_optimized' => $this->getConfiguration('PAYPLUG_ONEY_OPTIMIZED'),
                )
            );
        }

        $connected = !empty($configurations['email'])
            && (!empty($configurations['test_api_key']) || !empty($configurations['live_api_key']));

        if (count($this->validationErrors && !$connected)) {
            $this->context->smarty->assign(array(
                'validationErrors' => $this->validationErrors,
            ));
        }

        $valid_key = $this->getAPIKey();
        if (!empty($valid_key)) {
            $permissions = $this->getAccount($valid_key);
            //todo: premium toujours necessaire?
            $premium = (
                $permissions['can_save_cards']
                && $permissions['can_create_installment_plan']
                && $permissions['can_create_deferred_payment']
            );
        } else {
            $verified = false;
            $premium = false;
        }
        if (!empty($configurations['live_api_key'])) {
            $verified = true;
        } else {
            $verified = false;
        }

        $is_active = (bool)$configurations['show'];

        $p_error = '';
        if (!$connected) {
            if (isset($this->validationErrors['username_password'])) {
                $p_error .= $this->validationErrors['username_password'];
            } elseif (isset($this->validationErrors['login'])) {
                if (isset($this->validationErrors['username_password'])) {
                    $p_error .= ' ';
                }
                $p_error .= $this->validationErrors['login'];
            }
            $this->context->smarty->assign(array(
                'p_error' => $p_error,
            ));
        } else {
            $this->context->smarty->assign(array(
                'PAYPLUG_EMAIL' => $configurations['email'],
            ));
        }

        $this->addJsRC(__PS_BASE_URI__ . 'modules/payplug/views/js/admin.js');
        $this->addCSSRC(__PS_BASE_URI__ . 'modules/payplug/views/css/admin.css');

        $admin_ajax_url = $this->getAdminAjaxUrl();

        $login_infos = array();

        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            $installments_panel_url = 'index.php?tab=AdminPayPlugInstallment&token='
                . Tools::getAdminTokenLite('AdminPayPlugInstallment');
        } else {
            $installments_panel_url = 'index.php?controller=AdminPayPlugInstallment&token='
                . Tools::getAdminTokenLite('AdminPayPlugInstallment');
        }

        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            $version = 1.4;
        } elseif (version_compare(_PS_VERSION_, '1.6', '<')) {
            $version = 1.5;
        } elseif (version_compare(_PS_VERSION_, '1.6', '>=')) {
            $version = 1.6;
        }

        $switch_options = $this->getSwitchConfiguration($configurations);

        $faq_links = $this->getFAQLinks(Context::getContext()->language->iso_code);

        $this->context->smarty->assign(array(
            'form_action' => (string)($_SERVER['REQUEST_URI']),
            'url_logo' => __PS_BASE_URI__ . 'modules/payplug/views/img/logo_payplug.png',
            'payplug_module_dir' => _PS_MODULE_DIR_,
            'admin_ajax_url' => $admin_ajax_url,
            'check_configuration' => $this->check_configuration,
            'pp_version' => $this->version,
            'connected' => $connected,
            'verified' => $verified,
            'premium' => $premium,
            'is_active' => $is_active,
            'payplug_url' => $this->payplug_url,
            'PAYPLUG_SANDBOX_MODE' => $configurations['sandbox_mode'],
            'PAYPLUG_EMBEDDED_MODE' => $configurations['embedded_mode'],
            'PAYPLUG_ONE_CLICK' => $configurations['one_click'],
            'PAYPLUG_INST' => $configurations['inst'],
            'PAYPLUG_INST_MODE' => $configurations['inst_mode'],
            'PAYPLUG_INST_MIN_AMOUNT' => $configurations['inst_min_amount'],
            'PAYPLUG_SHOW' => $configurations['show'],
            'PAYPLUG_DEBUG_MODE' => $configurations['debug_mode'],
            'PAYPLUG_DEFERRED' => $configurations['deferred'],
            'PAYPLUG_DEFERRED_AUTO' => $configurations['deferred_auto'],
            'PAYPLUG_DEFERRED_STATE' => $configurations['deferred_state'],
            'login_infos' => $login_infos,
            'version' => $version,
            'iso' => $this->context->language->iso_code,
            'installments_panel_url' => $installments_panel_url,
            'order_states' => $this->getOrderStates(),
            'switch_deferred_payment' => $switch_options['switch_deferred_payment'],
            'switch_installment_plan' => $switch_options['switch_installment_plan'],
            'switch_one_click' => $switch_options['switch_one_click'],
            'switch_embedded' => $switch_options['switch_embedded'],
            'switch_sandbox' => $switch_options['switch_sandbox'],
            'faq_links' => $faq_links,
        ));


        if ($this->checkVersion()) {
            $amounts = $this->getOneyPriceLimit();
            $oney_min_amounts = ($amounts['min'] / 100);
            $oney_max_amounts = ($amounts['max'] / 100);

            $this->context->smarty->assign(array(
                'PAYPLUG_ONEY' => $configurations['oney'],
                'payplug_oney_tos' => $configurations['oney_tos'],
                'payplug_oney_tos_url' => $configurations['oney_tos_url'],
                'payplug_oney_optimized' => $configurations['oney_optimized'],
                'switch_oney' => $switch_options['switch_oney'],
                'oney_min_amounts' => $oney_min_amounts,
                'oney_max_amounts' => $oney_max_amounts,
            ));
        }
    }

    private function getSwitchConfiguration($configurations)
    {
        $connected = !empty($configurations['email'])
            && (!empty($configurations['test_api_key']) || !empty($configurations['live_api_key']));

        $switch_sandbox = array(
            'name' => 'payplug_sandbox',
            'label' => $this->l('Mode'),
            'active' => $connected,
            'checked' => $configurations['sandbox_mode'],
            'label_left' => $this->l('test'),
            'label_right' => $this->l('live'),
        );

        $switch_embedded = array(
            'name' => 'payplug_embedded',
            'label' => $this->l('Payment page'),
            'active' => $connected,
            'checked' => $configurations['embedded_mode'],
            'label_left' => $this->l('embedded'),
            'label_right' => $this->l('redirected'),
        );

        $switch_one_click = array(
            'name' => 'payplug_one_click',
            'label' => $this->l('Enable one-click payments'),
            'active' => $connected,
            'checked' => $configurations['one_click'],
            'label_left' => $this->l('yes'),
            'label_right' => $this->l('no'),
        );

        $switch_installment_plan = array(
            'name' => 'payplug_inst',
            'label' => $this->l('Enable payments by installments'),
            'active' => $connected,
            'checked' => $configurations['inst'],
            'label_left' => $this->l('yes'),
            'label_right' => $this->l('no'),
        );

        $switch_deferred_payment = array(
            'name' => 'payplug_deferred',
            'label' => $this->l('Defer the payment capture'),
            'active' => $connected,
            'checked' => $configurations['deferred'],
            'label_left' => $this->l('yes'),
            'label_right' => $this->l('no'),
        );

        $switch_options = array(
            'switch_deferred_payment' => $switch_deferred_payment,
            'switch_installment_plan' => $switch_installment_plan,
            'switch_one_click' => $switch_one_click,
            'switch_embedded' => $switch_embedded,
            'switch_sandbox' => $switch_sandbox,
        );

        if ($this->checkVersion()) {
            $switch_options['switch_oney'] = array(
                'name' => 'payplug_oney',
                'label' => $this->l('Activate Payments Oney'),
                'active' => $connected,
                'checked' => $configurations['oney'],
                'label_left' => $this->l('yes'),
                'label_right' => $this->l('no'),
            );
        }

        return $switch_options;
    }

    public function getUninstallContent()
    {
        $this->html = '';
        $this->html .= $this->postProcess();

        $PAYPLUG_KEEP_CARDS = $this->getConfiguration('PAYPLUG_KEEP_CARDS');

        $this->addJsRC(__PS_BASE_URI__ . 'modules/payplug/views/js/admin.js');
        $this->addCSSRC(__PS_BASE_URI__ . 'modules/payplug/views/css/admin.css');

        $this->context->smarty->assign(array(
            'form_action' => (string)($_SERVER['REQUEST_URI']),
            'url_logo' => __PS_BASE_URI__ . 'modules/payplug/views/img/logo_payplug.png',
            'payplug_url' => $this->payplug_url,
            'PAYPLUG_KEEP_CARDS' => $PAYPLUG_KEEP_CARDS,
        ));

        $this->html .= $this->fetchTemplateRC('views/templates/admin/admin_uninstall_configuration.tpl');

        return $this->html;
    }

    /**
     * Fetch smarty template
     *
     * @param string $file
     * @return string
     */
    public function fetchTemplateRC($file)
    {
        $output = $this->display(__FILE__, $file);
        return $output;
    }

    /**
     * Check various configurations
     *
     * @return string
     */
    public function getCheckFieldset()
    {
        require_once(_PS_MODULE_DIR_ . 'payplug/classes/PayplugAdmin.php');
        $this->payplugAdmin = new PayplugAdmin();
        $this->checkConfiguration();
        $this->html = '';

        $admin_ajax_url = $this->getAdminAjaxUrl();

        $this->context->smarty->assign(array(
            'admin_ajax_url' => $admin_ajax_url,
            'check_configuration' => $this->check_configuration,
            'pp_version' => $this->version,
        ));
        $this->html = $this->fetchTemplateRC('views/templates/admin/fieldset.tpl');

        return $this->html;
    }

    public function checkRequirements()
    {
        $php_min_version = 50300;
        $curl_min_version = '7.21';
        $openssl_min_version = 0x1000100f;
        $report = array(
            'php' => array(
                'version' => 0,
                'installed' => true,
                'up2date' => false,
            ),
            'curl' => array(
                'version' => 0,
                'installed' => false,
                'up2date' => false,
            ),
            'openssl' => array(
                'version' => 0,
                'installed' => false,
                'up2date' => false,
            ),
        );

        //PHP
        if (!defined('PHP_VERSION_ID')) {
            $report['php']['version'] = PHP_VERSION;
            $php_version = explode('.', PHP_VERSION);
            define('PHP_VERSION_ID', ($php_version[0] * 10000 + $php_version[1] * 100 + $php_version[2]));
        }
        $report['php']['up2date'] = PHP_VERSION_ID >= $php_min_version ? true : false;

        //cURL
        $curl_exists = extension_loaded('curl');
        if ($curl_exists) {
            $curl_version = curl_version();
            $report['curl']['version'] = $curl_version['version'];
            $report['curl']['installed'] = true;
            $report['curl']['up2date'] =
                version_compare($curl_version['version'], $curl_min_version, '>=') ? true : false;
        }

        //OpenSSl
        $openssl_exists = extension_loaded('openssl');
        if ($openssl_exists) {
            $report['openssl']['version'] = OPENSSL_VERSION_NUMBER;
            $report['openssl']['installed'] = true;
            $report['openssl']['up2date'] = OPENSSL_VERSION_NUMBER >= $openssl_min_version ? true : false;
        }

        return $report;
    }

    public function checkConfiguration()
    {
        $payplug_email = $this->getConfiguration('PAYPLUG_EMAIL');
        $payplug_test_api_key = $this->getConfiguration('PAYPLUG_TEST_API_KEY');
        $payplug_live_api_key = $this->getConfiguration('PAYPLUG_LIVE_API_KEY');
        $payplug_oney = $this->getConfiguration('PAYPLUG_ONEY');
        $payplug_oney_tos = $this->getConfiguration('PAYPLUG_ONEY_TOS');
        $payplug_oney_tos_url = $this->getConfiguration('PAYPLUG_ONEY_TOS_URL');    // Oney input TOS URL

        $report = $this->checkRequirements();

        if (empty($payplug_email) || (empty($payplug_test_api_key) && empty($payplug_live_api_key))) {
            $is_payplug_connected = false;
        } else {
            $is_payplug_connected = true;
        }

        if ($report['curl']['installed'] &&
            $report['php']['up2date'] &&
            $report['openssl']['installed'] &&
            $report['openssl']['up2date'] &&
            $is_payplug_connected
        ) {
            $is_payplug_configured = true;
        } else {
            $is_payplug_configured = false;
        }

        $this->check_configuration = array('warning' => array(), 'error' => array(), 'success' => array());

        $curl_warning = $this->l('PHP cURL extension must be enabled on your server');
        if ($report['curl']['installed']) {
            $this->check_configuration['success'][] .= $curl_warning;
        } else {
            $this->check_configuration['error'][] .= $curl_warning;
        }

        $php_warning = $this->l('Your server must run PHP 5.3 or greater');
        if ($report['php']['up2date']) {
            $this->check_configuration['success'][] .= $php_warning;
        } else {
            $this->check_configuration['error'][] .= $php_warning;
        }

        $openssl_warning = $this->l('OpenSSL 1.0.1 or later');
        if ($report['openssl']['installed'] && $report['openssl']['up2date']) {
            $this->check_configuration['success'][] .= $openssl_warning;
        } else {
            $this->check_configuration['error'][] .= $openssl_warning;
        }

        $connexion_warning = $this->l('You must connect your Payplug account');
        if ($is_payplug_connected) {
            $this->check_configuration['success'][] .= $connexion_warning;
        } else {
            $this->check_configuration['error'][] .= $connexion_warning;
        }

        //$configuration_warning =
        $check_warning = $this->l('Unfortunately at least one issue is preventing you from using Payplug.') . ' '
            . $this->l('Refresh the page or click "Check" once they are fixed');
        if (!$is_payplug_configured) {
            $this->check_configuration['warning'][] .= $check_warning;
        }

        // Display warning message, if Oney and TOS switches activated, but empty URL
        if ($payplug_oney && $payplug_oney_tos && ((!$payplug_oney_tos_url) || (empty($payplug_oney_tos_url)))) {
            $tos_warning_message = $this->l('Please manage the “General terms and conditions” part for Oney');
            $this->check_configuration['warning'][] .= $tos_warning_message;
        }

        return true;
    }

    /**
     * Display the right pop-in
     *
     * @param string $type
     * @param array $args
     * @return string
     */
    public function displayPopin($type, $args = null)
    {
        if ($type == 'confirm') {
            $this->context->smarty->assign(array(
                'sandbox' => $args['sandbox'],
                'embedded' => $args['embedded'],
                'one_click' => $args['one_click'],
                'oney' => $args['oney'],
                'installment' => $args['installment'],
                'deferred' => $args['deferred'],
                'activate' => $args['activate'],
            ));
        }

        $admin_ajax_url = $this->getAdminAjaxUrl();

        $this->context->smarty->assign(array(
            'type' => $type,
            'admin_ajax_url' => $admin_ajax_url,
            'payplug_url' => $this->payplug_url,
        ));
        $this->html = $this->fetchTemplateRC('views/templates/admin/popin.tpl');

        die(Tools::jsonEncode(array('content' => $this->html)));
    }

    /**
     * Include js script in template
     *
     * @param string $js_uri
     * @return void
     */
    public function addJsRC($js_uri)
    {
        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            $js_files = array();
            $this->html .= '<script type="text/javascript" src="' . $js_uri . '"></script>';
            $js_files[] = $js_uri;
        } else {
            $this->context->controller->addJS($js_uri);
        }
    }

    /**
     * Include css in template
     *
     * @param string $css_uri
     * @param string $css_media_type
     * @return void
     */
    public function addCSSRC($css_uri, $css_media_type = 'all')
    {
        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            $this->html .= '<link type="text/css" rel="stylesheet" href="' . $css_uri . '" />';
        } else {
            $this->context->controller->addCSS($css_uri, $css_media_type);
        }
    }

    /**
     * Retrieve payment stored
     *
     * @param int $id_cart
     * @return int OR bool
     */
    public function getPaymentByCart($id_cart)
    {
        $sql = 'SELECT ppc.id_payment 
                FROM ' . _DB_PREFIX_ . 'payplug_payment_cart ppc  
                WHERE ppc.id_cart = ' . (int)$id_cart;
        return Db::getInstance()->getValue($sql);
    }

    /**
     * Retrieve installment stored
     *
     * @param int $id_cart
     * @return int OR bool
     */
    public function getInstallmentByCart($id_cart)
    {
        $sql = 'SELECT pic.id_installment 
                FROM ' . _DB_PREFIX_ . 'payplug_installment_cart pic 
                WHERE pic.id_cart = ' . (int)$id_cart;
        return Db::getInstance()->getValue($sql);
    }

    /**
     * Register transaction as pending to establish link with order in case of error
     *
     * @param int $id_cart
     * @return bool
     */
    public function registerPendingTransaction($id_cart)
    {
        $sql = 'UPDATE ' . _DB_PREFIX_ . 'payplug_payment_cart ppc  
                SET ppc.is_pending = 1
                WHERE ppc.id_cart = ' . (int)$id_cart;
        return Db::getInstance()->execute($sql);
    }

    /**
     * Get id_payment from a pending transaction for a given cart
     *
     * @param int $id_cart
     * @return string id_payment OR bool
     */
    public function isTransactionPending($id_cart)
    {
        $sql = 'SELECT ppc.id_payment 
                FROM ' . _DB_PREFIX_ . 'payplug_payment_cart ppc  
                WHERE ppc.id_cart = ' . (int)$id_cart . '
                AND ppc.is_pending = 1';
        return Db::getInstance()->getValue($sql);
    }

    /**
     * check if currency is allowed
     *
     * @param Cart $cart
     * @return bool
     */
    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);

        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * Check if amount is correct
     *
     * @param Cart $cart
     * @return bool
     */
    public function checkAmount($cart)
    {
        $currency = new Currency($cart->id_currency);
        $amounts_by_currency = $this->getAmountsByCurrency($currency->iso_code);
        if (version_compare(_PS_VERSION_, '1.4', '<')) {
            $amount = $cart->getOrderTotal(true, 3) * 100;
        } else {
            $amount = $cart->getOrderTotal(true, Cart::BOTH) * 100;
        }
        if ($amount < $amounts_by_currency['min_amount'] || $amount > $amounts_by_currency['max_amount']) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * Get amounts with the right currency
     *
     * @param string $iso_code
     * @return array
     */
    public function getAmountsByCurrency($iso_code)
    {
        $min_amounts = array();
        $max_amounts = array();
        $oney_min_amounts = array();
        $oney_max_amounts = array();
        $iso_code = Tools::strtoupper($iso_code);

        foreach (explode(';', Tools::strtoupper($this->getConfiguration('PAYPLUG_MIN_AMOUNTS'))) as $amount_cur) {
            $cur = array();
            preg_match('/^([A-Z]{3}):([0-9]*)$/', $amount_cur, $cur);
            $min_amounts[$cur[1]] = (int)$cur[2];
        }
        foreach (explode(';', Tools::strtoupper($this->getConfiguration('PAYPLUG_MAX_AMOUNTS'))) as $amount_cur) {
            $cur = array();
            preg_match('/^([A-Z]{3}):([0-9]*)$/', $amount_cur, $cur);
            $max_amounts[$cur[1]] = (int)$cur[2];
        }
        foreach (explode(';', Tools::strtoupper($this->getConfiguration('PAYPLUG_ONEY_MIN_AMOUNTS'))) as $amount_cur) {
            $cur = array();
            preg_match('/^([A-Z]{3}):([0-9]*)$/', $amount_cur, $cur);
            $oney_min_amounts[$cur[1]] = (int)$cur[2];
        }
        foreach (explode(';', Tools::strtoupper($this->getConfiguration('PAYPLUG_ONEY_MAX_AMOUNTS'))) as $amount_cur) {
            $cur = array();
            preg_match('/^([A-Z]{3}):([0-9]*)$/', $amount_cur, $cur);
            $oney_max_amounts[$cur[1]] = (int)$cur[2];
        }
        $current_min_amount = $min_amounts[$iso_code];
        $current_max_amount = $max_amounts[$iso_code];
        $current_oney_min_amount = $oney_min_amounts[$iso_code];
        $current_oney_max_amount = $oney_max_amounts[$iso_code];

        return array(
            'min_amount' => $current_min_amount,
            'max_amount' => $current_max_amount,
            'oney_min_amount' => $current_oney_min_amount,
            'oney_max_amount' => $current_oney_max_amount,
        );
    }

    /**
     * Get all country iso-code of ISO 3166-1 alpha-2 norm
     * Source: DB PayPlug
     *
     * @return array | null
     */
    public function getIsoCodeList()
    {
        $country_list_path = _PS_MODULE_DIR_ . 'payplug/lib/iso_3166-1_alpha-2/data.csv';
        $iso_code_list = array();
        if (($handle = fopen($country_list_path, 'r')) !== false) {
            while (($data = fgetcsv($handle, 1000, ',')) !== false) {
                $iso_code_list[] = Tools::strtoupper($data[0]);
            }
            fclose($handle);
            return $iso_code_list;
        } else {
            return null;
        }
    }

    /**
     * Get the right country iso-code or null if it does'nt fit the ISO 3166-1 alpha-2 norm
     *
     * @param int $country_id
     * @return int | null
     */
    public function getIsoCodeByCountryId($country_id)
    {
        $iso_code_list = $this->getIsoCodeList();
        if (!is_array($iso_code_list) || empty($iso_code_list) || !count($iso_code_list)) {
            return null;
        }
        if (!Validate::isInt($country_id)) {
            return null;
        }
        $country = new Country($country_id);
        if (!Validate::isLoadedObject($country)) {
            return null;
        }
        if (!in_array(Tools::strtoupper($country->iso_code), $iso_code_list)) {
            return null;
        } else {
            return Tools::strtoupper($country->iso_code);
        }
    }

    /**
     * prepare payment
     *
     * @param int $id_cart
     * @param string $id_card
     * @return mixed
     */
    public function preparePayment($id_card = null)
    {
        $this->log_payment->info('Starting payment.');

        if (!Validate::isLoadedObject($this->context->cart)) {
            $this->log_payment->info('Payment abort: missing cart');
            $data = array(
                'result' => false,
                'response' => $this->l('The cart is missing'),
            );
            return Tools::jsonEncode($data);
        }

        if ($this->isPaymentPending($this->context->cart->id)) {
            $this->log_payment->info('Payment abort: is pending');
            $data = array(
                'result' => false,
                'response' => $this->l('payment is pending'),
            );
            return Tools::jsonEncode($data);
        }

        $deferred = $this->getConfiguration('PAYPLUG_DEFERRED');
        $options = array();

        if ($deferred) {
            $options['deferred'] = true;
        }
        if ($oney_type = Tools::getValue('io')) {
            // todo: set var in method PayPlugPaymentOney
            $options['oney_type'] = $oney_type;
            $options['oney_form'] = Tools::getValue('form');

            if (!$this->getConfiguration('PAYPLUG_ONEY_OPTIMIZED')) {
                // check if oney was elligible then return if not
                $is_elligible = $this->isOneyElligible($this->context->cart);
                if (!$is_elligible['result']) {
                    $data = array('result' => false, 'response' => $is_elligible['error']);
                    return Tools::jsonEncode($data);
                }

                // else check the cookie
                $payment_tab = $this->getPaymentDataCookie();

                if (!empty($payment_tab)) {
                    $options['oney_form'] = $payment_tab['oney_form'];
                } else {
                    $has_required_fields = $this->getOneyRequiredFields();
                    if (!empty($has_required_fields)) {
                        $this->setPaymentErrorsCookie(array('oney_required_field'));
                        $data = array('result' => false, 'response' => false);
                        return Tools::jsonEncode($data);
                    }
                }
            }
        }

        $payplug_method_name = $this->getCurrentPaymentMethod($id_card);
        $payplug_payment = new $payplug_method_name($id_card, $options);

        try {
            $result = $payplug_payment->create();
            $payment = $result['resource'];

            if (!$payment) {
                $messages = $this->catchErrorsFromApi($result['message']);
                $this->log_payment->info(
                    'Payment abort: ' . count($messages) > 1 ? json_encode($messages) : reset($messages)
                );
                $data = array(
                    'result' => false,
                    'response' => count($messages) > 1 ? $messages : reset($messages),
                );
                return Tools::jsonEncode($data);
            } elseif (!$payplug_payment->isValidPayment($payment)) {
                $this->log_payment->info('Payment abort: ' . $payment->failure->message);
                $data = array(
                    'result' => false,
                    'response' => $payment->failure->message,
                );
                return Tools::jsonEncode($data);
            }
        } catch (Exception $e) {
            $messages = $this->catchErrorsFromApi($e->__toString());
            $this->log_payment->info(
                'Payment abort: ' . count($messages) > 1 ? json_encode($messages) : reset($messages)
            );
            $data = array(
                'result' => false,
                'response' => count($messages) > 1 ? $messages : reset($messages),
            );
            return Tools::jsonEncode($data);
        }

        $payplug_payment->register($payment->id);

        switch ($payplug_payment->type) {
            case 'oneclick':
                $redirect = $payment->is_paid;
                if (!$redirect && $deferred) {
                    $redirect = (bool)$payment->authorization->authorized_at;
                }
                $data = array(
                    'result' => true,
                    'embedded' => true,
                    'redirect' => $redirect, // force `true` we are in 3DS 1
                    'return_url' => $redirect ?
                        $payplug_payment->payment_url['return'] : $payment->hosted_payment->payment_url,
                );
                break;
            case 'oney':
                $data = array(
                    'result' => 'new_card',
                    'embedded' => false,
                    'redirect' => true,
                    'return_url' => $payment->hosted_payment->payment_url,
                );
                break;
            case 'standard':
            case 'installment':
            default:
                $data = array(
                    'result' => 'new_card',
                    'embedded' => $this->getConfiguration('PAYPLUG_EMBEDDED_MODE') && !$this->isMobiledevice(),
                    'redirect' => false,
                    'return_url' => $payment->hosted_payment->payment_url,
                );
                break;
        }

        $this->log_payment->info('Payment valided');

        return Tools::jsonEncode($data);
    }

    /**
     * check if a payment for the same id cart is pending
     *
     * @param int $id_cart
     * @return bool
     */
    public function isPaymentPending($id_cart)
    {
        $current_time = strtotime(date('Y-m-d H:i:s'));
        $timeout_delay = 9;
        $req_payment_cart_exists = '
            SELECT *
            FROM ' . _DB_PREFIX_ . 'payplug_payment_cart ppc
            WHERE ppc.id_cart = ' . (int)$id_cart . '
            AND ppc.id_payment LIKE \'pending\'';
        $res_payment_cart_exists = Db::getInstance()->getRow($req_payment_cart_exists);
        if (!$res_payment_cart_exists) {
            return false;
        } elseif (($current_time - strtotime($res_payment_cart_exists['date_upd'])) >= $timeout_delay) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * Determine witch environnement is used
     *
     * @param PayplugPayment $payment
     * @return bool
     */
    public function saveCard($payment)
    {
        $payplug_card = new PayPlugCard();

        $id_customer = $payment->metadata['Client'];

        $payplug_card->brand = $payment->card->brand;
        $payplug_card->country = $payment->card->country ? $payment->card->country : '';
        $payplug_card->exp_month = $payment->card->exp_month;
        $payplug_card->exp_year = $payment->card->exp_year;
        $payplug_card->id_card = $payment->card->id;
        $payplug_card->id_customer = $id_customer;
        $payplug_card->last4 = $payment->card->last4;

        $payplug_card->metadata = $payment->card->metadata ? serialize($payment->card->metadata) : '';

        $_errors = $payplug_card->validateController();

        if ($_errors) {
            // @todo: log error while card creation
            return false;
        }

        return $payplug_card->save();
    }

    /**
     * @param array $params
     * @return string
     * @see Module::hookPayment()
     *
     */
    public function hookPayment($params)
    {
        if (!$this->isAllowed()) {
            return false;
        }

        $use_taxes = $this->getConfiguration('PS_TAX');
        $base_total_tax_inc = $params['cart']->getOrderTotal(true);
        $base_total_tax_exc = $params['cart']->getOrderTotal(false);

        if ($use_taxes) {
            $price2display = $base_total_tax_inc;
        } else {
            $price2display = $base_total_tax_exc;
        }

        if (!$this->getConfiguration('PAYPLUG_ONEY_OPTIMIZED')) {
            $paymentOptions = $this->buildPaymentOptions($params['cart']);
            $this->smarty->assign(array(
                'payplug_payment_options' => $paymentOptions,
                'spinner_url' => PayplugBackward::getHttpHost(true)
                    . __PS_BASE_URI__ . 'modules/payplug/views/img/admin/spinner.gif',
                'front_ajax_url' => PayplugBackward::getModuleLink($this->name, 'ajax', array(), true),
                'api_url' => $this->api_url,
                'price2display' => $price2display,
                'this_path' => $this->_path,
            ));

            return $this->display(__FILE__, 'payment_options_display.tpl');
        }

        $this->assignPaymentOptions($params['cart']);

        // Different tpl depending version
        if (version_compare(_PS_VERSION_, '1.6', '<')) {
            $this->smarty->assign(array(
                'version' => _PS_VERSION_,
            ));
            if (version_compare(_PS_VERSION_, '1.5', '<')) {
                return $this->display(__FILE__, './views/templates/hook/payment_1_5.tpl');
            } else {
                return $this->display(__FILE__, 'payment_1_5.tpl');
            }
        } else {
            if ($this->getConfiguration('PAYPLUG_ONEY_OPTIMIZED')) {
                $this->assignOneyPaymentOptions($params['cart']);
            }
            return $this->display(__FILE__, 'payment_1_6.tpl');
        }
    }

    public function buildPaymentOptions($cart)
    {
        $paymentAllowedOptions = $this->getAllowedPaymentOptions($cart);
        $paymentOptions = array();
        $payment_class = 'payplug';
        $logo_class = 'paymentLogo';
        $unified_mode = (bool)!$this->getConfiguration('PAYPLUG_ONEY_OPTIMIZED');
        $error = false;
        $is_oney_available = true;

        $current_lang = explode('-', $this->context->language->language_code);
        $current_lang = $current_lang[0];
        if (in_array($current_lang, array('it', 'en'))) {
            $img_lang = $current_lang;
        } else {
            $img_lang = 'default';
        }

        if ($this->getConfiguration('PAYPLUG_ONEY')) {
            // check if at least one carrier is available for this cart
            // get the available carrier
            $package_list = $cart->getPackageList();
            $carrier_ids = array();
            foreach ($package_list as $address) {
                foreach ($address as $package) {
                    $carrier_ids = array_merge($carrier_ids, $package['carrier_list']);
                }
            }

            if (Validate::isLoadedObject($cart) && $cart->id_address_invoice && $cart->id_address_delivery) {
                $is_elligible = $this->isOneyElligible($cart);
                $error = !$is_elligible['result'];
            } else {
                $id_currency = $this->context->currency->id;
                $amount = $cart->getOrderTotal(true, Cart::BOTH);
                $is_elligible = $this->isValidOneyAmount($amount, $id_currency);
                $error = !$is_elligible['result'];
            }

            $this->smarty->assign(array(
                'payplug_module_dir' => _PS_MODULE_DIR_,
                'payplug_oney' => true,
                'payplug_oney_required_field' => $this->displayOneyRequiredFields(),
                'payplug_oney_allowed' => $is_elligible['result'],
                'payplug_oney_error' => $is_elligible['error'],
                'payplug_oney_loading_msg' => $this->l('Loading')
            ));
        }

        $installment_mode = $this->getConfiguration('PAYPLUG_INST_MODE');
        $this->smarty->assign(array(
            'installment_mode' => $installment_mode,
        ));

        // One-click Payment
        if ($this->getConfiguration('PAYPLUG_ONE_CLICK')) {
            $payplug_card = new PayPlugCard();
            $payplug_cards = $payplug_card->getByCustomer($cart->id_customer, true);

            if ($unified_mode && false) {
                $paymentOptions[] = array(
                    'extra_classes' => $payment_class . ' '
                        . $logo_class . ' '
                        . $logo_class . '-'
                        . $img_lang,
                    'label' => $this->l('Credit card payment'),
                    'logo_url' => $this->_path . 'views/img/logos_schemes_' . $img_lang . '.png',
                    'payment_url' => PayplugBackward::getModuleLink(
                        $this->name,
                        'payment',
                        array('type' => 'oneclick', 'pc' => null),
                        true
                    ),
                    'tpl' => _PS_MODULE_DIR_ . 'payplug/views/templates/hook/unified_payment.tpl',
                );
                if (is_array($payplug_cards) && count($payplug_cards) > 0) {
                    foreach ($payplug_cards as $card) {
                        $paymentOptions[] = array(
                            'extra_classes' => $payment_class . ' '
                                . $logo_class . ' '
                                . $logo_class . '-'
                                . Tools::strtolower($card['brand']),
                            'label' => '**** **** **** '
                                . $card['last4'] . ' '
                                . $card['brand'] . ' '
                                . $this->l('Expiry date: ')
                                . $card['expiry_date'],
                            'logo_url' => $this->_path . 'views/img/' . Tools::strtolower($card['brand']) . '.png',
                            'payment_url' => PayplugBackward::getModuleLink(
                                $this->name,
                                'payment',
                                array('type' => 'oneclick', 'pc' => (int)$card['id_payplug_card']),
                                true
                            ),
                            'tpl' => _PS_MODULE_DIR_ . 'payplug/views/templates/hook/unified_payment.tpl',
                        );
                    }
                }
            } else {
                $paymentOptions[] = array(
                    'tpl' => _PS_MODULE_DIR_ . 'payplug/views/templates/hook/one_click_payment.tpl',
                );
                $this->smarty->assign(array(
                    'payplug_cards' => $payplug_cards,
                    'payment_controller_url' => PayplugBackward::getModuleLink($this->name, 'payment', array(), true),
                ));
            }
        } else {
            // Standard Payment
            $paymentOptions[] = array(
                'extra_classes' => $payment_class . ' ' . $logo_class . ' ' . $logo_class . '-' . $img_lang,
                'label' => $this->l('Pay with credit card'),
                'logo_url' => $this->_path . 'views/img/logos_schemes_' . $img_lang . '.png',
                'payment_url' => PayplugBackward::getModuleLink(
                    $this->name,
                    'payment',
                    array('type' => 'standard'),
                    true
                ),
                'tpl' => _PS_MODULE_DIR_ . 'payplug/views/templates/hook/standard_payment.tpl',
            );
        }

        // Oney
        if ($this->getConfiguration('PAYPLUG_ONEY') && $is_oney_available) {
            if ($unified_mode) {
                if ($error) {
                    switch ($is_elligible['error_type']) {
                        case 'invalid_addresses':
                            $err_label = $this->l('Available for France only');
                            break;
                        case 'invalid_amount_bottom':
                        case 'invalid_amount_top':
                            $err_label = $this->l('Between 100€ and 3000€ only');
                            break;
                        case 'invalid_carrier':
                            $err_label = $this->l('Unavailable for this shipping method');
                            break;
                        default:
                        case 'invalid_cart':
                            $err_label = $this->l('Your cart is unavailable');
                            break;
                    }
                }

                $paymentOptions[] = array(
                    'extra_classes' => $payment_class . ' '
                        . $logo_class . ' '
                        . $logo_class
                        . '-oney3x'
                        . ($error ? '-alt' : ''),
                    'label' => $error ? $err_label : $this->l('Pay by card in 3x with Oney'),
                    'logo_url' => $this->_path . 'views/img/oney/3x' . ($error ? '-alt' : '') . '.svg',
                    'payment_url' => PayplugBackward::getModuleLink(
                        $this->name,
                        'payment',
                        array('type' => 'oney', 'io' => 3),
                        true
                    ),
                    'tpl' => _PS_MODULE_DIR_ . 'payplug/views/templates/hook/unified_payment.tpl',
                );
                $paymentOptions[] = array(
                    'extra_classes' => $payment_class . ' '
                        . $logo_class . ' '
                        . $logo_class
                        . '-oney4x'
                        . ($error ? '-alt' : ''),
                    'label' => $error ? $err_label : $this->l('Pay by card in 4x with Oney'),
                    'logo_url' => $this->_path . 'views/img/oney/4x' . ($error ? '-alt' : '') . '.svg',
                    'payment_url' => PayplugBackward::getModuleLink(
                        $this->name,
                        'payment',
                        array('type' => 'oney', 'io' => 4),
                        true
                    ),
                    'tpl' => _PS_MODULE_DIR_ . 'payplug/views/templates/hook/unified_payment.tpl',
                );
            } else {
                $paymentOptions[] = array(
                    'tpl' => _PS_MODULE_DIR_ . 'payplug/views/templates/hook/oney_payment.tpl',
                );
            }
        }

        // Installments
        if ($paymentAllowedOptions['installment']) {
            if ($unified_mode && false) {
                $paymentOptions[] = array(
                    'extra_classes' => $payment_class
                        . ' '
                        . $logo_class . ' '
                        . $logo_class . '-'
                        . $img_lang . '-x'
                        . (int)$installment_mode,
                    'label' => $this->l('Pay by card in') . (int)$installment_mode . $this->l('installments'),
                    'logo_url' => $this->_path . 'views/img/logos_schemes_installment_'
                        . (int)$installment_mode . '_'
                        . $img_lang . '.png',
                    'payment_url' => PayplugBackward::getModuleLink(
                        $this->name,
                        'payment',
                        array('type' => 'installment', 'i' => (int)$installment_mode),
                        true
                    ),
                    'tpl' => _PS_MODULE_DIR_ . 'payplug/views/templates/hook/unified_payment.tpl',
                );
            } else {
                $this->smarty->assign(array(
                    'installment_controller_url' => PayplugBackward::getModuleLink(
                        $this->name,
                        'payment',
                        array('i' => 1),
                        true
                    ),
                ));
                $paymentOptions[] = array(
                    'tpl' => _PS_MODULE_DIR_ . 'payplug/views/templates/hook/installment_payment.tpl',
                );
            }
        }

        return $paymentOptions;
    }

    public function assignSimplifiedPaymentOptions($cart)
    {
        $allow_methods = $this->getAllowedPaymentOptions($cart);
    }

    public function getAllowedPaymentOptions($cart)
    {
        $options = array(
            'standard' => false,
            'oneclick' => false,
            'installment' => false,
            'oney' => false,
        );

        if (!$this->active ||
            !$this->getConfiguration('PAYPLUG_SHOW') ||
            !$this->checkCurrency($cart) ||
            !$this->checkAmount($cart)) {
            return $options;
        }

        // check if installment allowed
        $installment = $this->getConfiguration('PAYPLUG_INST');
        $installment_min_amount = $this->getConfiguration('PAYPLUG_INST_MIN_AMOUNT');
        $order_total = $cart->getOrderTotal(true);
        $installment = $installment && $order_total >= $installment_min_amount;

        // check if one click allowed
        $one_click = $this->getConfiguration('PAYPLUG_ONE_CLICK');
        $payplug_card = new PayPlugCard();
        $payplug_cards = $payplug_card->getByCustomer($cart->id_customer, true);
        $one_click = (bool)($one_click && !empty($payplug_cards));

        // check if oney is allowed
        $oney = $this->getConfiguration('PAYPLUG_ONEY') && $this->checkVersion();

        $options = array(
            'standard' => true,
            'oneclick' => $one_click,
            'installment' => $installment,
            'oney' => $oney,
        );

        return $options;
    }

    /**
     * @param $cart
     */
    public function assignPaymentOptions($cart)
    {
        $one_click = $this->getConfiguration('PAYPLUG_ONE_CLICK');
        $installment = $this->getConfiguration('PAYPLUG_INST');
        $installment_mode = $this->getConfiguration('PAYPLUG_INST_MODE');
        $installment_min_amount = $this->getConfiguration('PAYPLUG_INST_MIN_AMOUNT');

        if (!$this->checkCurrency($cart) ||
            !$this->checkAmount($cart)) {
            return false;
        }

        $path_ssl = Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__ . 'modules/' . $this->name . '/';

        $payplug_card = new PayPlugCard();
        $payplug_cards = $payplug_card->getByCustomer($cart->id_customer, true);

        $use_taxes = $this->getConfiguration('PS_TAX');
        $base_total_tax_inc = $cart->getOrderTotal(true);
        $base_total_tax_exc = $cart->getOrderTotal(false);

        if ($base_total_tax_inc < $installment_min_amount) {
            $installment = 0;
        }

        if ($use_taxes) {
            $price2display = $base_total_tax_inc;
        } else {
            $price2display = $base_total_tax_exc;
        }

        $this->smarty->assign(array(
            'this_path' => $this->_path,
            'this_path_ssl' => $path_ssl,
            'iso_lang' => $this->context->language->iso_code,
            'price2display' => $price2display,
        ));

        $front_ajax_url = PayplugBackward::getModuleLink($this->name, 'ajax', array(), true);

        $this->smarty->assign(array(
            'front_ajax_url' => $front_ajax_url,
            'api_url' => $this->api_url
        ));

        if (!empty($payplug_cards) && $one_click == 1) {
            $this->smarty->assign(array(
                'payplug_cards' => $payplug_cards,
                'payplug_one_click' => 1,
            ));
        }

        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            $payment_url = 'order.php?step=3';
        } else {
            $payment_url = 'index.php?controller=order&step=3';
        }

        $payment_controller_url = PayplugBackward::getModuleLink($this->name, 'payment', array(), true);
        $installment_controller_url = PayplugBackward::getModuleLink($this->name, 'payment', array('i' => 1), true);
        $current_lang = explode('-', $this->context->language->language_code);
        $current_lang = $current_lang[0];
        if (in_array($current_lang, array('it', 'en'))) {
            $img_lang = $current_lang;
        } else {
            $img_lang = 'default';
        }

        $this->smarty->assign(array(
            'spinner_url' => PayplugBackward::getHttpHost(true)
                . __PS_BASE_URI__ . 'modules/payplug/views/img/admin/spinner.gif',
            'payment_url' => $payment_url,
            'payment_controller_url' => $payment_controller_url,
            'installment_controller_url' => $installment_controller_url,
            'img_lang' => $img_lang,
            'payplug_installment' => $installment,
            'installment_mode' => $installment_mode,
        ));
    }

    /**
     * @param array $params
     * @return string
     * @see Module::hookPaymentReturn()
     *
     */
    public function hookPaymentReturn($params)
    {
        if (!$this->isAllowed()) {
            return false;
        }

        $order_id = Tools::getValue('id_order');
        $order = new Order($order_id);

        // Check order state to display appropriate message
        $state = null;
        if (isset($order->current_state)
            && $order->current_state == $this->getConfiguration('PAYPLUG_ORDER_STATE_PENDING')
        ) {
            $state = 'pending';
        } elseif (isset($order->current_state)
            && $order->current_state == $this->getConfiguration('PAYPLUG_ORDER_STATE_PAID')
        ) {
            $state = 'paid';
        } elseif (isset($order->current_state)
            && $order->current_state == $this->getConfiguration('PAYPLUG_ORDER_STATE_PENDING_TEST')
        ) {
            $state = 'pending_test';
        } elseif (isset($order->current_state)
            && $order->current_state == $this->getConfiguration('PAYPLUG_ORDER_STATE_PAID_TEST')
        ) {
            $state = 'paid_test';
        }

        $this->assignForVersion('state', $state);
        // Get order information for display
        $total_paid = number_format($order->total_paid, 2, ',', '');
        $context = array('totalPaid' => $total_paid);
        if (isset($order->reference)) {
            $context['reference'] = $order->reference;
        }
        $this->assignForVersion($context);
        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            return $this->display(__FILE__, './views/templates/hook/confirmation.tpl');
        } else {
            return $this->display(__FILE__, 'confirmation.tpl');
        }
    }

    /**
     * @param array $params
     * @return string
     * @see Module::hookHeader()
     *
     */
    public function hookHeader($params)
    {
        if (!$this->isAllowed()) {
            return false;
        }

        if (Tools::getValue('error')) {
            Media::addJsDef(array(
                'payment_errors' => true,
            ));
        }

        if (version_compare(_PS_VERSION_, '1.6', '>=')) {
            $this->addCSSRC(__PS_BASE_URI__ . 'modules/payplug/views/css/front_1_6.css');
            $this->addJsRC(__PS_BASE_URI__ . 'modules/payplug/views/js/front.js');
            Media::addJsDef(array(
                'payplug_ajax_url' => PayplugBackward::getModuleLink($this->name, 'ajax', array(), true),
            ));
            $this->assignOneyJSVar();
        } elseif (version_compare(_PS_VERSION_, '1.5', '>=')) {
            $this->addCSSRC(__PS_BASE_URI__ . 'modules/payplug/views/css/front_1_5.css');
            $this->addJsRC(__PS_BASE_URI__ . 'modules/payplug/views/js/front.js');
        } else {
            Tools::addCSS(__PS_BASE_URI__ . 'modules/payplug/views/css/front_1_4.css');
            Tools::addJS(__PS_BASE_URI__ . 'modules/payplug/views/js/front.js');
        }
    }

    /**
     * Make a refund
     *
     * @param string $pay_id
     * @param int $amount
     * @param string $metadata
     * @param string $pay_mode
     * @return string
     */
    public function makeRefund($pay_id, $amount, $metadata, $pay_mode = 'LIVE', $inst_id = null)
    {
        $this->initializeApi($pay_mode != 'LIVE');

        if ($pay_id == null) {
            if ($inst_id != null) {
                try {
                    $installment = $this->retrieveInstallment($inst_id);

                    if (!$installment) {
                        return false;
                    }

                    if (isset($installment->schedule)) {
                        $total_amount = $amount;
                        $refund_to_go = array();
                        $truly_refundable_amount = 0;
                        foreach ($installment->schedule as $schedule) {
                            if (!empty($schedule->payment_ids)) {
                                foreach ($schedule->payment_ids as $p_id) {
                                    $p = \Payplug\Payment::retrieve($p_id);
                                    if ($p->is_paid && !$p->is_refunded && $amount > 0) {
                                        $amount_refundable = $p->amount - $p->amount_refunded;
                                        $truly_refundable_amount += $amount_refundable;
                                        if ($truly_refundable_amount < 10) {
                                            continue;
                                        } elseif ($amount >= $amount_refundable) {
                                            $data = array(
                                                'amount' => $amount_refundable,
                                                'metadata' => $metadata
                                            );
                                            $amount -= $amount_refundable;
                                        } else {
                                            $data = array(
                                                'amount' => $amount,
                                                'metadata' => $metadata
                                            );
                                            $amount = 0;
                                        }
                                        $refund_to_go[] = array('id' => $p_id, 'data' => $data);
                                    }
                                }
                            }
                        }
                        if ($truly_refundable_amount < $total_amount) {
                            return ('error');
                        }
                        if (!empty($refund_to_go)) {
                            foreach ($refund_to_go as $refnd) {
                                try {
                                    $refund = \Payplug\Refund::create($refnd['id'], $refnd['data']);
                                } catch (Exception $e) {
                                    return ('error');
                                }
                            }
                        }
                    }
                } catch (Exception $e) {
                    return ('error');
                }
                $this->updatePayplugInstallment($installment);
            } else {
                return ('error');
            }
        } else {
            $data = array(
                'amount' => (int)$amount,
                'metadata' => $metadata
            );

            try {
                $refund = \Payplug\Refund::create($pay_id, $data);
            } catch (Exception $e) {
                return ('error');
            }
        }

        return $refund;
    }

    /**
     * @param bool $force_all
     * @return bool
     * @see Module::enable()
     *
     */
    public function enable($force_all = false)
    {
        if (version_compare(_PS_VERSION_, '1.4', '>=')) {
            return parent::enable($force_all);
        }

        $req_enable = '
            UPDATE `' . _DB_PREFIX_ . 'module`
            SET `active`= 1
            WHERE `name` = \'' . pSQL($this->name) . '\'';

        $res_enable = Db::getInstance()->execute($req_enable);
        if (!$res_enable) {
            return false;
        }

        return true;
    }

    /**
     * @param bool $force_all
     * @return bool
     * @see Module::disable()
     *
     */
    public function disable($force_all = false)
    {
        $this->setConfiguration('PAYPLUG_SHOW', 0);
        if (version_compare(_PS_VERSION_, '1.4', '>=')) {
            parent::disable($force_all);
        }

        $req_disable = '
            UPDATE `' . _DB_PREFIX_ . 'module`
            SET `active`= 0
            WHERE `name` = \'' . pSQL($this->name) . '\'';

        $res_disable = Db::getInstance()->execute($req_disable);
        if (!$res_disable) {
            return false;
        }

        return true;
    }

    /**
     * submit password
     *
     * @param string $pwd
     * @return string
     */
    public function submitPopinPwd($pwd)
    {
        $email = $this->getConfiguration('PAYPLUG_EMAIL');
        $connected = $this->login($email, $pwd);
        $use_live_mode = false;
        if ($connected) {
            if ($valid_key = $this->getConfiguration('PAYPLUG_LIVE_API_KEY')) {
                $use_live_mode = true;
                $permissions = $this->getAccount($valid_key);
                $can_save_cards = $permissions['can_save_cards'];
                $can_create_installment_plan = $permissions['can_create_installment_plan'];
                $can_create_deferred_payment = $permissions['can_create_deferred_payment'];
                $can_use_oney = $permissions['can_use_oney'];
            }
        } else {
            die(Tools::jsonEncode(array('content' => 'wrong_pwd')));
        }
        // todo: ajouter le can_create_deferred_payment & oney si besoin
        if (!$use_live_mode) {
            die(Tools::jsonEncode(array('content' => 'activate')));
        } elseif ($can_save_cards && $can_create_installment_plan) {
            die(Tools::jsonEncode(array('content' => 'live_ok')));
        } elseif ($can_save_cards && !$can_create_installment_plan) {
            die(Tools::jsonEncode(array('content' => 'live_ok_no_inst')));
        } elseif (!$can_save_cards && $can_create_installment_plan) {
            die(Tools::jsonEncode(array('content' => 'live_ok_no_oneclick')));
        } else {
            die(Tools::jsonEncode(array('content' => 'live_ok_not_premium')));
        }
    }

    /**
     * Check if account is premium
     *
     * @param string $api_key
     * @return bool
     */
    public function checkPremium($api_key = null)
    {
        if ($api_key == null) {
            $api_key = $this->getAPIKey();
        }
        $permissions = $this->getAccount($api_key);

        return $permissions;
    }

    /**
     * Determine wich API key to use
     *
     * @return string
     */
    public function getAPIKey()
    {
        if ($this->getConfiguration('PAYPLUG_SANDBOX_MODE')) {
            return $this->getConfiguration('PAYPLUG_TEST_API_KEY');
        } else {
            return $this->getConfiguration('PAYPLUG_LIVE_API_KEY');
        }
    }

    /**
     * @param array $params
     * @return string
     * @see Module::hookAdminOrder()
     *
     */
    public function hookAdminOrder($params)
    {
        if (!$this->active || !$this->getConfiguration('PAYPLUG_SHOW')) {
            return false;
        }

        $this->html = '';
        $order = new Order($params['id_order']);
        if (!Validate::isLoadedObject($order)) {
            return false;
        }

        if ($order->module != $this->name) {
            return false;
        }


        $show_popin = false;
        $display_refund = false;
        $refund_delay_oney = false;
        $show_menu_refunded = false;
        $show_menu_update = false;
        $show_menu_installment = false;
        $show_menu_payment = false;
        $pay_error = '';
        $amount_refunded_payplug = 0;
        $amount_available = 0;
        $id_new_order_state = 0;
        $id_pending_order_state = 0;
        $pay_mode = '';

        $admin_ajax_url = $this->getAdminAjaxUrl('AdminModules', $params['id_order']);
        $amount_refunded_presta = $this->getTotalRefunded($order->id);

        if ($inst_id = $this->getPayplugInstallmentCart($order->id_cart)) {
            $payment_list = array();
            if (!$inst_id || empty($inst_id) || !$installment = $this->retrieveInstallment($inst_id)) {
                if ($this->getConfiguration('PAYPLUG_SANDBOX_MODE')) {
                    $this->initializeApi(false);
                    if (empty($inst_id) || !$installment = $this->retrieveInstallment($inst_id)) {
                        $this->initializeApi(true);
                        return false;
                    }
                } else {
                    $this->initializeApi(true);
                    if (empty($inst_id) || !$installment = $this->retrieveInstallment($inst_id)) {
                        $this->initializeApi(false);
                        return false;
                    }
                }
            }
            $pay_mode = $installment->is_live ? $this->l('LIVE') : $this->l('TEST');

            $payment_list_new = array();
            foreach ($installment->schedule as $schedule) {
                if ($schedule->payment_ids != null) {
                    foreach ($schedule->payment_ids as $pay_id) {
                        $p = $this->retrievePayment($pay_id);
                        $payment_list_new[] = $this->buildPaymentDetails($p);
                        if (!$p->is_paid) {
                            $amount_refunded_payplug += 0;
                            $amount_available += 0;
                        } elseif ($p->is_refunded) {
                            $amount_refunded_payplug += ($p->amount_refunded) / 100;
                            $amount_available += ($p->amount - $p->amount_refunded) / 100;
                        } elseif ($p->amount_refunded > 0) {
                            $amount_refunded_payplug += ($p->amount_refunded) / 100;
                            $amount_refundable_payment = ($p->amount - $p->amount_refunded);
                            $amount_available += (
                            $amount_refundable_payment >= 10 ? $amount_refundable_payment / 100 : 0
                            );
                        } else {
                            $amount_available += ($p->amount >= 10 ? $p->amount / 100 : 0);
                        }

                        if ($amount_available > 0) {
                            $display_refund = true;
                        }

                        if ($p->amount_refunded > 0) {
                            $show_menu_refunded = true;
                        }
                    }
                } else {
                    $payment_list_new[] = array(
                        'id' => null,
                        'status' => $inst_status = $installment->is_active ?
                            $this->payment_status[6] : $this->payment_status[7],
                        'amount' => (int)$schedule->amount / 100,
                        'card_brand' => null,
                        'card_mask' => null,
                        'tds' => null,
                        'card_date' => null,
                        'mode' => null,
                        'authorization' => null,
                        'status_class' => $inst_status = $installment->is_active ? 'pp_neutral' : 'pp_error',
                        'date' => date('d/m/Y', strtotime($schedule->date)),
                    );
                }
            }

            $state_addons = ($installment->is_live ? '' : '_TEST');
            $paid_state = $this->getConfiguration('PAYPLUG_ORDER_STATE_PAID' . $state_addons);
            $inst_state = $this->getConfiguration('PAYPLUG_ORDER_STATE_PAID' . $state_addons);

            if ($installment->is_fully_paid && $order->getCurrentState() == $inst_state) {
                $new_order_state = $paid_state;
                $order_history = new OrderHistory();
                $order_history->id_order = $order->id;
                $order_history->changeIdOrderState($new_order_state, $order->id, true);
                $order_history->save();
            }

            $id_currency = Currency::getIdByIsoCode($installment->currency);
            $show_menu_installment = true;
            $inst_status = $installment->is_active ? $this->l('ongoing') : (
            $installment->is_fully_paid ? $this->l('paid') : $this->l('suspended')
            );
            $inst_aborted = !$installment->is_active;
            $ppInstallment = new PPPaymentInstallment($installment->id);
            $instPaymentOne = $ppInstallment->getFirstPayment();
            $inst_can_be_aborted = !($inst_aborted || ($instPaymentOne->isDeferred() && !$instPaymentOne->isPaid()));
            $inst_paid = $installment->is_fully_paid;
            $this->context->smarty->assign(array(
                'inst_id' => $inst_id,
                'inst_status' => $inst_status,
                'inst_aborted' => $inst_aborted,
                'inst_paid' => $inst_paid,
                'payment_list' => $payment_list,
                'payment_list_new' => $payment_list_new,
                'inst_can_be_aborted' => $inst_can_be_aborted,
            ));

            $sandbox = !$installment->is_live;

            $id_new_order_state = $this->getConfiguration('PAYPLUG_ORDER_STATE_REFUND' . ($sandbox ? '_TEST' : ''));

            $this->updatePayplugInstallment($installment);
        } else {
            if (!$pay_id = $this->isTransactionPending($order->id_cart)) {
                $pay_id = $this->getPayplugOrderPayment($order->id);

                // if order created before upgrade 2.28.0
                if (!$pay_id) {
                    $payments = $order->getOrderPaymentCollection();
                    if (count($payments->getResults()) > 1 || !$payments->getFirst()) {
                        return false;
                    } else {
                        $pay_id = $payments->getFirst()->transaction_id;
                    }
                }
            }

            if (!$pay_id || empty($pay_id) || !$payment = $this->retrievePayment($pay_id)) {
                if ($this->getConfiguration('PAYPLUG_SANDBOX_MODE')) {
                    $this->initializeApi(false);
                    if (empty($pay_id) || !$payment = $this->retrievePayment($pay_id)) {
                        $this->initializeApi(true);
                        return false;
                    }
                } else {
                    $this->initializeApi(true);
                    if (empty($pay_id) || !$payment = $this->retrievePayment($pay_id)) {
                        $this->initializeApi(false);
                        return false;
                    }
                }
            }

            $state_addons = ($payment->is_live ? '' : '_TEST');
            $paid_state = $this->getConfiguration('PAYPLUG_ORDER_STATE_PAID' . $state_addons);
            $oney_state = $this->getConfiguration('PAYPLUG_ORDER_STATE_ONEY_PG' . $state_addons);
            $cancelled_state = $this->getConfiguration('PS_OS_CANCELED');

            if (isset($payment->payment_method) && isset($payment->payment_method['type'])) {
                if (in_array($payment->payment_method['type'], array('oney_x3_with_fees', 'oney_x4_with_fees'))) {
                    $refund_delay_oney = true;
                    $refund_list = \Payplug\Refund::listRefunds($payment);
                    $lastest_operation = 0;
                    if (!empty($refund_list)) {
                        $lastest_operation = end($refund_list)->created_at;
                    } elseif ($payment->is_paid) {
                        $lastest_operation = $payment->paid_at;
                    }
                    if (time() > ($lastest_operation + 172800)) {
                        $refund_delay_oney = false;
                    }
                    if ($order->getCurrentState() == $oney_state && $payment->is_paid) {
                        $new_order_state = $paid_state;
                        $order_history = new OrderHistory();
                        $order_history->id_order = $order->id;
                        $order_history->changeIdOrderState($new_order_state, $order->id, true);
                        $order_history->save();
                    } elseif ($order->getCurrentState() == $oney_state
                        && isset($payment->failure)
                        && $payment->failure !== null
                    ) {
                        $new_order_state = $cancelled_state;
                        $order_history = new OrderHistory();
                        $order_history->id_order = $order->id;
                        $order_history->changeIdOrderState($new_order_state, $order->id, true);
                        $order_history->save();
                    }
                }
            }

            $single_payment = $this->buildPaymentDetails($payment);
            $amount_refunded_payplug = ($payment->amount_refunded) / 100;
            $amount_available_payment = ($payment->amount - $payment->amount_refunded);
            $amount_available = ($amount_available_payment >= 10 ? $amount_available_payment / 100 : 0);

            $id_currency = Currency::getIdByIsoCode($payment->currency);

            $sandbox = !$payment->is_live;


            $id_new_order_state = $this->getConfiguration('PAYPLUG_ORDER_STATE_REFUND' . ($sandbox ? '_TEST' : ''));
            $id_pending_order_state = $this->getConfiguration(
                'PAYPLUG_ORDER_STATE_PENDING' . ($sandbox ? '_TEST' : '')
            );
            $current_state = $order->getCurrentState();

            if (!$payment->is_paid) {
                if (isset($payment->failure) && isset($payment->failure->message)) {
                    $pay_error = '(' . $payment->failure->message . ')';
                } else {
                    $pay_error = '';
                }
                $display_refund = false;
                if ($current_state != 0 && $current_state == $id_pending_order_state) {
                    $show_menu_update = true;
                }
            } elseif (($payment->amount_refunded > 0 || $amount_refunded_presta > 0) && !$payment->is_refunded) {
                $display_refund = true;
            } elseif ($payment->is_refunded) {
                $show_menu_refunded = true;
                $display_refund = false;
            } else {
                $display_refund = true;
            }

            $conf = Tools::getValue('conf');
            if (($conf == 24 && version_compare(_PS_VERSION_, '1.5', '<'))
                || (($conf == 30 || $conf == 31)
                    && version_compare(_PS_VERSION_, '1.5', '>=')
                )
            ) {
                $show_popin = true;

                $admin_ajax_url = $this->getAdminAjaxUrl('AdminModules', $params['id_order']);

                $this->html .= '
                <a class="pp_admin_ajax_url" href="' . $admin_ajax_url . '"></a>
            ';
            }

            if ($show_popin && $display_refund) {
                if (version_compare(_PS_VERSION_, '1.5', '<')) {
                    $this->html .= '<script type="text/javascript" src="' . __PS_BASE_URI__
                        . 'modules/payplug/views/js/admin_order_popin.js"></script>';
                } else {
                    $this->addJsRC(__PS_BASE_URI__ . 'modules/payplug/views/js/admin_order_popin.js');
                }
            }

            $pay_status = $payment->is_paid ? $this->l('PAID') : $this->l('NOT PAID');
            if ($payment->is_refunded) {
                $pay_status = $this->l('REFUNDED');
            } elseif ($payment->amount_refunded > 0) {
                $pay_status = $this->l('PARTIALLY REFUNDED');
            }
            $pay_amount = $payment->amount / 100;
            $pay_date = date('d/m/Y H:i', $payment->created_at);
            if ($payment->card->brand != '') {
                $pay_brand = $payment->card->brand;
            } else {
                $pay_brand = $this->l('Unavailable in test mode');
            }
            if ($payment->card->country != '') {
                $pay_brand .= ' ' . $this->l('Card') . ' (' . $payment->card->country . ')';
            }
            if ($payment->card->last4 != '') {
                $pay_card_mask = '**** **** **** ' . $payment->card->last4;
            } else {
                $pay_card_mask = $this->l('Unavailable in test mode');
            }

            //Deferred payment does'nt display 3DS option before capture so we have to consider it null
            if ($payment->is_3ds !== null) {
                $pay_tds = $payment->is_3ds ? $this->l('YES') : $this->l('NO');
                $this->context->smarty->assign(array('pay_tds' => $pay_tds));
            }

            //$pay_fees = '';
            $pay_mode = $payment->is_live ? $this->l('LIVE') : $this->l('TEST');
            //$pay_card_date = $payment->card->exp_month.'/'.date('y', strtotime($payment->card->exp_year));

            if ($payment->card->exp_month === null) {
                $pay_card_date = $this->l('Unavailable in test mode');
            } else {
                $pay_card_date = date(
                    'm/y',
                    strtotime('01.' . $payment->card->exp_month . '.' . $payment->card->exp_year)
                );
            }

            $show_menu_payment = true;

            $this->context->smarty->assign(array(
                'pay_id' => $pay_id,
                'pay_status' => $pay_status,
                'pay_amount' => $pay_amount,
                'pay_date' => $pay_date,
                'pay_brand' => $pay_brand,
                'pay_card_mask' => $pay_card_mask,
                'pay_card_date' => $pay_card_date,
                'pay_error' => $pay_error,
            ));
        }

        $currency = new Currency($id_currency);
        if (!Validate::isLoadedObject($currency)) {
            return false;
        }

        $amount_suggested = (min($amount_refunded_presta, $amount_available) - $amount_refunded_payplug);
        $amount_suggested = number_format((float)$amount_suggested, 2);
        if ($amount_suggested < 0) {
            $amount_suggested = 0;
        }

        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            $version = 1.4;
        } elseif (version_compare(_PS_VERSION_, '1.6', '<')) {
            $version = 1.5;
        } elseif (version_compare(_PS_VERSION_, '1.6', '>=')) {
            $version = 1.6;
        }

        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            $this->html .= '<script type="text/javascript" src="' . __PS_BASE_URI__
                . 'modules/payplug/views/js/admin_order.js"></script>';
            $this->html .= '<link type="text/css" rel="stylesheet" href="' . __PS_BASE_URI__
                . 'modules/payplug/views/css/admin_order.css" />';
        } else {
            $this->addJsRC(__PS_BASE_URI__ . 'modules/payplug/views/js/admin_order.js');
            $this->addCSSRC(__PS_BASE_URI__ . 'modules/payplug/views/css/admin_order.css');
        }

        if ($display_refund) {
            $this->context->smarty->assign(array(
                'order' => $order,
                'amount_refunded_payplug' => $amount_refunded_payplug,
                'amount_available' => $amount_available,
                'amount_refunded_presta' => $amount_refunded_presta,
                'currency' => $currency,
                'amount_suggested' => $amount_suggested,
                'id_new_order_state' => $id_new_order_state,
            ));
        } elseif ($show_menu_refunded) {
            $this->context->smarty->assign(array(
                'amount_refunded_payplug' => $amount_refunded_payplug,
                'currency' => $currency,
            ));
        } elseif ($show_menu_update) {
            $this->context->smarty->assign(array(
                'admin_ajax_url' => $admin_ajax_url,
                'order' => $order,
            ));
        }

        $display_single_payment = $show_menu_payment;
        $this->context->smarty->assign(array(
            'version' => $version,
            'logo_url' => __PS_BASE_URI__ . 'modules/payplug/views/img/logo_payplug.png',
            'admin_ajax_url' => $admin_ajax_url,
            'display_single_payment' => $display_single_payment,
            'display_refund' => $display_refund,
            'refund_delay_oney' => $refund_delay_oney,
            'show_menu_payment' => $show_menu_payment,
            'show_menu_refunded' => $show_menu_refunded,
            'show_menu_update' => $show_menu_update,
            'show_menu_installment' => $show_menu_installment,
            'payplug_module_dir' => _PS_MODULE_DIR_,
            'pay_mode' => $pay_mode,
            'order' => $order,
        ));

        if ($display_single_payment) {
            $this->context->smarty->assign(array(
                'single_payment' => $single_payment,
            ));
        }

        $this->html .= $this->fetchTemplateRC('views/templates/admin/admin_order.tpl');
        return $this->html;
    }

    /**
     * Generate refund form
     *
     * @param int $amount_refunded_payplug
     * @param int $amount_available
     * @return string
     */
    public function getRefundData($amount_refunded_payplug, $amount_available)
    {
        $this->context->smarty->assign(array(
            'amount_refunded_payplug' => $amount_refunded_payplug,
            'amount_available' => $amount_available,
        ));

        $this->html = $this->fetchTemplateRC('views/templates/admin/admin_order_refund_data.tpl');

        return $this->html;
    }

    /**
     * Retrieve payment informations
     *
     * @param string $pay_id
     * @return PayplugPayment
     */
    public function retrievePayment($pay_id)
    {
        try {
            $payment = \Payplug\Payment::retrieve($pay_id);
        } catch (Exception $e) {
            return false;
        }

        return $payment;
    }

    /**
     * Retrieve payment informations
     *
     * @param string $inst_id
     * @return PayplugInstallment
     */
    public function retrieveInstallment($inst_id)
    {
        if (!$inst_id) {
            return false;
        }

        try {
            $installment = \Payplug\InstallmentPlan::retrieve($inst_id);
            if (!$installment) {
                return false;
            }
        } catch (Exception $e) {
            return false;
        }
        return $installment;
    }

    /**
     * Get total amount already refunded
     *
     * @param int $id_order
     * @return int
     */
    public function getTotalRefunded($id_order)
    {
        $order = new Order($id_order);
        if (!Validate::isLoadedObject($order)) {
            return false;
        } else {
            $amount_refunded_presta = 0;
            $flag_shipping_refunded = false;
            if (version_compare(_PS_VERSION_, '1.6', '<')) {
                $products = $order->getProducts(false, false, false);
                if (isset($products) && !empty($products) && sizeof($products)) {
                    foreach ($products as $product) {
                        $amount_refunded_presta += (
                            $product['product_quantity_refunded'] * $product['product_price_wt']
                        );
                    }
                }
                if ($amount_refunded_presta > 0) {
                    $amount_refunded_presta += $order->total_shipping;
                }
            } else {
                $order_slips = OrderSlip::getOrdersSlip($order->id_customer, $order->id);
                if (isset($order_slips) && !empty($order_slips) && sizeof($order_slips)) {
                    foreach ($order_slips as $order_slip) {
                        $amount_refunded_presta += $order_slip['amount'];
                        if (!$flag_shipping_refunded && $order_slip['shipping_cost'] == 1) {
                            $amount_refunded_presta += $order_slip['shipping_cost_amount'];
                            $flag_shipping_refunded = true;
                        }
                    }
                }
            }
            return $amount_refunded_presta;
        }
    }

    /**
     * Add Order Payment
     *
     * @param int $id_order
     * @param string $id_payment
     * @return bool
     */
    public function addPayplugOrderPayment($id_order, $id_payment)
    {
        $req_order_payment = '
            INSERT INTO ' . _DB_PREFIX_ . 'payplug_order_payment (id_order, id_payment) 
            VALUE (' . (int)$id_order . ',\'' . pSQL($id_payment) . '\')';
        $res_order_payment = Db::getInstance()->execute($req_order_payment);

        return $res_order_payment;
    }

    /**
     * get order payment
     *
     * @param int $id_order
     * @return bool
     */
    public function getPayplugOrderPayment($id_order)
    {
        $sql = 'SELECT `id_payment` 
                FROM `' . _DB_PREFIX_ . 'payplug_order_payment`
                WHERE `id_order` = ' . (int)$id_order;

        return Db::getInstance()->getValue($sql);
    }

    /**
     * @description
     * get all order payment for given id order
     *
     * @param int $id_order
     * @return array
     */
    public function getPayplugOrderPayments($id_order)
    {
        $sql = 'SELECT * 
                FROM `' . _DB_PREFIX_ . 'payplug_order_payment` 
                WHERE `id_order` = ' . (int)$id_order;

        return Db::getInstance()->executeS($sql);
    }

    /**
     * get cart installment
     *
     * @param int $id_order
     * @return bool
     */
    public function getPayplugInstallmentCart($id_cart)
    {
        $req_cart_installment = '
            SELECT pic.id_installment 
            FROM ' . _DB_PREFIX_ . 'payplug_installment_cart pic
            WHERE pic.id_cart = ' . (int)$id_cart;
        $res_cart_installment = Db::getInstance()->getValue($req_cart_installment);

        return $res_cart_installment;
    }

    /**
     * Check amount to refund
     *
     * @param int $amount
     * @return string
     */
    public function checkAmountToRefund($amount)
    {
        $amount = str_replace(',', '.', $amount);
        return is_numeric($amount) && ($amount >= 0.1);
    }

    public function adminAjaxController()
    {
        if (Tools::getValue('_ajax')) {
            if (Tools::getValue('en') && !$this->setConfiguration('PAYPLUG_SHOW')) {
                $this->setConfiguration('PAYPLUG_SHOW', 1);
                $this->enable();
                die(true);
            }
            if (Tools::getIsset('en') && !Tools::getValue('en') && $this->getConfiguration('PAYPLUG_SHOW')) {
                $this->setConfiguration('PAYPLUG_SHOW', 0);
                die(true);
            }
            if (Tools::getIsset('db')) {
                die(true);
            }
            if (Tools::getValue('popin')) {
                $args = null;
                if (Tools::getValue('type') == 'confirm') {
                    $keys = array(
                        'sandbox',
                        'embedded',
                        'one_click',
                        'oney',
                        'installment',
                        'activate',
                        'deferred',
                    );
                    $args = array();
                    foreach ($keys as $key) {
                        $args[$key] = Tools::getValue($key);
                    }
                }
                $this->displayPopin(Tools::getValue('type'), $args);
            }
            if (Tools::getValue('submit') == 'submitPopin_pwd') {
                $this->submitPopinPwd(Tools::getValue('pwd'));
            }
            if (Tools::getValue('submit') == 'submitPopin_confirm') {
                die(Tools::jsonEncode(array('content' => 'confirm_ok')));
            }
            if (Tools::getValue('submit') == 'submitPopin_confirm_a') {
                die(Tools::jsonEncode(array('content' => 'confirm_ok_activate')));
            }
            if (Tools::getValue('submit') == 'submitPopin_desactivate') {
                die(Tools::jsonEncode(array('content' => 'confirm_ok_desactivate')));
            }
            if (Tools::getValue('submit') == 'submitPopin_abort') {
                $inst_id = Tools::getValue('inst_id');
                $id_order = Tools::getValue('id_order');
                $pay_mode = Tools::getValue('pay_mode');

                $this->initializeApi($pay_mode != 'LIVE');
                try {
                    $abort = \Payplug\InstallmentPlan::abort($inst_id);
                } catch (Exception $e) {
                    die(Tools::jsonEncode(array(
                        'status' => 'error',
                        'data' => $this->l('Cannot abort installment.')
                    )));
                }

                if ($abort == 'error') {
                    die(Tools::jsonEncode(array(
                        'status' => 'error',
                        'data' => $this->l('Cannot abort installment.')
                    )));
                } else {
                    $installment = $this->retrieveInstallment($inst_id);
                    $new_state = $this->getConfiguration('PS_OS_CANCELED');

                    $order = new Order($id_order);
                    if (Validate::isLoadedObject($order)) {
                        $current_state = $order->getCurrentState();
                        if ($current_state != 0 && $current_state != $new_state) {
                            $history = new OrderHistory();
                            $history->id_order = $order->id;
                            $history->changeIdOrderState($new_state, $order->id);
                            $history->addWithemail();
                        }
                    }
                    $this->updatePayplugInstallment($installment);
                    $reload = true;

                    die(Tools::jsonEncode(array('reload' => $reload)));
                }
            }

            if (Tools::getValue('check')) {
                $content = $this->getCheckFieldset();
                die(Tools::jsonEncode(array('content' => $content)));
            }
            if (Tools::getValue('log')) {
                $content = $this->getLogin();
                die(Tools::jsonEncode(array('content' => $content)));
            }
            if (Tools::getValue('checkPremium')) {
                $api_key = $this->getConfiguration('PAYPLUG_LIVE_API_KEY');
                die(Tools::jsonEncode($this->checkPremium($api_key)));
            }
            if (Tools::getValue('has_live_key')) {
                die(Tools::jsonEncode(array('result' => $this->hasLiveKey())));
            }
            if (Tools::getValue('refund')) {
                if (!$this->checkAmountToRefund(Tools::getValue('amount'))) {
                    die(Tools::jsonEncode(array(
                        'status' => 'error',
                        'data' => $this->l('Incorrect amount to refund')
                    )));
                } else {
                    $amount = str_replace(',', '.', Tools::getValue('amount'));
                    $amount = (float)($amount * 1000); // we use this trick to avoid rounding while converting to int
                    $amount = (float)($amount / 10); // otherwise, sometimes 17.90 become 17.89 \o/
                    $amount = (int)$amount;
                }

                $id_order = Tools::getValue('id_order');
                $pay_id = Tools::getValue('pay_id');
                $inst_id = Tools::getValue('inst_id');
                $metadata = array(
                    'Client' => Tools::getValue('id_customer'),
                    'reason' => 'Refunded with Prestashop'
                );
                $pay_mode = Tools::getValue('pay_mode');

                $refund = $this->makeRefund($pay_id, $amount, $metadata, $pay_mode, $inst_id);

                if ($refund == 'error') {
                    die(Tools::jsonEncode(array(
                        'status' => 'error',
                        'data' => $this->l('Cannot refund that amount.')
                    )));
                } else {
                    $new_state = 7;
                    $reload = false;

                    if ($inst_id != null) {
                        $installment = $this->retrieveInstallment($inst_id);
                        $amount_available = 0;
                        $amount_refunded_payplug = 0;
                        if (isset($installment->schedule)) {
                            foreach ($installment->schedule as $schedule) {
                                if (!empty($schedule->payment_ids)) {
                                    foreach ($schedule->payment_ids as $p_id) {
                                        $p = \Payplug\Payment::retrieve($p_id);
                                        if ($p->is_paid && !$p->is_refunded) {
                                            $amount_available += $p->amount - $p->amount_refunded;
                                        }
                                        $amount_refunded_payplug += $p->amount_refunded;
                                    }
                                }
                            }
                        }
                        $amount_available = $this->convertAmount($amount_available, true);
                        $amount_refunded_payplug = $this->convertAmount($amount_refunded_payplug, true);
                        if (Tools::getValue('id_state') || !$amount_available) {
                            if (!$new_state = Tools::getValue('id_state')) {
                                $new_state = $this->getConfiguration(
                                    'PAYPLUG_ORDER_STATE_REFUND' . ($installment->is_live ? '' : '_TEST')
                                );
                            }
                            $order = new Order($id_order);
                            if (Validate::isLoadedObject($order)) {
                                $current_state = $order->getCurrentState();
                                if ($current_state != 0 && $current_state != $new_state) {
                                    $history = new OrderHistory();
                                    $history->id_order = $order->id;
                                    $history->changeIdOrderState($new_state, $order->id);
                                    $history->addWithemail();
                                }
                            }
                            $reload = true;
                        }
                    } else {
                        $payment = $this->retrievePayment($refund->payment_id);
                        $new_state = Tools::getValue('id_state');
                        if (!$new_state && $payment->is_refunded) {
                            $new_state = $this->getConfiguration(
                                'PAYPLUG_ORDER_STATE_REFUND' . ($payment->is_live ? '' : '_TEST')
                            );
                        }

                        if ($new_state || ($payment->is_refunded == 1 && empty($inst_id))) {
                            $order = new Order($id_order);
                            if (Validate::isLoadedObject($order)) {
                                $current_state = $order->getCurrentState();
                                if ($current_state && $current_state != $new_state) {
                                    $history = new OrderHistory();
                                    $history->id_order = $order->id;
                                    $history->changeIdOrderState($new_state, $order->id);
                                    $history->addWithemail();
                                }
                            }
                            $reload = true;
                        }

                        $amount_refunded_payplug = ($payment->amount_refunded) / 100;
                        $amount_available = ($payment->amount - $payment->amount_refunded) / 100;
                    }

                    $data = $this->getRefundData(
                        $amount_refunded_payplug,
                        $amount_available
                    );
                    die(Tools::jsonEncode(array(
                        'status' => 'ok',
                        'data' => $data,
                        'message' => $this->l('Amount successfully refunded.'),
                        'reload' => $reload
                    )));
                }
            }
            if (Tools::getValue('capture')) {
                $pay_id = Tools::getValue('pay_id');
                $id_order = Tools::getValue('id_order');
                $payment = new PPPayment($pay_id);
                $capture = $payment->capture();
                $payment->refresh();
                if ($payment->resource->card->id !== null) {
                    $this->saveCard($payment->resource);
                }
                if ($capture['code'] >= 300) {
                    die(Tools::jsonEncode(array(
                        'status' => 'error',
                        'data' => $this->l('Cannot capture this payment.'),
                        'message' => $capture['message'],
                    )));
                } else {
                    $state_addons = ($payment->resource->is_live ? '' : '_TEST');
                    $new_state = $this->getConfiguration('PAYPLUG_ORDER_STATE_PAID' . $state_addons);

                    $order = new Order($id_order);
                    if (Validate::isLoadedObject($order)) {
                        $order->setInvoice(true);
                        $current_state = $order->getCurrentState();
                        if ($current_state != 0 && $current_state != $new_state) {
                            $history = new OrderHistory();
                            $history->id_order = $order->id;
                            $history->changeIdOrderState($new_state, $order->id);
                            $history->addWithemail();
                        }

                        // Update id transaction on last order payment.
                        if ($transaction_id = $this->getPayplugOrderPayment($order->id)) {
                            if ($orderPayments = $order->getOrderPayments()) {
                                $order_payment = end($orderPayments);
                                $order_payment->transaction_id = $transaction_id;
                                $order_payment->update();
                            }
                        }
                    }

                    die(Tools::jsonEncode(array(
                        'status' => 'ok',
                        'data' => '',
                        'message' => $this->l('Payment successfully captured.'),
                        'reload' => true,
                    )));
                }
            }
            if ((int)Tools::getValue('cancel')) {
                $pay_id = Tools::getValue('pay_id');
                $payment = new PPPayment($pay_id);
                $cancel = $payment->abort();
                $payment->refresh();
                if ($cancel['code'] >= 300) {
                    die(Tools::jsonEncode(array(
                        'status' => 'error',
                        'data' => $this->l('Cannot cancel this payment.'),
                        'message' => $cancel['message'],
                    )));
                } else {
                    die(Tools::jsonEncode(array(
                        'status' => 'ok',
                        'data' => '',
                        'message' => $this->l('Payment successfully cancel.'),
                        'reload' => true,
                    )));
                }
            }
            if (Tools::getValue('popinRefund')) {
                $popin = $this->displayPopin('refund');
                die(Tools::jsonEncode(array('content' => $popin)));
            }
            if (Tools::getValue('update')) {
                $pay_id = Tools::getValue('pay_id');
                $payment = $this->retrievePayment($pay_id);
                $id_order = Tools::getValue('id_order');

                if ($payment->is_paid) {
                    $new_state = $this->getConfiguration(
                        'PAYPLUG_ORDER_STATE_PAID' . ($payment->is_live ? '' : '_TEST')
                    );
                } else {
                    $new_state = $this->getConfiguration(
                        'PAYPLUG_ORDER_STATE_ERROR' . ($payment->is_live ? '' : '_TEST')
                    );
                }

                $order = new Order($id_order);
                if (Validate::isLoadedObject($order)) {
                    $current_state = $order->getCurrentState();
                    if ($current_state != 0 && $current_state != $new_state) {
                        $history = new OrderHistory();
                        $history->id_order = $order->id;
                        $history->changeIdOrderState($new_state, $order->id);
                        $history->addWithemail();
                    }
                }

                die(Tools::jsonEncode(array(
                    'message' => $this->l('Order successfully updated.'),
                    'reload' => true
                )));
            }
        } else {
            exit;
        }
    }

    public function getAdminAjaxUrl($controller_name = 'AdminModules', $id_order = 0)
    {
        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            $admin_ajax_url = PayplugBackward::getHttpHost(true) . __PS_BASE_URI__
                . 'modules/payplug/controllers/admin/AdminAjaxController.php';
        } else {
            if ($controller_name == 'AdminModules') {
                $admin_ajax_url = 'index.php?controller=' . $controller_name . '&configure=' . $this->name
                    . '&tab_module=payments_gateways&module_name=payplug&token='
                    . Tools::getAdminTokenLite($controller_name);
            } elseif ($controller_name == 'AdminOrders') {
                $admin_ajax_url = 'index.php?controller=' . $controller_name . '&id_order=' . $id_order
                    . '&vieworder&token=' . Tools::getAdminTokenLite($controller_name);
            }
        }
        return $admin_ajax_url;
    }

    public function hookCustomerAccount($params)
    {
        if (!$this->isAllowed()) {
            return false;
        }

        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            $params = array();
        } else {
            $params = array('process' => 'cardlist');
        }
        $payplug_cards_url = PayplugBackward::getModuleLink($this->name, 'cards', $params, true);

        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            $payplug_icon_url = PayplugBackward::getHttpHost(true) . __PS_BASE_URI__
                . 'modules/' . $this->name . '/views/img/logo16.gif';
            $version = 1.4;
        } elseif (version_compare(_PS_VERSION_, '1.6', '<')) {
            $payplug_icon_url = PayplugBackward::getHttpHost(true) . __PS_BASE_URI__
                . 'modules/' . $this->name . '/views/img/logo26.png';
            $version = 1.5;
        } elseif (version_compare(_PS_VERSION_, '1.6', '>=')) {
            $payplug_icon_url = PayplugBackward::getHttpHost(true) . __PS_BASE_URI__
                . 'modules/' . $this->name . '/views/img/logo26.png';
            $version = 1.6;
        }

        $this->smarty->assign(array(
            'payplug_cards_url' => $payplug_cards_url,
            'payplug_icon_url' => $payplug_icon_url,
            'version' => $version
        ));

        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            return $this->display(__FILE__, './views/templates/hook/my_account.tpl');
        } else {
            return $this->display(__FILE__, 'my_account.tpl');
        }
    }

    public function hookActionDeleteGDPRCustomer($customer)
    {
        if (!PayPlugCard::deleteAll($customer['id'])) {
            return json_encode($this->l('PayPlug : Unable to delete customer saved cards.'));
        }
        return json_encode(true);
    }

    public function hookActionExportGDPRData($customer)
    {
        if (!$cards = $this->gdprCardExport($customer['id'])) {
            return json_encode($this->l('PayPlug : Unable to export customer saved cards.'));
        } else {
            return json_encode($cards);
        }
    }

    public function displayGDPRConsent()
    {
        $this->context->smarty->assign(array('id_module' => $this->id));
        return $this->display(__FILE__, 'gdpr_consent.tpl');
    }

    public function hookRegisterGDPRConsent()
    {
    }

    public function gdprCardExport($id_customer)
    {
        if (!is_int($id_customer) || $id_customer === null) {
            return false;
        }

        $payplug_card = new PayPlugCard();
        $payplug_cards = $payplug_card->getByCustomer($id_customer);

        if (!$payplug_cards) {
            return null;
        }


        $cards = array();
        foreach ($payplug_cards as $k => &$card) {
            $card['expiry_date'] = date(
                'm / y',
                mktime(0, 0, 0, (int)$card['exp_month'], 1, (int)$card['exp_year'])
            );
            $cards[] = array(
                $this->l('#') => (int)$k + 1,
                $this->l('Brand') => $card['brand'],
                $this->l('Country') => $card['country'],
                $this->l('Card') => '**** **** **** ' . $card['last4'],
                $this->l('Expiry date') => $card['expiry_date']
            );
        }
        return $cards;
    }

    private function getOrderStates($id_lang = null)
    {
        if ($id_lang === null) {
            $id_lang = $this->context->language->id;
        }
        $order_states = OrderState::getOrderStates($id_lang);
        return $order_states;
    }

    public function hookUpdateOrderStatus($params)
    {
        return $this->hookActionOrderStatusUpdate($params);
    }

    public function hookActionOrderStatusUpdate($params)
    {
        $active = false;
        $order = new Order($params['id_order']);
        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            $active = $this->active;
        } else {
            $active = Module::isEnabled($this->name);
        }
        if (!$active
            || !$order->payment == $this->displayName
            || !$this->isReferredPaymentsActive()
            || !$this->isReferredAutoActive()
            || $params['newOrderStatus']->id != $this->getConfiguration('PAYPLUG_DEFERRED_STATE')
        ) {
            return;
        } else {
            $cart = new Cart($order->id_cart);
            $payment_method = $this->getPaymentMethodByCart($cart);

            if ($payment_method['type'] == 'installment') {
                $installment = new PPPaymentInstallment($payment_method['id']);
                $payment = $installment->getFirstPayment();
            } else {
                $payment = new PPPayment($payment_method['id']);
            }

            if (!$payment->isPaid()) {
                $payment->capture();
                $payment->refresh();
                if ($payment->resource->card->id !== null) {
                    $this->saveCard($payment->resource);
                }
            }
        }
    }

    private function isReferredPaymentsActive()
    {
        return $this->getConfiguration('PAYPLUG_DEFERRED');
    }

    private function isReferredAutoActive()
    {
        return $this->getConfiguration('PAYPLUG_DEFERRED_AUTO');
    }

    public function getPaymentStatusById($id_status, $id_lang = null)
    {
        return '<span class="instcolor' . (int)$id_status . '">' . $this->payment_status[$id_status] . '</span>';
    }

    private function getPaymentStatusByPayment($payment)
    {
        /*
            1 => 'not paid',
            2 => 'paid',
            3 => 'failed',
            4 => 'partially refunded',
            5 => 'refunded',
            6 => 'on going',
            7 => 'cancelled',
            8 => 'authorized',
            9 => 'authorization expired',
            10 => 'oney pending',
            11 => 'abandoned',
        */
        if (!is_object($payment)) {
            $payment = \Payplug\Payment::retrieve($payment);
        }

        if ($payment->installment_plan_id !== null) {
            $installment = $this->retrieveInstallment($payment->installment_plan_id);
            if (!$installment) {
                return false;
            }
        } else {
            $installment = null;
        }

        $pay_status = 1; //not paid
        if ((int)$payment->is_paid == 1) {
            $pay_status = 2; //paid
        } elseif (isset($payment->payment_method)
            && isset($payment->payment_method['is_pending'])
            && (int)$payment->payment_method['is_pending'] == 1
        ) {
            $pay_status = 10; //oney pending
        } elseif (count($payment->failure) > 0 && $pay_status != 9) {
            if ($payment->failure->code == 'aborted') {
                $pay_status = 7; //cancelled
            } elseif ($payment->failure->code == 'timeout') {
                $pay_status = 11; //abandoned
            } else {
                $pay_status = 3; //failed
            }
        } elseif ($payment->authorization !== null && ($payment->authorization->expires_at - time()) > 0) {
            $pay_status = 8; //authorized
        } elseif ($payment->authorization !== null && ($payment->authorization->expires_at - time()) <= 0) {
            $pay_status = 9; //authorization expired
        } elseif ($payment->installment_plan_id !== null && (int)$installment->is_active == 1) {
            $pay_status = 6; //ongoing
        }
        if ((int)$payment->is_refunded == 1) {
            $pay_status = 5; //refunded
        } elseif ((int)$payment->amount_refunded > 0) {
            $pay_status = 4; //partially refunded
        }

        return $pay_status;
    }

    public function addPayplugInstallment($installment, $order)
    {
        if (!is_object($installment)) {
            $installment = $this->retrieveInstallment($installment);
            if (!$installment) {
                return false;
            }
        }

        if ($this->getStoredInstallment($installment)) {
            $this->updatePayplugInstallment($installment);
        } else {
            if (isset($installment->schedule)) {
                $step_count = count($installment->schedule);
                $index = 0;
                foreach ($installment->schedule as $schedule) {
                    $index++;
                    $pay_id = '';
                    $status = 1; //not paid
                    if (count($schedule->payment_ids) > 0) {
                        $pay_id = $schedule->payment_ids[0];
                        $payment = \Payplug\Payment::retrieve($pay_id);
                        $status = $this->getPaymentStatusByPayment($payment);
                    } else {
                        if ($installment->is_active) {
                            $status = 6; //ongoing
                        } else {
                            $status = 7; //cancelled
                        }
                    }
                    $amount = $schedule->amount;
                    $step = $index . '/' . $step_count;
                    $date = $schedule->date;
                    $req_insert_installment = '
                INSERT INTO `' . _DB_PREFIX_ . 'payplug_installment` (
                    `id_installment`, `id_payment`, `id_order`, `id_customer`, 
                    `order_total`, `step`, `amount`, `status`, `scheduled_date`
                ) VALUES (
                    \'' . $installment->id . '\', \'' . $pay_id . '\', \'' . $order->id . '\', 
                    \'' . $order->id_customer . '\', \'' . (int)(($order->total_paid * 1000) / 10) . '\', 
                    \'' . $step . '\', \'' . $amount . '\', \'' . $status . '\', \'' . $date . '\'
                )';
                    $res_insert_installment = Db::getInstance()->execute($req_insert_installment);

                    if (!$res_insert_installment) {
                        return false;
                    }
                }
            }
        }

        return true;
    }

    public function updatePayplugInstallment($installment)
    {
        if (!is_object($installment)) {
            $installment = $this->retrieveInstallment($installment);
            if (!$installment) {
                return false;
            }
        }
        if ($installment && isset($installment->schedule)) {
            $step_count = count($installment->schedule);
            $index = 0;
            foreach ($installment->schedule as $schedule) {
                $index++;
                $pay_id = '';
                if (count($schedule->payment_ids) > 0) {
                    $pay_id = $schedule->payment_ids[0];
                    $payment = \Payplug\Payment::retrieve($pay_id);
                    $status = $this->getPaymentStatusByPayment($payment);
                } else {
                    if ($installment->is_active) {
                        $status = 6; //ongoing
                    } else {
                        $status = 7; //cancelled
                    }
                }
                $step = $index . '/' . $step_count;
                if ($step2update = $this->getStoredInstallmentTransaction($installment, $step)) {
                    $req_insert_installment = '
                        UPDATE `' . _DB_PREFIX_ . 'payplug_installment` 
                        SET `id_payment` = \'' . pSQL($pay_id) . '\', 
                        `status` = \'' . (int)$status . '\' 
                        WHERE `id_payplug_installment` = ' . (int)$step2update['id_payplug_installment'];
                    $res_insert_installment = Db::getInstance()->execute($req_insert_installment);

                    if (!$res_insert_installment) {
                        return false;
                    }
                } else {
                    return false;
                }
            }
        }
    }

    public function getStoredInstallment($installment)
    {
        if (!is_object($installment)) {
            $installment = $this->retrieveInstallment($installment);
            if (!$installment) {
                return false;
            }
        }
        $req_installment = '
            SELECT pi.*
            FROM `' . _DB_PREFIX_ . 'payplug_installment` pi 
            WHERE pi.id_installment = \'' . $installment->id . '\'';
        $res_installment = Db::getInstance()->executeS($req_installment);

        if (!$res_installment) {
            return false;
        } else {
            return $res_installment;
        }
    }

    public function getStoredInstallmentTransaction($installment, $step)
    {
        if (!is_object($installment)) {
            $installment = $this->retrieveInstallment($installment);
            if (!$installment) {
                return false;
            }
        }
        $req_installment = '
            SELECT pi.*
            FROM `' . _DB_PREFIX_ . 'payplug_installment` pi 
            WHERE pi.id_installment = \'' . $installment->id . '\' 
            AND pi.step = ' . (int)$step;
        $res_installment = Db::getInstance()->getRow($req_installment);

        if (!$res_installment) {
            return false;
        } else {
            return $res_installment;
        }
    }

    public function hookDisplayBackOfficeHeader()
    {
        if (!Module::isEnabled($this->name)) {
            return false;
        }

        if (version_compare(_PS_VERSION_, '1.6', '>=')) {
            $this->context->controller->addCss($this->_path . 'views/css/tab.css');
        }
    }

    /**
     * Check payment method for given cart object
     *
     * @param object Cart
     * @return array|bool pay_id or inst_id or False
     */
    public function getPaymentMethodByCart($cart)
    {
        if (!is_object($cart)) {
            $cart = new Cart($cart);
        }

        if (!Validate::isLoadedObject($cart)) {
            return false;
        }

        $inst_id = $this->getInstallmentByCart($cart->id);
        if ($inst_id) {
            return array('id' => $inst_id, 'type' => 'installment');
        }

        $pay_id = $this->getPaymentByCart($cart->id);
        if ($pay_id) {
            return array('id' => $pay_id, 'type' => 'payment');
        }

        return false;
    }

    public function buildPaymentDetails($payment)
    {
        if (!is_object($payment)) {
            try {
                $payment = \Payplug\Payment::retrieve($payment);
            } catch (Exception $exception) {
                return $exception;
            }
        }
        $pay_status = $this->getPaymentStatusByPayment($payment);
        $status_class = null;
        switch ($pay_status) {
            case 1: // not paid
            case 5: // refunded
            case 8: // authorized
            case 11: // abandoned
                $status_class = 'pp_warning';
                break;
            case 2: // paid
                $status_class = 'pp_success';
                break;
            case 3: // failed
            case 7: // cancelled
            case 9: // authorization expired
                $status_class = 'pp_error';
                break;
            case 4: // partially refunded
            case 6: // on going
                $status_class = 'pp_neutral';
                break;
            default:
                $status_class = 'pp_other';
                break;
        }
        $pay_status = $this->payment_status[$pay_status];

        $pay_brand = $this->getCardBrandByPayment($payment);
        if ($payment->card->country != '') {
            $pay_brand .= ' ' . $this->l('Card') . ' (' . $payment->card->country . ')';
        }

        $payment_details = array(
            'id' => $payment->id,
            'status' => $pay_status,
            'status_class' => $status_class,
            'amount' => (int)$payment->amount / 100,
            'card_brand' => $pay_brand,
            'card_mask' => $this->getCardMaskByPayment($payment),
            'card_date' => $this->getCardExpiryDateByPayment($payment),
            'mode' => $payment->is_live ? $this->l('LIVE') : $this->l('TEST'),
            'paid' => (bool)$payment->is_paid,
        );

        //Deferred payment does'nt display 3DS option before capture so we have to consider it null
        if ($payment->is_3ds !== null) {
            $payment_details['tds'] = $payment->is_3ds ? $this->l('YES') : $this->l('NO');
        }

        if (isset($payment->payment_method) && isset($payment->payment_method['type'])) {
            switch ($payment->payment_method['type']) {
                case 'oney_x3_with_fees':
                    $payment_details['type'] = 'Oney 3x';
                    break;
                case 'oney_x4_with_fees':
                    $payment_details['type'] = 'Oney 4x';
                    break;
                default:
                    $payment_details['type'] = $payment->payment_method['type'];
            }
        }
        if ($payment->authorization !== null) {
            $payment_details['authorization'] = true;
            if ($payment->is_paid) {
                $payment_details['date'] = date('d/m/Y', $payment->paid_at);
                $payment_details['can_be_cancelled'] = false;
                $payment_details['can_be_captured'] = false;
                if (!isset($payment_details['type'])) {
                    $payment_details['status_message'] = $this->l('(deferred)');
                }
            } else {
                $expiration = date('d/m/Y', $payment->authorization->expires_at);
                if (isset($payment->authorization->expires_at) && $payment->authorization->expires_at - time() > 0) {
                    if (isset($payment->failure) && count($payment->failure) > 0) {
                        $payment_details['can_be_cancelled'] = false;
                        $payment_details['can_be_captured'] = false;
                    } else {
                        $payment_details['can_be_captured'] = true;
                        $payment_details['can_be_cancelled'] = true;
                        $payment_details['status_message'] = sprintf(
                            $this->l('(capture authorized before %s)'),
                            $expiration
                        );
                    }
                    $payment_details['date'] = date('d/m/Y', $payment->authorization->authorized_at);
                    $payment_details['date_expiration'] = $expiration;
                    $payment_details['expiration_display'] = sprintf(
                        $this->l(
                            'Capture of this payment is authorized before %s. '
                            . 'After this date, you will not be able to get paid.'
                        ),
                        $expiration
                    );
                } elseif (isset($payment->authorization->authorized_at)
                    && $payment->authorization->authorized_at != null
                ) {
                    $payment_details['date'] = date('d/m/Y', $payment->authorization->authorized_at);
                    $payment_details['can_be_cancelled'] = false;
                    $payment_details['can_be_captured'] = false;
                } else {
                    $payment_details['can_be_cancelled'] = false;
                    $payment_details['can_be_captured'] = false;
                }
            }
        } else {
            $payment_details['authorization'] = false;
            $payment_details['date'] = date('d/m/Y', $payment->created_at);
            $payment_details['can_be_cancelled'] = false;
            $payment_details['can_be_captured'] = false;
        }

        if (isset($payment->failure) && isset($payment->failure->message)) {
            $payment_details['error'] = '(' . $payment->failure->message . ')';
        }

        if (isset($payment_details['type']) && in_array($payment_details['type'], array('Oney 3x', 'Oney 4x'))) {
            unset($payment_details['card_brand']);
            unset($payment_details['card_mask']);
            unset($payment_details['card_date']);
        }

        return $payment_details;
    }

    public function getCardBrandByPayment($payment)
    {
        if (!is_object($payment)) {
            try {
                $payment = \Payplug\Payment::retrieve($payment);
            } catch (Exception $exception) {
                return $exception;
            }
        }

        if ($payment->card->brand != '') {
            $brand = $payment->card->brand;
        } else {
            $brand = $this->l('Unavailable');
        }
        return $brand;
    }

    public function getCardMaskByPayment($payment)
    {
        if (!is_object($payment)) {
            try {
                $payment = \Payplug\Payment::retrieve($payment);
            } catch (Exception $exception) {
                return $exception;
            }
        }

        if ($payment->card->last4 != '') {
            $card_mask = '**** **** **** ' . $payment->card->last4;
        } else {
            $card_mask = $this->l('Unavailable');
        }
        return $card_mask;
    }

    public function getCardExpiryDateByPayment($payment)
    {
        if (!is_object($payment)) {
            try {
                $payment = \Payplug\Payment::retrieve($payment);
            } catch (Exception $exception) {
                return $exception;
            }
        }

        if ($payment->card->exp_month === null) {
            $card_expiry_date = $this->l('Unavailable');
        } else {
            $card_expiry_date = date(
                'm/y',
                strtotime('01.' . $payment->card->exp_month . '.' . $payment->card->exp_year)
            );
        }
        return $card_expiry_date;
    }

    /**
     * Check if payment method is valid for given id
     *
     * @param string $payment_id
     * @param string $type default payment
     * @return bool
     */
    public function isPaidPaymentMethod($payment_id, $type = 'payment')
    {
        switch ($type) {
            case 'installment':
                $installment = $this->retrieveInstallment($payment_id);
                if (!$installment) {
                    return false;
                }
                if ($installment && $installment->is_active) {
                    $schedules = $installment->schedule;
                    foreach ($schedules as $schedule) {
                        foreach ($schedule->payment_ids as $pay_id) {
                            $inst_payment = \Payplug\Payment::retrieve($pay_id);
                            if ($inst_payment && $inst_payment->is_paid) {
                                return true;
                            }
                        }
                    }
                }
                break;
            case 'payment':
            default:
                $payment = \Payplug\Payment::retrieve($payment_id);
                return $payment && $payment->is_paid;
                break;
        }

        return false;
    }

    /**
     * get the payment method for a given payment card
     *
     * @param string $card
     * @return object PayPlugPaymentStandard|PayPlugPaymentInstallment|PayPlugPaymentOneClick|PayPlugPaymentOney
     */
    public function getCurrentPaymentMethod($card = null)
    {
        $card = $card != null ? $card : Tools::getValue('pc', null);

        // check if is Installment
        if (Tools::getValue('io') || Tools::getValue('type') == 'oney') {
            $payment_method = 'PayPlugPaymentOney';
        } elseif (Tools::getValue('i') || Tools::getValue('type') == 'installment') {
            $payment_method = 'PayPlugPaymentInstallment';
        } elseif (($card != null && $card != 'new_card') || Tools::getValue('type') == 'oneclick') {
            $payment_method = 'PayPlugPaymentOneClick';
        } elseif (Tools::getValue('type') == 'standard') {
            $payment_method = 'PayPlugPaymentStandard';
        } else {
            $payment_method = 'PayPlugPaymentStandard';
        }

        return $payment_method;
    }

    /**
     * Return exeption error form API
     * @param $str
     * @return array
     */
    public function catchErrorsFromApi($str)
    {
        $parses = explode(';', $str);
        $response = null;
        foreach ($parses as $parse) {
            if (strpos($parse, 'HTTP Response') !== false) {
                $parse = str_replace('HTTP Response:', '', $parse);
                $parse = trim($parse);
                $response = Tools::jsonDecode($parse, true);
            }
        }
        $errors = array();

        if (!isset($response['details']) || empty($response['details'])) {
            if (isset($response['message']) && $response['message'] == 'Access to this feature is not available.') {
                $this->updatePermissions();
            }
            // set default error when no one is catch
            $error_key = md5('The transaction was not completed and your card was not charged.');
            $errors[$error_key] = $this->l('The transaction was not completed and your card was not charged.');
            return $errors;
        }

        foreach ($response['details'] as $key => $value) {
            // add specific error message
            switch ($key) {
                default:
                    $error_key = md5('The transaction was not completed and your card was not charged.');
                    // push error only if not catched before
                    if (!array_key_exists($error_key, $errors)) {
                        $errors[$error_key] = $this->l(
                            'The transaction was not completed and your card was not charged.'
                        );
                    }
            }
        }

        sort($errors);

        return $errors;
    }

    /**
     * Check if user has live key
     * @return bool
     */
    public function hasLiveKey()
    {
        return (bool)$this->getConfiguration('PAYPLUG_LIVE_API_KEY');
    }

    /**
     * Assign Oney var
     *
     * @param $cart Cart
     * @return bool
     */
    private function assignOneyPaymentOptions($cart)
    {
        if (!$this->getConfiguration('PAYPLUG_ONEY')) {
            return false;
        }

        // check if at least one carrier is available for this cart
        // get the available carrier
        $package_list = $cart->getPackageList();
        $carrier_ids = array();
        foreach ($package_list as $address) {
            foreach ($address as $package) {
                $carrier_ids = array_merge($carrier_ids, $package['carrier_list']);
            }
        }

        if (Validate::isLoadedObject($cart) && $cart->id_address_invoice && $cart->id_address_delivery) {
            $is_elligible = $this->isOneyElligible($cart);
        } else {
            $id_currency = $this->context->currency->id;
            $amount = $cart->getOrderTotal(true, Cart::BOTH);
            $is_elligible = $this->isValidOneyAmount($amount, $id_currency);
        }

        $this->smarty->assign(array(
            'payplug_module_dir' => _PS_MODULE_DIR_,
            'payplug_oney' => true,
            'oney_required_fields' => $this->getOneyRequiredFields(),
            'payplug_oney_allowed' => $is_elligible['result'],
            'payplug_oney_error' => $is_elligible['error'],
            'payplug_oney_loading_msg' => $this->l('Loading')
        ));
    }

    /**
     * Assign Oney javascript variable
     */
    private function assignOneyJSVar()
    {
        $js_var = array(
            'loading_msg' => $this->l('Loading'),
            'can_use_oney' => $this->getConfiguration('PAYPLUG_ONEY'),
        );
        return Media::addJsDef($js_var);
    }

    /**
     * todo: to clean or update
     * @return array
     */
    public function checkOneyRequiredFields($payment_data)
    {
        $form_data = isset($payment_data['oney_form']) && $payment_data['oney_form'] ?
            $payment_data['oney_form']
            : false;
        $errors = array();

        if (!$form_data) {
            return array($this->l('Please fill in the required fields'));
        }

        foreach ($form_data as $key => $data) {
            $parsed = explode('-', $key);
            $type = $parsed[0];
            $field = $parsed[1];
            switch ($field) {
                case 'email':
                    if (Tools::strlen($data) > 100 && Tools::strpos($data, '+') !== false) {
                        $text = $this->l(
                            'Your email address is too long and the + character is not valid, '
                            . 'please change it to another address (max 100 characters).'
                        );
                        $errors[] = $text;
                    } elseif (Tools::strlen($data) > 100) {
                        $text = $this->l(
                            'Your email address is too long, please change it to a shorter one (max 100 characters).'
                        );
                        $errors[] = $text;
                    } elseif (Tools::strpos($data, '+') !== false) {
                        $text = $this->l(
                            'The + character is not valid. Please change your email address (100 characters max).'
                        );
                        $errors[] = $text;
                    }
                    break;
                case 'mobile_phone_number':
                    $id_address = $type == 'shipping' ?
                        $this->context->cart->id_address_delivery
                        : $this->context->cart->id_address_invoice;
                    $address = new Address($id_address);
                    $country = new Country($address->id_country);
                    $valid = $this->isValidMobilePhoneNumber($data, $country->iso_code);
                    if (!$valid) {
                        $errors[] = $this->l('Please enter your mobile phone number.');
                    }
                    break;
                case 'first_name':
                    if (!Validate::isPostCode($data)) {
                        $text = $type == 'shipping' ?
                            $this->l('Please enter your shipping firstname.') :
                            $this->l('Please enter your billing firstname.');
                        $errors[] = $text;
                    }
                    break;
                case 'last_name':
                    if (!Validate::isPostCode($data)) {
                        $text = $type == 'shipping' ?
                            $this->l('Please enter your shipping lastname.') :
                            $this->l('Please enter your billing lastname.');
                        $errors[] = $text;
                    }
                    break;
                case 'address1':
                    if (!Validate::isPostCode($data)) {
                        $text = $type == 'shipping' ?
                            $this->l('Please enter your shipping address.') :
                            $this->l('Please enter your billing address.');
                        $errors[] = $text;
                    }
                    break;
                case 'postcode':
                    if (!Validate::isPostCode($data)) {
                        $text = $type == 'shipping' ?
                            $this->l('Please enter your shipping postcode.') :
                            $this->l('Please enter your billing postcode.');
                        $errors[] = $text;
                    }
                    break;
                case 'city':
                    if (!Validate::isCityName($data)) {
                        $text = $type == 'shipping' ?
                            $this->l('Please enter your shipping city.') :
                            $this->l('Please enter your billing city.');
                        $errors[] = $text;
                    } elseif (Tools::strlen($data) > 32) {
                        $text = $this->l('Your city name is too long (max 32 characters). ')
                            . $this->l('Please change it to another one or select another payment method.');
                        $errors[] = $text;
                    }
                    break;
            }
        }

        return $errors;
    }

    /**
     * Check if payplug order state are well installed
     */
    public function checkOrderStates()
    {
        $order_states = array_merge($this->order_states, $this->oney_order_state);

        foreach ($order_states as $key => $state) {
            // Check live OrderState
            $key_config_live = 'PAYPLUG_ORDER_STATE_' . Tools::strtoupper($key);
            $id_order_state_live = Configuration::get($key_config_live);
            $order_state_live = new OrderState((int)$id_order_state_live);
            if (!Validate::isLoadedObject($order_state_live)) {
                $this->createOrderState($key, $state, false, true);
            }

            // Check sandbox OrderState
            $key_config_sandbox = $key_config_live . '_TEST';
            $id_order_state_sandbox = Configuration::get($key_config_sandbox);
            $order_state_sandbox = new OrderState((int)$id_order_state_sandbox);

            if (!Validate::isLoadedObject($order_state_sandbox)) {
                $this->createOrderState($key, $state, true, true);
            }
        }
    }

    /**
     * Check Prestashop version for new feature
     * @return bool
     */
    public function checkVersion($min = '1.6')
    {
        return (bool)version_compare(_PS_VERSION_, $min, '>=');
    }

    /**
     * Format amount float to int or int to float
     *
     * @param $amount
     * @param bool $to_cents
     * @return float|int
     */
    public function convertAmount($amount, $to_cents = false)
    {
        if ($to_cents) {
            return (float)($amount / 100);
        } else {
            $amount = (float)($amount * 1000); // we use this trick to avoid rounding while converting to int
            $amount = (float)($amount / 10); // otherwise, sometimes 17.90 become 17.89 \o/
            return (int)Tools::ps_round($amount);
        }
    }

    /**
     * Display payment errors template
     *
     * @param array $errors
     * @return mixed
     */
    public function displayPaymentErrors($errors = array())
    {
        if (empty($errors)) {
            return false;
        }

        $payment_messages = array();
        $with_msg_button = false;
        foreach ($errors as $error) {
            if ($error == 'oney_required_field') {
                $fields = $this->getOneyRequiredFields();
                $this->smarty->assign(array(
                    'is_popin_tpl' => true,
                    'oney_required_fields' => $fields
                ));
                $payment_messages[] = array(
                    'type' => 'template',
                    'value' => _PS_MODULE_DIR_. 'payplug/views/templates/hook/oney_form.tpl',
                );
            } else {
                $with_msg_button = true;
                $payment_messages[] = array(
                    'type' => 'string',
                    'value' => $error,
                );
            }
        }

        $this->smarty->assign(array(
            'payment_messages' => $payment_messages,
            'with_msg_button' => $with_msg_button,
        ));

        return $this->display(__FILE__, 'payment_messages.tpl');
    }

    /**
     * Display Oney popin payment option
     *
     * @return mixed
     */
    public function displayOneyPaymentOptions()
    {
        $this->smarty->assign(array(
            'payplug_module_dir' => _PS_MODULE_DIR_,
            'payplug_oney_loading_msg' => $this->l('Loading'),
        ));

        return $this->display(__FILE__, 'oney_payment.tpl');
    }

    /**
     * Display Oney popin template
     *
     * @return mixed
     */
    public function displayOneyPopin()
    {
        $limits = $this->getOneyPriceLimit();
        $min_amount = $this->convertAmount($limits['min'], true);
        $max_amount = $this->convertAmount($limits['max'], true);

        $legal_text = 'Offre de financement avec apport obligatoire, '
            . 'réservée aux particuliers et valable pour tout achat de %s à %s. ';
        $legal_text .= 'Sous réserve d\'acceptation par Oney Bank. ';
        $legal_text .= 'Vous disposez d\'un délai de 14 jours pour renoncer à votre crédit. ';
        $legal_text .= 'Oney Bank - SA au capital de 51 286 585€ - 34 Avenue de Flandre 59170 Croix '
            . '- 546 380 197 RCS Lille Métropole - n° Orias 07 023 261 www.orias.fr ';
        $legal_text .= 'Correspondance : CS 60 006 - 59895 Lille Cedex - www.oney.fr';

        $this->smarty->assign(array(
            'legal_notice' => sprintf(
                $this->l($legal_text),
                Tools::displayPrice($min_amount),
                Tools::displayPrice($max_amount)
            ),
            'tos_active' => $this->getConfiguration('PAYPLUG_ONEY_TOS'),
            'tos_url' => $this->getConfiguration('PAYPLUG_ONEY_TOS_URL')
        ));

        return $this->display(__FILE__, 'oney_popin.tpl');
    }

    /**
     * Display Oney required fields template
     *
     * @return mixed
     */
    public function displayOneyRequiredFields()
    {
        $fields = $this->getOneyRequiredFields();

        if (empty($fields)) {
            return false;
        }

        $this->smarty->assign(array(
            'oney_required_fields' => $fields
        ));

        return $this->display(__FILE__, 'oney_form.tpl');
    }

    /**
     * Format Oney simulation from resource
     *
     * @param string $method
     * @param array $resource
     * @param bool $amount
     * @return array
     */
    private function formatOneyResource($method, $resource, $amount = false)
    {
        $type = explode('_', $method);

        $resource['split'] = (int)str_replace('x', '', $type[0]);
        $resource['title'] = sprintf($this->l('Payment in %sx'), $resource['split']);

        // format price
        if (isset($resource['installments']) && $resource['installments']) {
            $amount = $this->convertAmount($amount, true);
            $resource['total_cost'] = $this->convertAmount($resource['total_cost'], true);
            $resource['down_payment_amount'] = $this->convertAmount($resource['down_payment_amount'], true);
            foreach ($resource['installments'] as &$installment) {
                $installment['amount'] = $this->convertAmount($installment['amount'], true);
            }
        }

        return $resource;
    }

    /**
     * Get payment data from cookie
     *
     * @return mixed
     */
    public function getPaymentDataCookie()
    {
        // get payplug data
        $cookie_data = $this->context->cookie->__get('payplug_data');
        $payplug_data = !empty($cookie_data) ? $cookie_data : false;

        // then flush to avoid repetition
        $this->context->cookie->__set('payplug_data', '');

        // if no error all good then return true
        return Tools::jsonDecode($payplug_data, true);
    }

    /**
     * Get payment errors from cookie
     *
     * @return mixed
     */
    public function getPaymentErrorsCookie()
    {
        // get payplug errors
        $cookie_errors = $this->context->cookie->__get('payplug_errors');
        $payplug_errors = !empty($cookie_errors) ? $cookie_errors : false;

        // then flush to avoid repetition
        $this->context->cookie->__set('payplug_errors', '');

        // if no error all good then return true
        return Tools::jsonDecode($payplug_errors, true);
    }

    /**
     *  Get Oney call to action
     *
     * @param string $env
     * @param bool $allowed
     * @return mixed
     */
    public function getOneyCTA($env = null)
    {
        $this->smarty->assign(array(
            'this_path' => $this->_path,
            'env' => $env,
            'payplug_module_dir' => _MODULE_DIR_,
            'payplug_oney_loading_msg' => $this->l('Loading')
        ));

        return $this->display(__FILE__, 'oney_cta.tpl');
    }

    /**
     * Temp get valid iso code for french overseas,
     * todo: remove when it's fix in API
     *
     * @param $iso_country
     * @return string
     */
    public function getOneyCountry($iso_country)
    {
        $overseas_iso = array('GP', 'MQ', 'GF', 'RE', 'YT');
        if (in_array($iso_country, $overseas_iso)) {
            return 'FR';
        }
        return $iso_country;
    }

    /**
     * Get Oney payment options
     *
     * @param float $order_total
     * @param string $country
     * @return array
     */
    public function getOneyPaymentOptionsList($amount, $country = false)
    {
        // get Oney resource
        $payment_list = array();
        $amount = $this->convertAmount($amount);

        if (!$country) {
            $iso_code_list = $this->getConfiguration('PAYPLUG_ONEY_ALLOWED_COUNTRIES');
            $iso_list = explode(',', $iso_code_list);
            $country = reset($iso_list);
        }

        $country = Tools::strtoupper($country);

        $oney_sims = $this->getOneySimulations($amount, $country, $this->available_oney_payments);

        if (!$oney_sims['result']) {
            foreach ($this->available_oney_payments as $available_oney_payments) {
                $oney_sims['simulations'][$available_oney_payments] = array(
                    'without_price' => true
                );
            }
        }

        foreach ($oney_sims['simulations'] as $method => $oney_sim) {
            $payment_list[$method] = $this->formatOneyResource($method, $oney_sim, $amount);
        }

        return $payment_list;
    }

    /**
     * Get Oney price limit
     *
     * @param int $id_currency
     * @return array
     */
    public function getOneyPriceLimit($id_currency = false)
    {
        if (Validate::isLoadedObject($id_currency)) {
            $currency = $id_currency;
        } else {
            if (!is_int($id_currency) && Validate::isLanguageIsoCode($id_currency)) {
                $id_currency = Country::getByIso($id_currency);
            }
            if (!$id_currency) {
                $id_currency = $this->getConfiguration('PS_CURRENCY_DEFAULT');
            }
            $currency = new Currency($id_currency);
        }

        $limits = array(
            'min' => false,
            'max' => false
        );

        if (!Validate::isLoadedObject($currency)) {
            return $limits;
        }

        $iso_code = Tools::strtoupper($currency->iso_code);

        $oney_min_amounts = explode(',', Tools::strtoupper($this->getConfiguration('PAYPLUG_ONEY_MIN_AMOUNTS')));
        foreach ($oney_min_amounts as $min_amount) {
            $min = explode(':', $min_amount);
            if ($min[0] == $iso_code) {
                $limits['min'] = (int)$min[1];
                break;
            }
        }

        $oney_max_amounts = explode(',', Tools::strtoupper($this->getConfiguration('PAYPLUG_ONEY_MAX_AMOUNTS')));
        foreach ($oney_max_amounts as $max_amount) {
            $max = explode(':', $max_amount);
            if ($max[0] == $iso_code) {
                $limits['max'] = (int)$max[1];
                break;
            }
        }

        return $limits;
    }

    /**
     * Display Oney payment options
     *
     * @param $cart Cart
     * @param float $order_total
     * @param string $country
     * @return array
     */
    public function getOneyPriceAndPaymentOptions($cart, $amount, $country = false)
    {
        if (Validate::isLoadedObject($cart) && $cart->id_address_invoice && $cart->id_address_delivery) {
            $is_elligible = $this->isOneyElligible($cart, $amount, $country);
        } else {
            $id_currency = $this->context->currency->id;
            $is_elligible = $this->isValidOneyAmount($amount, $id_currency);
        }

        $error = false;
        if ($is_elligible['result']) {
            $oney_payment_options = $this->getOneyPaymentOptionsList($amount, $country);
        } else {
            $oney_payment_options = false;
            $error = $is_elligible['error'] ? $is_elligible['error'] : $this->l('Oney is momentarily unavailable.');
        }

        $valid_payment_option = false;
        foreach ($oney_payment_options as $oney_option) {
            if (isset($oney_option['installments']) && $oney_option['installments']) {
                $valid_payment_option = true;
            }
        }

        $error = $is_elligible['error'] ? $is_elligible['error'] : (
            $oney_payment_options && $valid_payment_option ? false : $this->l('Oney is momentarily unavailable.')
        );

        $this->smarty->assign(array(
            'payplug_oney_required_field' => $this->displayOneyRequiredFields(),
            'payplug_oney_amount' => $amount,
            'payplug_oney_allowed' => $is_elligible['result'] && $oney_payment_options,
            'payplug_oney_error' => $error,
        ));

        if ($oney_payment_options) {
            $this->smarty->assign(array(
                'oney_payment_options' => $oney_payment_options,
            ));
        }

        $popin_tpl = $this->displayOneyPopin();

        $payment_tpl = $this->displayOneyPaymentOptions();

        return array(
            'result' => $is_elligible['result'] && $oney_payment_options,
            'error' => $error,
            'popin' => $popin_tpl,
            'payment' => $payment_tpl,
        );
    }

    /**
     * todo: to clean or update
     * @return array
     */
    public function getOneyRequiredFields()
    {
        $is_same = $this->context->cart->id_address_delivery == $this->context->cart->id_address_invoice;

        $fields = array();
        $shipping_fields = array();

        $shipping_address = new Address($this->context->cart->id_address_delivery);
        $shipping_country = new Country($shipping_address->id_country);

        // Validate email format
        if (Tools::strlen($this->context->customer->email) > 100
            && Tools::strpos($this->context->customer->email, '+') !== false
        ) {
            $text = $this->l('Your email address is too long and the + character is not valid,') .
                $this->l(' please change it to another address (max 100 characters).');
            $shipping_fields['email'] = array(
                'text' => $text,
                'input' => array(
                    array(
                        'name' => 'email',
                        'value' => $this->context->customer->email,
                        'type' => 'text'
                    )
                ),
            );
        } elseif (Tools::strlen($this->context->customer->email) > 100) {
            $text = $this->l('Your email address is too long, please change it to a shorter one (max 100 characters).');
            $shipping_fields['email'] = array(
                'text' => $text,
                'input' => array(
                    array(
                        'name' => 'email',
                        'value' => $this->context->customer->email,
                        'type' => 'text'
                    )
                ),
            );
        } elseif (Tools::strpos($this->context->customer->email, '+') !== false) {
            $text = $this->l('The + character is not valid. Please change your email address (100 characters max).');
            $shipping_fields['email'] = array(
                'text' => $text,
                'input' => array(
                    array(
                        'name' => 'email',
                        'value' => $this->context->customer->email,
                        'type' => 'text'
                    )
                ),
            );
        }

        // Validate phone number
        $is_valid_mobile_phone_number = $this->isValidMobilePhoneNumber(
            $shipping_address->phone_mobile,
            $shipping_country->iso_code
        );
        if (!$is_valid_mobile_phone_number) {
            $shipping_fields['mobile_phone_number'] = array(
                'text' => $this->l('Please enter your mobile phone number'),
                'input' => array(
                    array(
                        'name' => 'mobile_phone_number',
                        'value' => $shipping_address->phone_mobile,
                        'type' => 'text'
                    )
                ),
            );
        }

        // Validate address
        if (Tools::strlen($shipping_address->city) > 32) {
            $text = $this->l('Your city name is too long (max 32 characters). ')
                . $this->l('Please change it to another one or select another payment method.');
            $shipping_fields['city'] = array(
                'text' => $text,
                'input' => array(
                    array(
                        'name' => 'first_name',
                        'value' => $shipping_address->firstname,
                        'type' => 'text'
                    ),
                    array(
                        'name' => 'last_name',
                        'value' => $shipping_address->lastname,
                        'type' => 'text'
                    ),
                    array(
                        'name' => 'address1',
                        'value' => $shipping_address->address1,
                        'type' => 'text'
                    ),
                    array(
                        'name' => 'postcode',
                        'value' => $shipping_address->postcode,
                        'type' => 'text'
                    ),
                    array(
                        'name' => 'city',
                        'value' => $shipping_address->city,
                        'type' => 'text'
                    ),
                ),
            );
        }

        if ($is_same && !empty($shipping_fields)) {
            $fields['same'] = $shipping_fields;
        } else {
            if (!empty($shipping_fields)) {
                $fields['shipping'] = $shipping_fields;
            }
            $billing_fields = array();
            $billing_address = new Address($this->context->cart->id_address_invoice);
            $billing_country = new Country($billing_address->id_country);

            $is_valid_mobile_phone_number = $this->isValidMobilePhoneNumber(
                $billing_address->phone_mobile,
                $billing_country->iso_code
            );
            if (!$is_valid_mobile_phone_number) {
                $billing_fields['mobile_phone_number'] = array(
                    'text' => $this->l('Please enter your mobile phone number'),
                    'input' => array(
                        array(
                            'name' => 'mobile_phone_number',
                            'value' => $shipping_address->phone_mobile,
                            'type' => 'text'
                        )
                    ),
                );
            }

            if (Tools::strlen($billing_address->city) > 32) {
                $text = $this->l('Your city name is too long (max 32 characters). ')
                    . $this->l('Please change it to another one or select another payment method.');
                $billing_fields['city'] = array(
                    'text' => $text,
                    'input' => array(
                        array(
                            'name' => 'first_name',
                            'value' => $billing_address->firstname,
                            'type' => 'text'
                        ),
                        array(
                            'name' => 'last_name',
                            'value' => $billing_address->lastname,
                            'type' => 'text'
                        ),
                        array(
                            'name' => 'address1',
                            'value' => $billing_address->address1,
                            'type' => 'text'
                        ),
                        array(
                            'name' => 'postcode',
                            'value' => $billing_address->postcode,
                            'type' => 'text'
                        ),
                        array(
                            'name' => 'city',
                            'value' => $billing_address->city,
                            'type' => 'text'
                        ),
                    ),
                );
            }

            if (!empty($billing_fields)) {
                $fields['billing'] = $billing_fields;
            }
        }

        return $fields;
    }

    /**
     * Get Oney Payment Simulations
     *
     * @param int $amount
     * @param string $country
     * @param array $operation contain x3|4_with_fees or x3|4_without_fees
     * @return array
     */
    public function getOneySimulations($amount, $country, $operation)
    {
        $cache_id = 'Payplug::OneySimulations_' .
            (int)$amount . '_' .
            (string)$country . '_' .
            (string)implode('_', $operation) . '_' .
            ($this->getConfiguration('PAYPLUG_SANDBOX_MODE') ? 'test' : 'live');

        $cache_from_bdd = $this->payplug_cache->getCacheByKey($cache_id);

        // Checks if the current simulation is already saved in the database
        // If not, we do a simulation for Oney, and we will store it to the DB
        if (Validate::isLoadedObject($cache_from_bdd)) {
            return Tools::jsonDecode($cache_from_bdd->cache_value, true);
        }

        try {
            $data = array(
                'amount' => $amount,
                'country' => $this->getOneyCountry($country),
                'operations' => $operation,
            );

            $simulations = \Payplug\OneySimulation::getSimulations($data);

            if (isset($simulations['details'])
                && $simulations['details'] == 'Access to this feature is not available.'
            ) {
                $this->updatePermissions();
            } elseif (isset($simulations['object']) && $simulations['object'] == 'error') {
                return array(
                    'result' => false,
                    'error' => $simulations['message']
                );
            } else {
                if ($simulations) {
                    ksort($simulations);
                    $to_cache = array(
                        'result' => true,
                        'simulations' => $simulations
                    );

                    if (!$this->payplug_cache->setCache($cache_id, $to_cache)) {
                        $error_message = 'Error during setting Oney Simulation in DB cache [payplug.php]';
                        $error_level = 'error';
                        $this->logger->addLog($error_message, $error_level);
                    }
                }
            }

            return array(
                'result' => true,
                'simulations' => $simulations
            );
        } catch (Exception $exception) {
            die(var_dump($exception->__toString()));
            return array(
                'result' => false,
                'error' => $exception->__toString()
            );
        }
    }

    /**
     * Simule getter Oney Payment Simulations
     *
     * @param int $amount
     * @return bool
     */
    public function getOneySimulationsSandbox($amount)
    {
        if (!$this->getConfiguration('PAYPLUG_SANDBOX_MODE')) {
            return array(
                'result' => false,
                'error' => ''
            );
        }

        $options = array('x3_with_fees', 'x4_with_fees');

        $simulations = array();

        $rate = 19.6;

        foreach ($options as $option) {
            $type = explode('_', $option);
            $nb_inst = (int)str_replace('x', '', $type[0]);
            $installments = array();
            $part = (int)($amount / $nb_inst);
            $cost = $amount * ($rate / 1000);

            for ($i = 1; $i < $nb_inst; $i++) {
                $installments[] = array(
                    'amount' => $part,
                    'date' => date('Y-m-d')
                );
            }

            $simulations[$option] = array(
                'installments' => $installments,
                'total_cost' => $cost,
                'nominal_annual_percentage_rate' => 19.20,
                'effective_annual_percentage_rate' => $rate,
                'down_payment_amount' => $installments[0]['amount'] + $cost
            );
        }

        return array(
            'result' => true,
            'simulations' => $simulations
        );
    }

    /**
     * Generate a token hash based on the current version
     * Usefull to update templates, js and css without purging cache
     *
     * @return string
     */
    public function getVersionToken()
    {
        return hash('sha256', $this->version);
    }

    /**
     * Display Oney CTA on Shopping cart page
     *
     * @param array $params
     * @return bool|mixedf
     */
    public function hookDisplayBeforeShoppingCartBlock($params)
    {
        if (!$this->isOneyAllowed()) {
            return false;
        }

        $amount = $params['cart']->getOrderTotal(true, Cart::BOTH);
        $is_valid_amount = $this->isValidOneyAmount($amount, $params['cart']->id_currency);

        $this->smarty->assign(array(
            'payplug_oney_amount' => $amount,
            'payplug_oney_allowed' => $is_valid_amount['result'],
            'payplug_oney_error' => $is_valid_amount['error'],
        ));

        return $this->getOneyCTA('checkout');
    }

    /**
     * Display Oney CTA on Product page
     * @param array $params
     * @return bool|mixed
     */
    public function hookDisplayProductPriceBlock($params)
    {
        if (!$this->isOneyAllowed()) {
            return false;
        }

        // check if Oney is allowed and hook called only in the needed block
        if (!$this->getConfiguration('PAYPLUG_ONEY')
            || $params['type'] != 'price'
            || Dispatcher::getInstance()->getController() != 'product'
        ) {
            return false;
        }
        if (isset($params['from'])) {
            return false;
        }

        if (!Validate::isLoadedObject($params['product'])) {
            return false;
        }

        $amount = $params['product']->getPrice();
        $is_valid_amount = $this->isValidOneyAmount($amount, $params['cart']->id_currency);

        $this->smarty->assign(array(
            'payplug_oney_amount' => $amount,
            'payplug_oney_allowed' => $is_valid_amount['result'],
            'payplug_oney_error' => $is_valid_amount['error'],
        ));

        return $this->getOneyCTA('product');
    }

    /**
     * Get FAQ link for given iso lang
     * @param $iso_code
     * @return array
     */
    public function getFAQLinks($iso_code)
    {
        if ($iso_code == 'en') {
            $iso_code = 'en-gb';
        }

        return array(
            'activation' => 'https://support.payplug.com/hc/' . $iso_code . '/articles/360021328991',
            'deferred' => 'https://support.payplug.com/hc/' . $iso_code . '/articles/360010088420',
            'install' => 'https://support.payplug.com/hc/' . $iso_code . '/articles/360021389891',
            'installments' => 'https://support.payplug.com/hc/' . $iso_code . '/articles/360022447972',
            'one_click' => 'https://support.payplug.com/hc/' . $iso_code . '/articles/360022213892',
            'oney' => 'https://support.payplug.com/hc/' . $iso_code . '/articles/360009549540',
            'payment_page' => 'https://support.payplug.com/hc/' . $iso_code . '/articles/360021142312',
            'refund' => 'https://support.payplug.com/hc/' . $iso_code . '/articles/360022214692',
            'sandbox' => 'https://support.payplug.com/hc/' . $iso_code . '/articles/360021142492',
            'guide' => 'https://support.payplug.com/hc/' . $iso_code . '/articles/360011715080',
        );
    }

    /**
     * Install Oney feature
     */
    public function installOney()
    {
        $log = new MyLogPHP(_PS_MODULE_DIR_ . $this->name . '/log/install-log.csv');
        $log->info('Install Oney feature');
        return $this->installOneyHook()
            && $this->installOneyConfig()
            && $this->installOneyOrderStates();
    }

    /**
     * Install Oney Config
     * @return bool
     */
    private function installOneyConfig()
    {
        $log = new MyLogPHP(_PS_MODULE_DIR_ . 'payplug/log/install-log.csv');
        $flag = true;
        if (!$this->setConfiguration('PAYPLUG_ONEY', 0) ||
            !$this->setConfiguration('PAYPLUG_ONEY_ALLOWED_COUNTRIES', '') ||
            !$this->setConfiguration('PAYPLUG_ONEY_MAX_AMOUNTS', 'EUR:2000') ||
            !$this->setConfiguration('PAYPLUG_ONEY_MIN_AMOUNTS', 'EUR:150') ||
            !$this->setConfiguration('PAYPLUG_ONEY_TOS', 0)
        ) {
            $log->error('Installation failed: oney configurations failed.');
            $flag = false;
        }
        return $flag;
    }

    /**
     * Install Oney Hooks
     * @return bool
     */
    private function installOneyHook()
    {
        $hooks = array(
            'displayProductPriceBlock',
            'displayBeforeShoppingCartBlock',
        );

        $flag = true;
        foreach ($hooks as $hook) {
            $flag = $this->registerHook($hook) && $flag;
        }

        return $flag;
    }

    /**
     * Install Oney Order State
     */
    private function installOneyOrderStates()
    {
        $flag = true;

        foreach ($this->oney_order_state as $key => $state) {
            $flag = $flag && $this->createOrderState($key, $state, true) && $this->createOrderState(
                $key,
                $state,
                false
            );
        }

        return $flag;
    }

    /**
     * Check if Oney is allowed
     * @return bool
     */
    public function isOneyAllowed()
    {
        if (!$this->isAllowed()) {
            return false;
        }
        $context = Context::getContext();
        return $this->getConfiguration('PAYPLUG_ONEY')
            && $this->isOneyAllowedCurrency($context->currency);
    }

    /**
     * Check if Oney allow a given currency
     *
     * @param $id_currency
     * @return bool
     */
    public function isOneyAllowedCurrency($id_currency)
    {
        if (Validate::isLoadedObject($id_currency)) {
            $currency = $id_currency;
        } elseif (is_int($id_currency)) {
            $currency = new Currency($id_currency);
        } else {
            return false;
        }

        if (!Validate::isLoadedObject($currency)) {
            return false;
        }

        // we use the Oney limit to get allowed currencies
        $oney_min_amounts = Tools::strtoupper($this->getConfiguration('PAYPLUG_ONEY_MIN_AMOUNTS'));
        $iso_code = Tools::strtoupper($currency->iso_code);

        return strpos($oney_min_amounts, $iso_code) !== false;
    }

    /**
     * Check if a valid Cart for Oney
     *
     * @param $cart Cart
     * @param int $amount
     * @return array
     */
    public function isOneyElligible($cart, $amount = false)
    {
        // check if cart is valid
        $is_valid_cart = $this->isValidOneyCart($cart);
        if (!$is_valid_cart['result']) {
            return array(
                'result' => false,
                'error_type' => 'invalid_cart',
                'error' => $is_valid_cart['error']
            );
        }

        // check if cart address is valid
        $is_valid_addresses = $this->isValidOneyAddresses($cart->id_address_delivery, $cart->id_address_invoice);
        if (!$is_valid_addresses['result']) {
            return array(
                'result' => false,
                'error_type' => 'invalid_addresses',
                'error' => $is_valid_addresses['error']
            );
        }

        // check if current amount is between min and max values
        $amount = $amount ? $amount : $cart->getOrderTotal(true, Cart::BOTH);
        $is_valid_amount = $this->isValidOneyAmount($amount, $cart->id_currency);
        if (!$is_valid_amount['result']) {
            $limits = $this->getOneyPriceLimit($cart->id_currency);
            $converted_amount = $this->convertAmount($amount);
            $error_type = $converted_amount > $limits['min'] ? 'invalid_amount_top' : 'invalid_amount_bottom';

            return array('result' => false, 'error_type' => $error_type, 'error' => $is_valid_amount['error']);
        }

        return array('result' => true, 'error' => false);
    }

    /**
     * Check if given phone number is valid mobile phone number
     * @param string $phone_number
     * @param string $iso_code
     * @return bool
     */
    public function isValidMobilePhoneNumber($phone_number, $iso_code)
    {
        try {
            //load libphonenumber
            if (!class_exists('libphonenumber\PhoneNumberUtil')) {
                include_once(_PS_MODULE_DIR_ . 'payplug/lib/libphonenumber/init.php');
            }

            // then format code
            $phone_util = libphonenumber\PhoneNumberUtil::getInstance();
            $parsed = $phone_util->parse($phone_number, $iso_code);
            $is_mobile = $phone_util->getNumberType($parsed);
            return (bool)(in_array($is_mobile, array(1, 2)));
        } catch (Exception $e) {
            // todo: add log
            return false;
        }
    }

    /**
     * Check if billing and shipping addresses are valid
     *
     * @param int $id_shipping
     * @param int $id_billing
     * @return array
     */
    public function isValidOneyAddresses($id_shipping, $id_billing)
    {
        $shipping = new Address($id_shipping);
        $shipping_country = new Country($shipping->id_country);

        $billing = new Address($id_billing);
        $billing_country = new Country($billing->id_country);

        return $this->isValidOneyCountry($shipping_country->iso_code, $billing_country->iso_code);
    }

    /**
     * Check if amount is valid for Oney
     *
     * @param float $amount
     * @param int $id_currency
     * @return array
     */
    public function isValidOneyAmount($amount, $id_currency = false)
    {
        $limits = $this->getOneyPriceLimit($id_currency);
        $convert_amount = $this->convertAmount($amount);
        if (($limits['min'] > $convert_amount) || ($convert_amount > $limits['max'])) {
            $min_amount = $this->convertAmount($limits['min'], true);
            $max_amount = $this->convertAmount($limits['max'], true);

            return array(
                'result' => false,
                'error' => sprintf(
                    $this->l('The total amount of your order should be between %s and %s to pay with Oney.'),
                    Tools::displayPrice($min_amount),
                    Tools::displayPrice($max_amount)
                )
            );
        }

        return array('result' => true, 'error' => false);
    }

    /**
     * Check if cart is valid for Oney
     *
     * @param Cart $cart
     * @return array
     */
    public function isValidOneyCart($cart)
    {
        if (!Validate::isLoadedObject($cart)) {
            return array(
                'result' => false,
                'error' => $this->l('The cart is unvalid')
            );
        }

        $nb_products = $this->context->cart->nbProducts();

        // todo: set as a constant
        $max = 1000;

        if ($nb_products >= $max) {
            $error = 'The payment with Oney is not available because you have more than 1000 items in your cart.';
            return array(
                'result' => false,
                'error' => $this->l($error)
            );
        }


        return array('result' => true, 'error' => false);
    }

    /**
     * Check if billing and shipping addresses are valid
     *
     * @param string $shipping_iso
     * @param string $billing_iso
     * @return array
     */
    public function isValidOneyCountry($shipping_iso, $billing_iso)
    {
        // check if the billing country and the shipping country are different then return false
        if ($shipping_iso != $billing_iso) {
            $error = 'Delivery and billing addresses must be in the same country to pay with Oney.';
            return array(
                'result' => false,
                'type' => 'different',
                'error' => $this->l($error)
            );
        }

        // check if the shipping country are different then return false
        $iso_code = Tools::strtoupper($shipping_iso);
        $allow_countries = Tools::strtoupper($this->getConfiguration('PAYPLUG_ONEY_ALLOWED_COUNTRIES'));
        if (!$allow_countries) {
            return array(
                'result' => false,
                'type' => 'no_country',
                'error' => $this->l('No countries are configured to use oney.')
            );
        }

        $iso_list = explode(',', $allow_countries);
        if (!in_array($iso_code, $iso_list)) {
            $list = array();
            foreach ($iso_list as $iso) {
                $id_country = Country::getByIso($iso);
                $list[] = Country::getNameById($this->context->language->id, $id_country);
            }
            return array(
                'result' => false,
                'type' => 'invalid',
                'error' => sprintf(
                    $this->l('For a payment with Oney, delivery and billing addresses must be in %s'),
                    implode(', ', $list)
                )
            );
        }

        return array('result' => true, 'error' => false);
    }

    /**
     * Set payment data in cookie
     *
     * @return mixed
     */
    public function setPaymentDataCookie($payplug_data = array())
    {
        if (empty($payplug_data)) {
            return false;
        }

        $value = Tools::jsonEncode($payplug_data);

        $this->context->cookie->__set('payplug_data', $value);
        return (bool)$this->context->cookie->__get('payplug_data');
    }

    /**
     * Set payment errors in cookie
     *
     * @return mixed
     */
    public function setPaymentErrorsCookie($payplug_errors = array())
    {
        if (empty($payplug_errors)) {
            return false;
        }

        $value = Tools::jsonEncode($payplug_errors);

        $this->context->cookie->__set('payplug_errors', $value);
        return (bool)$this->context->cookie->__get('payplug_errors');
    }

    /**
     * Update config permission from account
     */
    public function updatePermissions()
    {
        $permissions = $this->getAccount($this->current_api_key);
        $this->setConfiguration('PAYPLUG_ONE_CLICK', (bool)$permissions['can_save_cards']);
        $this->setConfiguration('PAYPLUG_INST', (int)$permissions['can_create_installment_plan']);
        $this->setConfiguration('PAYPLUG_DEFERRED', (bool)$permissions['can_create_deferred_payment']);
        $this->setConfiguration('PAYPLUG_ONEY', (bool)$permissions['can_use_oney']);
    }

    public function setNotification()
    {
        return new PayPlugNotifications();
    }

    public function setValidation()
    {
        return new PayPlugValidation();
    }

    /**
     * Check if current device used is mobile
     *
     * @return bool
     */
    public function isMobiledevice()
    {
        $useragent = $_SERVER['HTTP_USER_AGENT'];

        if (preg_match('/(android|bb\d+|meego).+mobile|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\.(browser|link)|vodafone|wap|windows (ce|phone)|xda|xiino/i', $useragent)
            || preg_match('/1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s\-)|ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|\-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|bw\-(n|u)|c55\/|capi|ccwa|cdm\-|cell|chtm|cldc|cmd\-|co(mp|nd)|craw|da(it|ll|ng)|dbte|dc\-s|devi|dica|dmob|do(c|p)o|ds(12|\-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(\-|_)|g1 u|g560|gene|gf\-5|g\-mo|go(\.w|od)|gr(ad|un)|haie|hcit|hd\-(m|p|t)|hei\-|hi(pt|ta)|hp( i|ip)|hs\-c|ht(c(\-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i\-(20|go|ma)|i230|iac( |\-|\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\/)|klon|kpt |kwc\-|kyo(c|k)|le(no|xi)|lg( g|\/(k|l|u)|50|54|\-[a-w])|libw|lynx|m1\-w|m3ga|m50\/|ma(te|ui|xo)|mc(01|21|ca)|m\-cr|me(rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi|de|do|t(\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)\-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg|pg(13|\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\-2|po(ck|rt|se)|prox|psio|pt\-g|qa\-a|qc(07|12|21|32|60|\-[2-7]|i\-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h\-|oo|p\-)|sdk\/|se(c(\-|0|1)|47|mc|nd|ri)|sgh\-|shar|sie(\-|m)|sk\-0|sl(45|id)|sm(al|ar|b3|it|t5)|so(ft|ny)|sp(01|h\-|v\-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl\-|tdg\-|tel(i|m)|tim\-|t\-mo|to(pl|sh)|ts(70|m\-|m3|m5)|tx\-9|up(\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)|vk(40|5[0-3]|\-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(\-| )|webc|whit|wi(g |nc|nw)|wmlb|wonu|x700|yas\-|your|zeto|zte\-/i', Tools::substr($useragent, 0, 4))
        ) {
            return true;
        }

        return false;
    }

    public function hookActionAdminPerformanceControllerAfter($params)
    {
        if (!Tools::getValue('empty_smarty_cache')) {
            return false;
        }

        // Purge PayPlug cache
        if (!$this->payplug_cache->flushCache()) {
            $error_message = 'Error during flushing PayPLug DB cache [payplug.php]';
            $error_level = 'error';
            $this->payplug_cache->logger->addLog($error_message, $error_level);
        }
    }

    public function isAllowed()
    {
        if (!Module::isEnabled($this->name) || !$this->getConfiguration('PAYPLUG_SHOW')) {
            return false;
        }
        return true;
    }

    /**
     * @description Set the context
     * @param null $context
     * @return bool
     */
    public function setContext($context = null)
    {
        if (!$context || !is_object($context)) {
            return false;
        }
        $this->context = $context;
    }
}
